@@ -93,6 +93,8 @@ struct rt_fifo_get_info_struct{
 
 #ifdef __KERNEL__
 
+#include <rtai.h>
+
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
@@ -103,6 +105,8 @@ void __rtai_fifos_exit(void);
 
 int rtf_init(void);
 
+typedef int (*rtf_handler_t)(unsigned int fifo, int rw);
+
 /* Attach a handler to an RT-FIFO.
  *
  * Allow function handler to be called when a user process reads or writes to 
@@ -111,7 +115,7 @@ int rtf_init(void);
  */
 
 int rtf_create_handler(unsigned int fifo,	/* RT-FIFO */
-		       int (*handler)(unsigned int fifo)	/* function to be called */);
+		       void *handler		/* function to be called */);
 
 
 /**
@@ -126,7 +130,7 @@ int rtf_create_handler(unsigned int fifo,	/* RT-FIFO */
  * to allow the user to easily understand if the handler was called at fifo
  * read (@a rw is 'r') or write (rw is 'w').
  */
-#define X_FIFO_HANDLER(handler) ((int (*)(unsigned int))(handler))
+#define X_FIFO_HANDLER(handler) ((int (*)(unsigned int, int rw))(handler))
 
 /* Create an RT-FIFO.
  * 
@@ -134,7 +138,7 @@ int rtf_create_handler(unsigned int fifo,	/* RT-FIFO */
  * Return value: On success, 0 is returned. On error, -1 is returned.
  */
 #undef rtf_create
-int rtf_create(unsigned int fifo, int size);
+RTAI_SYSCALL_MODE int rtf_create(unsigned int fifo, int size);
 
 /* Create an RT-FIFO with a name and size.
  *
@@ -154,7 +158,7 @@ int rtf_named_create(const char *name, int size);
  *               On error, -errno is returned.
  */
 
-int rtf_create_named(const char *name);
+RTAI_SYSCALL_MODE int rtf_create_named(const char *name);
 
 /* Look up a named RT-FIFO.
  *
@@ -163,7 +167,7 @@ int rtf_create_named(const char *name);
  *               On error, -errno is returned.
  */
 
-int rtf_getfifobyname(const char *name);
+RTAI_SYSCALL_MODE int rtf_getfifobyname(const char *name);
 
 /* Reset an RT-FIFO.
  * 
@@ -172,14 +176,14 @@ int rtf_getfifobyname(const char *name);
  * creation.
  */
 
-int rtf_reset(unsigned int fifo);
+RTAI_SYSCALL_MODE int rtf_reset(unsigned int fifo);
 
 /* destroy an RT-FIFO.
  * 
  * Return value: On success, 0 is returned.
  */
 
-int rtf_destroy(unsigned int fifo);
+RTAI_SYSCALL_MODE int rtf_destroy(unsigned int fifo);
 
 
 /* Resize an RT-FIFO.
@@ -188,7 +192,7 @@ int rtf_destroy(unsigned int fifo);
  * is returned.
  */
 
-int rtf_resize(unsigned int minor, int size);
+RTAI_SYSCALL_MODE int rtf_resize(unsigned int minor, int size);
 
 
 /* Write to an RT-FIFO.
@@ -196,7 +200,7 @@ int rtf_resize(unsigned int minor, int size);
  * Try to write count bytes to an FIFO. Returns the number of bytes written.
  */
 
-int rtf_put(unsigned int fifo,	/* RT-FIFO */
+RTAI_SYSCALL_MODE int rtf_put(unsigned int fifo,	/* RT-FIFO */
 	    void * buf,		/* buffer address */
 	    int count		/* number of bytes to write */);
 
@@ -207,7 +211,7 @@ int rtf_put(unsigned int fifo,	/* RT-FIFO */
  * Try to write count bytes to an FIFO. Returns 0.
  */
 
-int rtf_ovrwr_put(unsigned int fifo,	/* RT-FIFO */
+RTAI_SYSCALL_MODE int rtf_ovrwr_put(unsigned int fifo,	/* RT-FIFO */
 		  void * buf,		/* buffer address */
 		  int count		/* number of bytes to write */);
 
@@ -219,7 +223,7 @@ int rtf_ovrwr_put(unsigned int fifo,	/* RT-FIFO */
  * written.
  */
 
-extern int rtf_put_if (unsigned int fifo,	/* RT-FIFO */
+extern RTAI_SYSCALL_MODE int rtf_put_if (unsigned int fifo,	/* RT-FIFO */
 		void * buf,			/* buffer address */
 	       	int count			/* number of bytes to write */);
 
@@ -228,7 +232,7 @@ extern int rtf_put_if (unsigned int fifo,	/* RT-FIFO */
  * Try to read count bytes from a FIFO. Returns the number of bytes read.
  */
 
-int rtf_get(unsigned int fifo,	/* RT-FIFO */
+RTAI_SYSCALL_MODE int rtf_get(unsigned int fifo,	/* RT-FIFO */
 	    void * buf, 		/* buffer address */
 	    int count		/* number of bytes to read */);
 
@@ -238,7 +242,7 @@ int rtf_get(unsigned int fifo,	/* RT-FIFO */
  * Try to read count bytes in a block from an FIFO. Returns the number of bytes read.
  */
 
-int rtf_get_if(unsigned int fifo,	/* RT-FIFO */
+RTAI_SYSCALL_MODE int rtf_get_if(unsigned int fifo,	/* RT-FIFO */
 	    void * buf, 		/* buffer address */
 	    int count		/* number of bytes to read */);
 
@@ -255,7 +259,7 @@ int rtf_evdrp(unsigned int fifo,	/* RT-FIFO */
  *
  */
 
-int rtf_sem_init(unsigned int fifo,	/* RT-FIFO */
+RTAI_SYSCALL_MODE int rtf_sem_init(unsigned int fifo,	/* RT-FIFO */
 		 int value			/* initial semaphore value */);
 
 
@@ -263,21 +267,21 @@ int rtf_sem_init(unsigned int fifo,	/* RT-FIFO */
  *
  */
 
-int rtf_sem_post(unsigned int fifo	/* RT-FIFO */);
+RTAI_SYSCALL_MODE int rtf_sem_post(unsigned int fifo	/* RT-FIFO */);
 
 
 /* Try to acquire an RT-FIFO semaphore.
  *
  */
 
-int rtf_sem_trywait(unsigned int fifo	/* RT-FIFO */);
+RTAI_SYSCALL_MODE int rtf_sem_trywait(unsigned int fifo	/* RT-FIFO */);
 
 
 /* Destroy an RT-FIFO semaphore.
  *
  */
 
-int rtf_sem_destroy(unsigned int fifo	/* RT-FIFO */);
+RTAI_SYSCALL_MODE int rtf_sem_destroy(unsigned int fifo	/* RT-FIFO */);
 
 #define rtf_sem_delete rtf_sem_destroy
 
@@ -309,20 +313,20 @@ extern "C" {
 
 RTAI_PROTO(int, rtf_create,(unsigned int fifo, int size))
 {
-	struct { unsigned int fifo, size; } arg = { fifo, size }; 
+	struct { unsigned long fifo, size; } arg = { fifo, size }; 
 	return rtai_lxrt(FUN_FIFOS_LXRT_INDX, SIZARG, _CREATE, &arg).i[LOW];
 }
 
 RTAI_PROTO(int, rtf_destroy,(unsigned int fifo))
 {
-	struct { unsigned int fifo; } arg = { fifo };
+	struct { unsigned long fifo; } arg = { fifo };
 	return rtai_lxrt(FUN_FIFOS_LXRT_INDX, SIZARG, _DESTROY, &arg).i[LOW];
 }
 
 RTAI_PROTO(int, rtf_put,(unsigned int fifo, const void *buf, int count))
 {
 	char lbuf[count];
-	struct { unsigned int fifo; void *buf; int count; } arg = { fifo, lbuf, count };
+	struct { unsigned long fifo; void *buf; long count; } arg = { fifo, lbuf, count };
 	memcpy(lbuf, buf, count);
 	return rtai_lxrt(FUN_FIFOS_LXRT_INDX, SIZARG, _PUT, &arg).i[LOW];
 }
@@ -330,7 +334,7 @@ RTAI_PROTO(int, rtf_put,(unsigned int fifo, const void *buf, int count))
 RTAI_PROTO(int, rtf_put_if,(unsigned int fifo, const void *buf, int count))
 {
 	char lbuf[count];
-	struct { unsigned int fifo; void *buf; int count; } arg = { fifo, lbuf, count };
+	struct { unsigned long fifo; void *buf; long count; } arg = { fifo, lbuf, count };
 	memcpy(lbuf, buf, count);
 	return rtai_lxrt(FUN_FIFOS_LXRT_INDX, SIZARG, _PUT_IF, &arg).i[LOW];
 }
@@ -339,7 +343,7 @@ RTAI_PROTO(int, rtf_get,(unsigned int fifo, void *buf, int count))
 {
 	int retval;
 	char lbuf[count];
-	struct { unsigned int fifo; void *buf; int count; } arg = { fifo, lbuf, count };
+	struct { unsigned long fifo; void *buf; long count; } arg = { fifo, lbuf, count };
 	retval = rtai_lxrt(FUN_FIFOS_LXRT_INDX, SIZARG, _GET, &arg).i[LOW];
 	if (retval > 0) {
 		memcpy(buf, lbuf, retval);
@@ -351,7 +355,7 @@ RTAI_PROTO(int, rtf_get_if,(unsigned int fifo, void *buf, int count))
 {
 	int retval;
 	char lbuf[count];
-	struct { unsigned int fifo; void *buf; int count; } arg = { fifo, lbuf, count };
+	struct { unsigned long fifo; void *buf; long count; } arg = { fifo, lbuf, count };
 	retval = rtai_lxrt(FUN_FIFOS_LXRT_INDX, SIZARG, _GET_IF, &arg).i[LOW];
 	if (retval > 0) {
 		memcpy(buf, lbuf, retval);
@@ -361,37 +365,37 @@ RTAI_PROTO(int, rtf_get_if,(unsigned int fifo, void *buf, int count))
 
 RTAI_PROTO(int, rtf_reset_lxrt,(unsigned int fifo))
 {
-	struct { unsigned int fifo; } arg = { fifo };
+	struct { unsigned long fifo; } arg = { fifo };
 	return rtai_lxrt(FUN_FIFOS_LXRT_INDX, SIZARG, _RESET, &arg).i[LOW];
 }
 
 RTAI_PROTO(int, rtf_resize_lxrt,(unsigned int fifo, int size))
 {
-	struct { unsigned int fifo, size; } arg = { fifo, size }; 
+	struct { unsigned long fifo, size; } arg = { fifo, size }; 
 	return rtai_lxrt(FUN_FIFOS_LXRT_INDX, SIZARG, _RESIZE, &arg).i[LOW];
 }
 
 RTAI_PROTO(int, rtf_sem_init_lxrt,(unsigned int fifo, int value))
 {
-	struct { unsigned int fifo, value; } arg = { fifo, value }; 
+	struct { unsigned long fifo, value; } arg = { fifo, value }; 
 	return rtai_lxrt(FUN_FIFOS_LXRT_INDX, SIZARG, _SEM_INIT, &arg).i[LOW];
 }
 
 RTAI_PROTO(int, rtf_sem_post_lxrt,(unsigned int fifo))
 {
-	struct { unsigned int fifo; } arg = { fifo }; 
+	struct { unsigned long fifo; } arg = { fifo }; 
 	return rtai_lxrt(FUN_FIFOS_LXRT_INDX, SIZARG, _SEM_POST, &arg).i[LOW];
 }
 
 RTAI_PROTO(int, rtf_sem_trywait_lxrt,(unsigned int fifo))
 {
-	struct { unsigned int fifo; } arg = { fifo }; 
+	struct { unsigned long fifo; } arg = { fifo }; 
 	return rtai_lxrt(FUN_FIFOS_LXRT_INDX, SIZARG, _SEM_TRY, &arg).i[LOW];
 }
 
 RTAI_PROTO(int, rtf_sem_destroy_lxrt,(unsigned int fifo))
 {
-	struct { unsigned int fifo; } arg = { fifo }; 
+	struct { unsigned long fifo; } arg = { fifo }; 
 	return rtai_lxrt(FUN_FIFOS_LXRT_INDX, SIZARG, _SEM_DESTRY, &arg).i[LOW];
 }
 
@@ -399,7 +403,7 @@ RTAI_PROTO(int, rtf_named_create_lxrt,(const char *name, int size))
 {
 	int len;
 	char lname[len = strlen(name)];
-	struct { char * name; int size; } arg = { lname, size };
+	struct { char * name; long size; } arg = { lname, size };
 	strncpy(lname, name, len);
 	return rtai_lxrt(FUN_FIFOS_LXRT_INDX, SIZARG, _NAMED_CREATE, &arg).i[LOW];
 }
@@ -425,7 +429,7 @@ RTAI_PROTO(int, rtf_getfifobyname_lxrt,(const char *name))
 RTAI_PROTO(int, rtf_ovrwr_put,(unsigned int fifo, const void *buf, int count))
 {
 	char lbuf[count];
-	struct { unsigned int fifo; void *buf; int count; } arg = { fifo, lbuf, count };
+	struct { unsigned long fifo; void *buf; long count; } arg = { fifo, lbuf, count };
 	memcpy(lbuf, buf, count);
 	return rtai_lxrt(FUN_FIFOS_LXRT_INDX, SIZARG, _OVERWRITE, &arg).i[LOW];
 }
@@ -512,7 +516,7 @@ RTAI_PROTO(int, rtf_open_sized,(const char *dev, int perm, int size))
 
 RTAI_PROTO(int, rtf_evdrp,(int fd, void *buf, int count))
 {
-	struct { void *buf; int count; } args = { buf, count };
+	struct { void *buf; long count; } args = { buf, count };
 	int ret = ioctl(fd, EAVESDROP, &args);
 	return ret < 0 ? -errno : ret;
 }
@@ -535,7 +539,7 @@ RTAI_PROTO(int, rtf_evdrp,(int fd, void *buf, int count))
  */
 RTAI_PROTO(int, rtf_read_all_at_once,(int fd, void *buf, int count))
 {
-	struct { void *buf; int count; } args = { buf, count };
+	struct { void *buf; long count; } args = { buf, count };
 	int ret = ioctl(fd, READ_ALL_AT_ONCE, &args);
 	return ret < 0 ? -errno : ret;
 }
@@ -563,14 +567,14 @@ RTAI_PROTO(int, rtf_read_all_at_once,(int fd, void *buf, int count))
  */
 RTAI_PROTO(int, rtf_read_timed,(int fd, void *buf, int count, int ms_delay))
 {
-	struct { void *buf; int count, delay; } args = { buf, count, ms_delay };
+	struct { void *buf; long count, delay; } args = { buf, count, ms_delay };
 	int ret = ioctl(fd, READ_TIMED, &args);
 	return ret < 0 ? -errno : ret;
 }
 
 RTAI_PROTO(int, rtf_read_if,(int fd, void *buf, int count))
 {
-	struct { void *buf; int count; } args = { buf, count };
+	struct { void *buf; long count; } args = { buf, count };
 	int ret = ioctl(fd, READ_IF, &args);
 	return ret < 0 ? -errno : ret;
 }
@@ -598,21 +602,21 @@ RTAI_PROTO(int, rtf_read_if,(int fd, void *buf, int count))
  */
 RTAI_PROTO(int, rtf_write_timed,(int fd, void *buf, int count, int ms_delay))
 {
-	struct { void *buf; int count, delay; } args = { buf, count, ms_delay };
+	struct { void *buf; long count, delay; } args = { buf, count, ms_delay };
 	int ret = ioctl(fd, WRITE_TIMED, &args);
 	return ret < 0 ? -errno : ret;
 }
 
 RTAI_PROTO(int, rtf_overwrite,(int fd, void *buf, int count))
 {
-	struct { void *buf; int count; } args = { buf, count };
+	struct { void *buf; long count; } args = { buf, count };
 	int ret = ioctl(fd, OVRWRITE, &args);
 	return ret < 0 ? -errno : ret;
 }
 
 RTAI_PROTO(int, rtf_write_if,(int fd, void *buf, int count))
 {
-	struct { void *buf; int count; } args = { buf, count };
+	struct { void *buf; long count; } args = { buf, count };
 	int ret = ioctl(fd, WRITE_IF, &args);
 	return ret < 0 ? -errno : ret;
 }
