@@ -87,15 +87,15 @@
 #define SET_ONESHOT_MODE		14
 #define SIGNAL_HANDLER	 		15
 #define TASK_USE_FPU			16
-#define LINUX_USE_FPU			17
-#define PREEMPT_ALWAYS_GEN		18
+#define GET_PRIORITIES			17  // was LINUX_USE_FPU
+#define HARD_TIMER_COUNT		18
 #define GET_TIME_NS			19
 #define GET_CPU_TIME_NS			20
 #define SET_RUNNABLE_ON_CPUS		21 
 #define SET_RUNNABLE_ON_CPUID		22	 
 #define GET_TIMER_CPU			23	 
 #define START_RT_APIC_TIMERS		24
-#define PREEMPT_ALWAYS_CPUID		25
+#define HARD_TIMER_COUNT_CPUID		25
 #define COUNT2NANO_CPUID		26
 #define NANO2COUNT_CPUID		27
 #define GET_TIME_CPUID			28
@@ -312,10 +312,26 @@
 #define SCHED_LOCK		       209
 #define SCHED_UNLOCK		       210
 #define PEND_LINUX_IRQ		       211
-#define RECEIVE_LINUX_SYSCALL          212
-#define RETURN_LINUX_SYSCALL           213
+#define SET_LINUX_SYSCALL_MODE	       212
+/*#define RETURN_LINUX_SYSCALL         213 available */
+#define REQUEST_RTC                    214
+#define RELEASE_RTC                    215
+#define RT_GETTID                      216
+#define SET_NETRPC_TIMEOUT             217
+#define GET_REAL_TIME		       218
+#define GET_REAL_TIME_NS	       219
 
-#define MAX_LXRT_FUN                   215
+#define MQ_REG_USP_NOTIFIER	       220
+
+#define RT_SIGNAL_HELPER   	       221
+#define RT_SIGNAL_WAITSIG  	       222
+#define RT_SIGNAL_REQUEST  	       223
+#define RT_SIGNAL_RELEASE  	       224
+#define RT_SIGNAL_ENABLE	       225
+#define RT_SIGNAL_DISABLE	       226
+#define RT_SIGNAL_TRIGGER	       227
+
+#define MAX_LXRT_FUN		       230
 
 // not recovered yet 
 // Qblk's 
@@ -397,12 +413,20 @@
 
 // Keep LXRT call enc/decoding together, so you are sure to act consistently.
 // This is the encoding, note " | GT_NR_SYSCALLS" to ensure not a Linux syscall, ...
-#define GT_NR_SYSCALLS  (1 << 15)
-#define ENCODE_LXRT_REQ(dynx, srq, lsize)  (((dynx) << 28) | (((srq) & 0xFFF) << 16) | GT_NR_SYSCALLS | (lsize))
+#define GT_NR_SYSCALLS  (1 << 11)
+#define ENCODE_LXRT_REQ(dynx, srq, lsize)  (((dynx) << 24) | ((srq) << 12) | GT_NR_SYSCALLS | (lsize))
 // ... and this is the decoding.
-#define SRQ(x)   (((x) >> 16) & 0xFFF)
+#define SRQ(x)   (((x) >> 12) & 0xFFF)
 #define NARG(x)  ((x) & (GT_NR_SYSCALLS - 1))
-#define INDX(x)  (((x) >> 28) & 0xF)
+#define INDX(x)  (((x) >> 24) & 0xF)
+
+#define SYNC_LINUX_SYSCALL   1
+#define ASYNC_LINUX_SYSCALL  0
+
+#include <asm/ptrace.h>
+
+struct mode_regs { struct pt_regs regs; long mode; };
+struct linux_syscalls_list { long in, nr, mode; void *serv; struct mode_regs *moderegs; RT_TASK *task; void (*callback_fun)(long, long); long out, retval; };
 
 #ifdef __KERNEL__
 
@@ -417,32 +441,23 @@ arg  INDX   |||| .... .... .... .... .... .... ....
 */
 
 /*
-These USP (unsigned long long) type fields allow to read and write up to 2 arguments.  
+These USP (unsigned long) type fields allow to read and write up to 2 arguments.  
                                                
-RW marker .... .... .... .... .... .... .... ..|| .... .... .... .... .... .... .... ...|
+The high part of the unsigned long encodes writes
+W ARG1 BF .... .... ..|| |... .... .... .... ....
+W ARG1 SZ .... ...| ||.. .... .... .... .... ....
+W ARG2 BF .... |||. .... .... .... .... .... ....
+W ARG2 SZ .||| .... .... .... .... .... .... ....
 
-HIGH unsigned long encodes writes
-W ARG1 BF .... .... .... .... .... ...| |||| ||..
-W ARG1 SZ .... .... .... .... |||| |||. .... ....
-W ARG2 BF .... .... .||| |||| .... .... .... ....
-W ARG2 SZ ..|| |||| |... .... .... .... .... ....
-W 1st  LL .|.. .... .... .... .... .... .... ....
-W 2nd  LL |... .... .... .... .... .... .... ....
+The low part of the unsigned long encodes writes
+R ARG1 BF .... .... .... .... .... .... ..|| |...
+R ARG1 SZ .... .... .... .... .... ...| ||.. ....
+R ARG2 BF .... .... .... .... .... |||. .... ....
+R ARG2 SZ .... .... .... .... .||| .... .... ....
 
-LOW unsigned long encodes reads
-R ARG1 BF .... .... .... .... .... ...| |||| ||..
-R ARG1 SZ .... .... .... .... |||| |||. .... ....
-R ARG2 BF .... .... .||| |||| .... .... .... ....
-R ARG2 SZ ..|| |||| |... .... .... .... .... ....
-R 1st  LL .|.. .... .... .... .... .... .... ....
-R 2nd  LL |... .... .... .... .... .... .... ....
-
-LOW unsigned long encodes also
+The low part of the unsigned long encodes also
 RT Switch .... .... .... .... .... .... .... ...|
 
-and 
-Always 0  .... .... .... .... .... .... .... ..|.
-
 If SZ is zero sizeof(int) is copied by default, if LL bit is set sizeof(long long) is copied.
 */
 
@@ -450,42 +465,33 @@ If SZ is zero sizeof(int) is copied by default, if LL bit is set sizeof(long lon
 // them in fun entry type to obtain the desired encoding
 
 // for writes
-#define UW1(bf, sz)  ((((unsigned long long)((((bf) & 0x7F) <<  2) | (((sz) & 0x7F) <<  9))) << 32) | 0x300000001LL)
-#define UW2(bf, sz)  ((((unsigned long long)((((bf) & 0x7F) << 16) | (((sz) & 0x7F) << 23))) << 32) | 0x300000001LL)
-#define UWSZ1LL      (0x4000000300000001LL)
-#define UWSZ2LL      (0x8000000300000001LL)
+#define UW1(bf, sz)  ((((bf) & 0x7) << 19) | (((sz) & 0x7) << 22))
+#define UW2(bf, sz)  ((((bf) & 0x7) << 25) | (((sz) & 0x7) << 28))
 
 // for reads
-#define UR1(bf, sz)  ((((bf) & 0x7F) <<  2) | (((sz) & 0x7F) <<  9) | 0x300000001LL)
-#define UR2(bf, sz)  ((((bf) & 0x7F) << 16) | (((sz) & 0x7F) << 23) | 0x300000001LL)
-#define URSZ1LL      (0x340000001LL)
-#define URSZ2LL      (0x380000001LL)
+#define UR1(bf, sz)  ((((bf) & 0x7) << 3) | (((sz) & 0x7) <<  6))
+#define UR2(bf, sz)  ((((bf) & 0x7) << 9) | (((sz) & 0x7) << 12))
 
-// and these are for deciding what to do in lxrt.c
-#define	NEED_TO_RW(x)	(((unsigned long *)&(x))[HIGH])
+#define	NEED_TO_RW(x)	((x) & 0xFFFFFFFE)
 
-#define NEED_TO_R(x)	(((unsigned long *)&(x))[LOW]  & 0x0000FFFC)
-#define NEED_TO_W(x)	(((unsigned long *)&(x))[HIGH] & 0x0000FFFC)
+#define NEED_TO_W(x)	((x) & (0x3F << 19))
+#define NEED_TO_W2ND(x)	((x) & (0x3F << 25))
 
-#define NEED_TO_R2ND(x)	(((unsigned long *)&(x))[LOW]  & 0x3FFF0000)
-#define NEED_TO_W2ND(x)	(((unsigned long *)&(x))[HIGH] & 0x3FFF0000)
+#define NEED_TO_R(x)	((x) & (0x3F <<  3))
+#define NEED_TO_R2ND(x)	((x) & (0x3F <<  9))
 
-#define USP_RBF1(x)  	((((unsigned long *)&(x))[LOW] >>  2) & 0x7F)
-#define USP_RSZ1(x)    	((((unsigned long *)&(x))[LOW] >>  9) & 0x7F)
-#define USP_RBF2(x)    	((((unsigned long *)&(x))[LOW] >> 16) & 0x7F)
-#define USP_RSZ2(x)    	((((unsigned long *)&(x))[LOW] >> 23) & 0x7F)
-#define USP_RSZ1LL(x)  	(((unsigned long *)&(x))[LOW] & 0x40000000)
-#define USP_RSZ2LL(x)  	(((unsigned long *)&(x))[LOW] & 0x80000000)
+#define USP_WBF1(x)   	(((x) >> 19) & 0x7)
+#define USP_WSZ1(x)    	(((x) >> 22) & 0x7)
+#define USP_WBF2(x)    	(((x) >> 25) & 0x7)
+#define USP_WSZ2(x)    	(((x) >> 28) & 0x7)
 
-#define USP_WBF1(x)   	((((unsigned long *)&(x))[HIGH] >>  2) & 0x7F)
-#define USP_WSZ1(x)    	((((unsigned long *)&(x))[HIGH] >>  9) & 0x7F)
-#define USP_WBF2(x)    	((((unsigned long *)&(x))[HIGH] >> 16) & 0x7F)
-#define USP_WSZ2(x)    	((((unsigned long *)&(x))[HIGH] >> 23) & 0x7F)
-#define USP_WSZ1LL(x)   (((unsigned long *)&(x))[HIGH] & 0x40000000)
-#define USP_WSZ2LL(x)   (((unsigned long *)&(x))[HIGH] & 0x80000000)
+#define USP_RBF1(x)  	(((x) >>  3) & 0x7)
+#define USP_RSZ1(x)    	(((x) >>  6) & 0x7)
+#define USP_RBF2(x)    	(((x) >>  9) & 0x7)
+#define USP_RSZ2(x)    	(((x) >> 12) & 0x7)
 
 struct rt_fun_entry {
-    unsigned long long type;
+    unsigned long type;
     void *fun;
 };
 
@@ -506,14 +512,14 @@ extern "C" {
 
 #if CONFIG_RTAI_INTERNAL_LXRT_SUPPORT
  
-static inline struct rt_task_struct *pid2rttask(pid_t pid)
+static inline struct rt_task_struct *pid2rttask(long pid)
 {
-        return ((unsigned long)pid) > PID_MAX_LIMIT ? (struct rt_task_struct *)pid : find_task_by_pid(pid)->rtai_tskext(0);
+        return ((unsigned long)pid) > PID_MAX_LIMIT ? (struct rt_task_struct *)pid : find_task_by_pid(pid)->rtai_tskext(TSKEXT0);
 }
 
-static inline pid_t rttask2pid(struct rt_task_struct * task)
+static inline long rttask2pid(struct rt_task_struct * task)
 {
-    return task->lnxtsk ? task->lnxtsk->pid : (int) task;
+    return task->lnxtsk ? task->lnxtsk->pid : (long)task;
 }
 
 #else /* !CONFIG_RTAI_INTERNAL_LXRT_SUPPORT */
@@ -527,7 +533,7 @@ static inline struct rt_task_struct *pid2rttask(pid_t pid)
 // buddies also.
 static inline pid_t rttask2pid(struct rt_task_struct * task)
 {
-    return (int) task;
+    return (long) task;
 }
 
 #endif /* CONFIG_RTAI_INTERNAL_LXRT_SUPPORT */
@@ -555,20 +561,22 @@ void reset_rt_fun_ext_index(struct rt_fun_entry *fun,
 #else /* !__KERNEL__ */
 
 #include <sys/types.h>
+#include <sys/mman.h>
 #include <sched.h>
 #include <stdarg.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <asm/rtai_lxrt.h>
 
 struct apic_timer_setup_data;
 
 #define rt_grow_and_lock_stack(incr) \
-        do { \
-                char buf[incr]; \
-                memset(buf, 0, incr); \
-                mlockall(MCL_CURRENT | MCL_FUTURE); \
-        } while (0)
+	do { \
+		char buf[incr]; \
+		memset(buf, 0, incr); \
+		mlockall(MCL_CURRENT | MCL_FUTURE); \
+	} while (0)
 
 #define BIDX   0 // rt_fun_ext[0]
 #define SIZARG sizeof(arg)
@@ -584,9 +592,9 @@ extern "C" {
  *
  * @return the address associated to @a name on success, 0 on failure
  */
-RTAI_PROTO(void *,rt_get_adr,(unsigned long name))
+RTAI_PROTO(void *, rt_get_adr, (unsigned long name))
 {
-	struct { int name; } arg = { name };
+	struct { unsigned long name; } arg = { name };
 	return rtai_lxrt(BIDX, SIZARG, LXRT_GET_ADR, &arg).v[LOW];
 } 
 
@@ -598,172 +606,177 @@ RTAI_PROTO(void *,rt_get_adr,(unsigned long name))
  * @return the identifier pointed by the address @a adr on success, 0 on
  * failure.
  */
-RTAI_PROTO(unsigned long,rt_get_name,(void *adr))
+RTAI_PROTO(unsigned long, rt_get_name, (void *adr))
 {
 	struct { void *adr; } arg = { adr };
 	return rtai_lxrt(BIDX, SIZARG, LXRT_GET_NAME, &arg).i[LOW];
 }
 
-RTAI_PROTO(RT_TASK *,rt_task_init_schmod,(int name, int priority, int stack_size, int max_msg_size, int policy, int cpus_allowed))
+RTAI_PROTO(RT_TASK *, rt_task_init_schmod, (unsigned long name, int priority, int stack_size, int max_msg_size, int policy, int cpus_allowed))
 {
         struct sched_param mysched;
-        struct { int name, priority, stack_size, max_msg_size, cpus_allowed; } arg = { name, priority, stack_size, max_msg_size, cpus_allowed };
+        struct { unsigned long name; long priority, stack_size, max_msg_size, cpus_allowed; } arg = { name ? name : rt_get_name(NULL), priority, stack_size, max_msg_size, cpus_allowed };
 
-        mysched.sched_priority = sched_get_priority_max(policy) - priority;
-        if (mysched.sched_priority < 1 ) {
-        	mysched.sched_priority = 1;
+        if (policy == SCHED_OTHER) {
+        	mysched.sched_priority = 0;
+	} else if ((mysched.sched_priority = sched_get_priority_max(policy) - priority) < 1) {
+		mysched.sched_priority = 1;
 	}
         if (sched_setscheduler(0, policy, &mysched) < 0) {
                 return 0;
         }
 	rtai_iopl();
+	mlockall(MCL_CURRENT | MCL_FUTURE);
 
 	return (RT_TASK *)rtai_lxrt(BIDX, SIZARG, LXRT_TASK_INIT, &arg).v[LOW];
 }
 
-#define RT_THREAD_STACK_MIN 64*1024
+static inline int rt_clone(void *fun, void *args, long stack_size, unsigned long flags)
+{
+	void *sp;
+	if (!flags) {
+		flags = CLONE_VM | CLONE_FS | CLONE_FILES;
+	}
+	memset(sp = malloc(stack_size), 0, stack_size);
+	sp = (void *)(((unsigned long)sp + stack_size - 16) & ~0xF);
+	return clone((int (*)(void *))fun, sp, flags, args);
+}
+
+#define RT_THREAD_STACK_MIN  64*1024
 
-#if 1
 #include <pthread.h>
 
-RTAI_PROTO(int, rt_thread_create,(void *fun, void *args, int stack_size))
+RTAI_PROTO(long, rt_thread_create, (void *fun, void *args, int stack_size))
 {
-	pthread_t thread;
+	long thread;
 	pthread_attr_t attr;
+
         pthread_attr_init(&attr);
-        if (pthread_attr_setstacksize(&attr, stack_size > RT_THREAD_STACK_MIN ? stack_size : RT_THREAD_STACK_MIN)) {
-                return -1;
-        }
-	if (pthread_create(&thread, &attr, (void *(*)(void *))fun, args)) {
-		return -1;
+	if (!pthread_attr_setstacksize(&attr, stack_size > RT_THREAD_STACK_MIN ? stack_size : RT_THREAD_STACK_MIN)) {
+		struct { unsigned long hs; } arg = { 0 };
+		if ((arg.hs = rtai_lxrt(BIDX, SIZARG, IS_HARD, &arg).i[LOW])) {
+			rtai_lxrt(BIDX, SIZARG, MAKE_SOFT_RT, &arg);
+		}
+		if (pthread_create((pthread_t *)&thread, &attr, (void *(*)(void *))fun, args)) {
+			thread = 0;
+		}
+		if (arg.hs) {
+			rtai_lxrt(BIDX, SIZARG, MAKE_HARD_RT, &arg);
+		}
+	} else {
+		thread = 0;
 	}
 	return thread;
 }
 
-RTAI_PROTO(int, rt_thread_join, (int thread))
+RTAI_PROTO(int, rt_thread_join, (long thread))
 {
 	return pthread_join((pthread_t)thread, NULL);
 }
 
-#else
-
-#include <sys/wait.h>
-
-RTAI_PROTO(int, rt_thread_create, (void *fun, void *args, int stack_size))
-{
-	void *sp;
-	if (stack_size < RT_THREAD_STACK_MIN) {
-		stack_size = RT_THREAD_STACK_MIN;
-	}
-	memset(sp = malloc(stack_size), 0, stack_size);
-	sp = (void *)(((unsigned long)sp + stack_size - 16) & ~0xF);
-	return clone(fun, sp, CLONE_VM | CLONE_FS | CLONE_FILES, args);
-}
-
-RTAI_PROTO(int, rt_thread_join, (int thread))
-{
-	return waitpid(thread, NULL, 0);
-}
-
-#endif
-
 #ifndef __SUPPORT_LINUX_SERVER__
 #define __SUPPORT_LINUX_SERVER__
 
-#include <asm/ptrace.h>
 #include <unistd.h>
+#include <sys/mman.h>
 
-static inline void rt_receive_linux_syscall(RT_TASK *task, struct pt_regs *regs)
+static void linux_syscall_server_fun(struct linux_syscalls_list *list)
 {
-	struct { RT_TASK *task; struct pt_regs *regs; } arg = { task, regs };
-	rtai_lxrt(BIDX, SIZARG, RECEIVE_LINUX_SYSCALL, &arg);
-}
+	struct linux_syscalls_list syscalls = *list;
 
-static inline void rt_return_linux_syscall(RT_TASK *task, unsigned long retval)
-{
-	struct { RT_TASK *task; unsigned long retval; } arg = { task, retval };
-	rtai_lxrt(BIDX, SIZARG, RETURN_LINUX_SYSCALL, &arg);
-}
+	syscalls.serv = &syscalls;
+	syscalls.in = syscalls.out = 0;
 
-#include <rtai_msg.h>
-static void linux_syscall_server_fun(RT_TASK *task)
-{
-        struct pt_regs regs;
-	rtai_lxrt(BIDX, sizeof(RT_TASK *), LINUX_SERVER_INIT, &task);
-	rtai_lxrt(BIDX, sizeof(RT_TASK *), RESUME, &task);
-        for (;;) {
-#if 1
-                rt_receive_linux_syscall(task, &regs);
-		rt_return_linux_syscall(task, syscall(regs.LINUX_SYSCALL_NR, regs.LINUX_SYSCALL_REG1, regs.LINUX_SYSCALL_REG2, regs.LINUX_SYSCALL_REG3, regs.LINUX_SYSCALL_REG4, regs.LINUX_SYSCALL_REG5, regs.LINUX_SYSCALL_REG6));
-#else
-		int retval;
-		rt_receivex(task, &regs, sizeof(struct pt_regs), &retval);
-		retval = syscall(regs.LINUX_SYSCALL_NR, regs.LINUX_SYSCALL_REG1, regs.LINUX_SYSCALL_REG2, regs.LINUX_SYSCALL_REG3, regs.LINUX_SYSCALL_REG4, regs.LINUX_SYSCALL_REG5, regs.LINUX_SYSCALL_REG6);
-		rt_returnx(task, &retval, sizeof(retval));
-#endif
+	if ((syscalls.serv = rtai_lxrt(BIDX, sizeof(struct linux_syscalls_list), LINUX_SERVER_INIT, &syscalls).v[LOW])) {
+		struct pt_regs *regs;
+		syscalls.moderegs = (struct mode_regs *)malloc(syscalls.nr*sizeof(struct mode_regs));
+		memset(syscalls.moderegs, syscalls.nr*sizeof(struct mode_regs), 0);
+                mlockall(MCL_CURRENT | MCL_FUTURE);
+		rtai_lxrt(BIDX, sizeof(RT_TASK *), RESUME, &syscalls.task);
+		for (;;) {
+			if (abs(rtai_lxrt(BIDX, sizeof(RT_TASK *), SUSPEND, &syscalls.serv).i[LOW]) == RTE_UNBLKD) {
+				break;
+			}
+			regs = &syscalls.moderegs[syscalls.out].regs;
+			syscalls.retval = syscall(regs->LINUX_SYSCALL_NR, regs->LINUX_SYSCALL_REG1, regs->LINUX_SYSCALL_REG2, regs->LINUX_SYSCALL_REG3, regs->LINUX_SYSCALL_REG4, regs->LINUX_SYSCALL_REG5, regs->LINUX_SYSCALL_REG6);
+			if (syscalls.moderegs[syscalls.out].mode == SYNC_LINUX_SYSCALL) {
+				rtai_lxrt(BIDX, sizeof(RT_TASK *), RESUME, &syscalls.task);
+			} else if (syscalls.callback_fun) {
+				syscalls.callback_fun(regs->LINUX_SYSCALL_NR, syscalls.retval);
+			}
+			if (++syscalls.out >= syscalls.nr) {
+				syscalls.out = 0;
+			}
+		}
+		free(syscalls.moderegs);
         }
 }
 
 #endif /* __SUPPORT_LINUX_SERVER__ */
 
-RTAI_PROTO(int, rt_linux_syscall_server_create, (RT_TASK * task))
+RTAI_PROTO(void, rt_set_linux_syscall_mode, (int mode, void (*callback_fun)(long, long)))
 {
-	if (rt_thread_create((void *)linux_syscall_server_fun, task, 0) > 0) {
-		printf(" \b");
-		rtai_lxrt(BIDX, sizeof(RT_TASK *), SUSPEND, &task);
-		return 0;
+	struct { long mode; void (*callback_fun)(long, long); } arg = { mode, callback_fun };
+	rtai_lxrt(BIDX, SIZARG, SET_LINUX_SYSCALL_MODE, &arg);
+}
+
+RTAI_PROTO(int, rt_sync_async_linux_syscall_server_create, (RT_TASK *task, int mode, void (*callback_fun)(long, long), int nr_bufd_async_calls))
+{
+	if ((task || (task = (RT_TASK *)rtai_lxrt(BIDX, sizeof(RT_TASK *), RT_BUDDY, &task).v[LOW])) && nr_bufd_async_calls > 0) {
+		struct linux_syscalls_list syscalls;
+		syscalls.task         = task;
+		syscalls.callback_fun = callback_fun;
+		syscalls.mode         = mode;
+		syscalls.nr           = nr_bufd_async_calls;
+		if (rt_thread_create((void *)linux_syscall_server_fun, &syscalls, 0)) {
+			rtai_lxrt(BIDX, sizeof(RT_TASK *), SUSPEND, &task);
+			return 0;
+		}
 	}
 	return -1;
 }
 
-RTAI_PROTO(RT_TASK *, rt_thread_init, (int name, int priority, int max_msg_size, int policy, int cpus_allowed))
+#define rt_linux_syscall_server_create(task)  rt_sync_async_linux_syscall_server_create(task, SYNC_LINUX_SYSCALL, NULL, 1);
+
+RTAI_PROTO(RT_TASK *, rt_thread_init, (unsigned long name, int priority, int max_msg_size, int policy, int cpus_allowed))
 {
 	return rt_task_init_schmod(name, priority, 0, max_msg_size, policy, cpus_allowed);
 }
 
 /**
- * Create a new real time task in user space.
+ * Create an RTAI task extension for a Linux process/task in user space.
  * 
- * rt_task_init provides a real time buddy, also called proxy, task to the Linux
- * process that wants to access RTAI scheduler services.   It needs no task
- * function as none is used, but it does need to setup a task structure and
- * initialize it appropriately as the provided services are carried out as if
- * the Linux process has become an RTAI task.   Because of that it requires less
- * arguments and returns the pointer to the task that is to be used in related
- * calls.
+ * rt_task_init extends the Linux task structure, making it possible to use
+ * RTAI APIs that wants to access RTAI scheduler services.   It needs no task
+ * function as none is used, but it does need to setup an RTAI task structure
+ * and initialize it appropriately as the provided services are carried out as
+ * if the Linux process has become an RTAI task also.   Because of that it 
+ * requires less arguments and returns the pointer to the RTAI task extension
+ * that is to be used in related calls.
  *
- * @param name is a unique identifier that is possibly used by easing
- * referencing the buddy RTAItask, and thus its peer Linux process.
+ * @param name is a unique identifier that is possibly used to ease
+ * referencing the RTAI task extension of a peer Linux process.
  *
- * @param priority is the priority of the buddys priority.
+ * @param priority is the priority of the RTAI task extension.
  *
- * @param stack_size is just what is implied by such a name and refers to the
- * stack size used by the buddy.
+ * @param stack_size, a legacy parameter used nomore; kept for portability 
+ * reasons only. (It was just what is implied by such a name and referred to 
+ * the stack size used by the buddy in the very first implementation of LXRT).
  *
- * @param max_msg_size is a hint for the size of the most lengthy message than
- * is likely to be exchanged.
+ * @param max_msg_size is a hint for the size of the most lengthy intertask
+ * message that is likely to be exchanged.
  *
- * @a stack_size and @a max_msg_size can be zero, in which case the default
- * internal values are used.  The assignment of a different value should be
- * required only if you want to use task signal functions. In such a case note
- * that these signal functions are intended to catch asyncrounous events in
- * kernel space and, as such, must be programmed into a companion module and
- * interfaced to their parent Linux process through the available services.
- *
- * Keep an eye on the default stack (512) and message (256) sizes as they seem
- * to be acceptable, but this API has not been used extensively with complex
- * interrupt service  routines.   Since the latter are served on the stack of
- * any task being interrupted, and more than one can pile up on the same stack,
- * it can be possible that a larger stack is required.   In such a case either
- * recompile lxrt.c with macros STACK_SIZE and MSG_SIZE set appropriately, or
- * explicitly assign larger values at your buddy tasks  inits.   Note that while
- * the stack size can be critical the message size will not. In fact the module
- * reassigns it, appropriately sized, whenever it is needed.   The cost is a
- * kmalloc with GFP_KERNEL that can block, but within the Linux environment.
+ * @a max_msg_size can be zero, in which case a default internal value is
+ * used.  Keep an eye on such a default message (256) size. It could be 
+ * possible that a larger size is required to suite your needs best. In such 
+ * a case either recompile sys.c with the macro MSG_SIZE set appropriately, 
+ * or assign a larger size here esplicitly.  Note that the message size is 
+ * not critical though. In fact the module reassigns it, dynamically and 
+ * appropriately sized, whenever it is needed.  The cost is a real time 
+ * allocation of the new buffer.
  * Note also that @a max_msg_size is for a buffer to be used to copy whatever
- * message, either mailbox or inter task, from user to kernel space, as messages
- * are not necessarily copied immediately, and has nothing to do directly with
- * what you are doing.
+ * intertask message from user to kernel space, as intertask messages are not 
+ * necessarily used immediately.
  *
  * It is important to remark that the returned task pointers cannot be used
  * directly, they are for kernel space data, but just passed as arguments when
@@ -771,23 +784,23 @@ RTAI_PROTO(RT_TASK *, rt_thread_init, (int name, int priority, int max_msg_size,
  *
  * @return On success a pointer to the task structure initialized in kernel
  * space.
- * @return On failure a 0 value is returned if it was not possible to setup the
- * buddy task or something using the same name was found.
+ * @return On failure a NULL value is returned if it was not possible to setup 
+ * the RTAI task extension or something using the same name was found.
  */
-RTAI_PROTO(RT_TASK *,rt_task_init,(int name, int priority, int stack_size, int max_msg_size))
+RTAI_PROTO(RT_TASK *,rt_task_init,(unsigned long name, int priority, int stack_size, int max_msg_size))
 {
 	return rt_task_init_schmod(name, priority, 0, max_msg_size, SCHED_FIFO, 0xFF);
 }
 
 RTAI_PROTO(void,rt_set_sched_policy,(RT_TASK *task, int policy, int rr_quantum_ns))
 {
-	struct { RT_TASK *task; int policy; int rr_quantum_ns; } arg = { task, policy, rr_quantum_ns };
+	struct { RT_TASK *task; long policy; long rr_quantum_ns; } arg = { task, policy, rr_quantum_ns };
 	rtai_lxrt(BIDX, SIZARG, SET_SCHED_POLICY, &arg);
 }
 
 RTAI_PROTO(int,rt_change_prio,(RT_TASK *task, int priority))
 {
-	struct { RT_TASK *task; int priority; } arg = { task, priority };
+	struct { RT_TASK *task; long priority; } arg = { task, priority };
 	return rtai_lxrt(BIDX, SIZARG, CHANGE_TASK_PRIO, &arg).i[LOW];
 }
 
@@ -816,6 +829,8 @@ RTAI_PROTO(int,rt_task_delete,(RT_TASK *task))
 	return rtai_lxrt(BIDX, SIZARG, LXRT_TASK_DELETE, &arg).i[LOW];
 }
 
+#define rt_thread_delete(task)  rt_task_delete(task)
+
 RTAI_PROTO(int,rt_task_yield,(void))
 {
 	struct { unsigned long dummy; } arg;
@@ -854,13 +869,13 @@ RTAI_PROTO(int,rt_task_resume,(RT_TASK *task))
 
 RTAI_PROTO(void, rt_sched_lock, (void))
 {
-	struct { int dummy; } arg;
+	struct { long dummy; } arg;
 	rtai_lxrt(BIDX, SIZARG, SCHED_LOCK, &arg);
 }
 
 RTAI_PROTO(void, rt_sched_unlock, (void))
 {
-	struct { int dummy; } arg;
+	struct { long dummy; } arg;
 	rtai_lxrt(BIDX, SIZARG, SCHED_UNLOCK, &arg);
 }
 
@@ -902,7 +917,7 @@ RTAI_PROTO(int, rt_irq_signal, (unsigned irq))
 
 RTAI_PROTO(int, rt_request_irq_task, (unsigned irq, void *handler, int type, int affine2task))
 {
-	struct { unsigned irq; void *handler; int type, affine2task; } arg = { irq, handler, type, affine2task };
+	struct { unsigned irq; void *handler; long type, affine2task; } arg = { irq, handler, type, affine2task };
 	return rtai_lxrt(BIDX, SIZARG, REQUEST_IRQ_TASK, &arg).i[LOW];
 }
 
@@ -925,19 +940,19 @@ RTAI_PROTO(int,rt_task_make_periodic_relative_ns,(RT_TASK *task, RTIME start_del
 	return rtai_lxrt(BIDX, SIZARG, MAKE_PERIODIC_NS, &arg).i[LOW];
 }
 
-RTAI_PROTO(int, rt_task_wait_period,(void))
+RTAI_PROTO(int,rt_task_wait_period,(void))
 {
 	struct { unsigned long dummy; } arg;
 	return rtai_lxrt(BIDX, SIZARG, WAIT_PERIOD, &arg).i[LOW];
 }
 
-RTAI_PROTO(int, rt_sleep,(RTIME delay))
+RTAI_PROTO(int,rt_sleep,(RTIME delay))
 {
 	struct { RTIME delay; } arg = { delay };
 	return rtai_lxrt(BIDX, SIZARG, SLEEP, &arg).i[LOW];
 }
 
-RTAI_PROTO(int, rt_sleep_until,(RTIME time))
+RTAI_PROTO(int,rt_sleep_until,(RTIME time))
 {
 	struct { RTIME time; } arg = { time };
 	return rtai_lxrt(BIDX, SIZARG, SLEEP_UNTIL, &arg).i[LOW];
@@ -949,16 +964,44 @@ RTAI_PROTO(int,rt_is_hard_timer_running,(void))
 	return rtai_lxrt(BIDX, SIZARG, HARD_TIMER_RUNNING, &arg).i[LOW];
 }
 
-RTAI_PROTO(RTIME, start_rt_timer,(int period))
+RTAI_PROTO(RTIME, start_rt_timer, (int period))
 {
-	struct { int period; } arg = { period };
-	return rtai_lxrt(BIDX, SIZARG, START_TIMER, &arg).rt;
+	int hs;
+	RTIME retval;
+	struct { long period; } arg = { 0 };
+	if ((hs = rtai_lxrt(BIDX, SIZARG, IS_HARD, &arg).i[LOW])) {
+		rtai_lxrt(BIDX, SIZARG, MAKE_SOFT_RT, &arg);
+	}
+	arg.period = period;
+	retval = rtai_lxrt(BIDX, SIZARG, START_TIMER, &arg).rt;
+	if (hs) {
+		rtai_lxrt(BIDX, SIZARG, MAKE_HARD_RT, &arg);
+	}
+	return retval;
 }
 
-RTAI_PROTO(void, stop_rt_timer,(void))
+RTAI_PROTO(void, stop_rt_timer, (void))
+{
+	struct { long hs; } arg = { 0 };
+	if ((arg.hs = rtai_lxrt(BIDX, SIZARG, IS_HARD, &arg).i[LOW])) {
+		rtai_lxrt(BIDX, SIZARG, MAKE_SOFT_RT, &arg);
+	}
+	rtai_lxrt(BIDX, SIZARG, STOP_TIMER, &arg);
+	if (arg.hs) {
+		rtai_lxrt(BIDX, SIZARG, MAKE_HARD_RT, &arg);
+	}
+}
+
+RTAI_PROTO(void, rt_request_rtc,(int rtc_freq, void *handler))
+{
+	struct { long rtc_freq; void *handler; } arg = { rtc_freq, handler };
+	rtai_lxrt(BIDX, SIZARG, REQUEST_RTC, &arg);
+}
+
+RTAI_PROTO(void, rt_release_rtc,(void))
 {
 	struct { unsigned long dummy; } arg;
-	rtai_lxrt(BIDX, SIZARG, STOP_TIMER, &arg);
+	rtai_lxrt(BIDX, SIZARG, RELEASE_RTC, &arg);
 }
 
 RTAI_PROTO(RTIME,rt_get_time,(void))
@@ -967,6 +1010,18 @@ RTAI_PROTO(RTIME,rt_get_time,(void))
 	return rtai_lxrt(BIDX, SIZARG, GET_TIME, &arg).rt;
 }
 
+RTAI_PROTO(RTIME, rt_get_real_time, (void))
+{
+	struct { unsigned long dummy; } arg;
+	return rtai_lxrt(BIDX, SIZARG, GET_REAL_TIME, &arg).rt;
+}
+
+RTAI_PROTO(RTIME, rt_get_real_time_ns, (void))
+{
+	struct { unsigned long dummy; } arg;
+	return rtai_lxrt(BIDX, SIZARG, GET_REAL_TIME_NS, &arg).rt;
+}
+
 RTAI_PROTO(RTIME,count2nano,(RTIME count))
 {
 	struct { RTIME count; } arg = { count };
@@ -981,7 +1036,7 @@ RTAI_PROTO(RTIME,nano2count,(RTIME nanos))
 
 RTAI_PROTO(void,rt_busy_sleep,(int ns))
 {
-	struct { int ns; } arg = { ns };
+	struct { long ns; } arg = { ns };
 	rtai_lxrt(BIDX, SIZARG, BUSY_SLEEP, &arg);
 }
 
@@ -997,7 +1052,7 @@ RTAI_PROTO(void,rt_set_oneshot_mode,(void))
 	rtai_lxrt(BIDX, SIZARG, SET_ONESHOT_MODE, &arg);
 }
 
-RTAI_PROTO(int,rt_task_signal_handler,(RT_TASK *task, void (*handler)(void)))
+RTAI_PROTO(int, rt_task_signal_handler, (RT_TASK *task, void (*handler)(void)))
 {
 	struct { RT_TASK *task; void (*handler)(void); } arg = { task, handler };
 	return rtai_lxrt(BIDX, SIZARG, SIGNAL_HANDLER, &arg).i[LOW];
@@ -1005,7 +1060,7 @@ RTAI_PROTO(int,rt_task_signal_handler,(RT_TASK *task, void (*handler)(void)))
 
 RTAI_PROTO(int,rt_task_use_fpu,(RT_TASK *task, int use_fpu_flag))
 {
-        struct { RT_TASK *task; int use_fpu_flag; } arg = { task, use_fpu_flag };
+        struct { RT_TASK *task; long use_fpu_flag; } arg = { task, use_fpu_flag };
         if (rtai_lxrt(BIDX, SIZARG, RT_BUDDY, &arg).v[LOW] != task) {
                 return rtai_lxrt(BIDX, SIZARG, TASK_USE_FPU, &arg).i[LOW];
         } else {
@@ -1019,20 +1074,27 @@ RTAI_PROTO(int,rt_task_use_fpu,(RT_TASK *task, int use_fpu_flag))
 
 RTAI_PROTO(int,rt_buddy_task_use_fpu,(RT_TASK *task, int use_fpu_flag))
 {
-	struct { RT_TASK *task; int use_fpu_flag; } arg = { task, use_fpu_flag };
+	struct { RT_TASK *task; long use_fpu_flag; } arg = { task, use_fpu_flag };
 	return rtai_lxrt(BIDX, SIZARG, TASK_USE_FPU, &arg).i[LOW];
 }
 
+/*
 RTAI_PROTO(int,rt_linux_use_fpu,(int use_fpu_flag))
 {
-	struct { int use_fpu_flag; } arg = { use_fpu_flag };
+	struct { long use_fpu_flag; } arg = { use_fpu_flag };
 	return rtai_lxrt(BIDX, SIZARG, LINUX_USE_FPU, &arg).i[LOW];
 }
-
-RTAI_PROTO(void,rt_preempt_always,(int yes_no))
+*/
+RTAI_PROTO(int, rt_get_priorities, (RT_TASK *task, int *priority, int *base_priority))
 {
-	struct { int yes_no; } arg = { yes_no };
-	rtai_lxrt(BIDX, SIZARG, PREEMPT_ALWAYS_GEN, &arg);
+	struct { RT_TASK *task; int *priority, *base_priority; } arg = { task, priority, base_priority };
+	return rtai_lxrt(BIDX, SIZARG, GET_PRIORITIES, &arg).i[LOW];
+}
+
+RTAI_PROTO(int, rt_hard_timer_tick, (void))
+{
+	struct { long dummy; } arg;
+	return rtai_lxrt(BIDX, SIZARG, HARD_TIMER_COUNT, &arg).i[LOW];
 }
 
 RTAI_PROTO(RTIME,rt_get_time_ns,(void))
@@ -1061,7 +1123,7 @@ RTAI_PROTO(void,rt_set_runnable_on_cpus,(RT_TASK *task, unsigned long cpu_mask))
 
 RTAI_PROTO(void,rt_set_runnable_on_cpuid,(RT_TASK *task, unsigned int cpuid))
 {
-	struct { RT_TASK *task; unsigned int cpuid; } arg = { task, cpuid };
+	struct { RT_TASK *task; unsigned long cpuid; } arg = { task, cpuid };
 	rtai_lxrt(BIDX, SIZARG, SET_RUNNABLE_ON_CPUID, &arg);
 }
 
@@ -1073,55 +1135,55 @@ RTAI_PROTO(int,rt_get_timer_cpu,(void))
 
 RTAI_PROTO(void,start_rt_apic_timers,(struct apic_timer_setup_data *setup_mode, unsigned int rcvr_jiffies_cpuid))
 {
-	struct { struct apic_timer_setup_data *setup_mode; unsigned int rcvr_jiffies_cpuid; } arg = { setup_mode, rcvr_jiffies_cpuid };
+	struct { struct apic_timer_setup_data *setup_mode; unsigned long rcvr_jiffies_cpuid; } arg = { setup_mode, rcvr_jiffies_cpuid };
 	rtai_lxrt(BIDX, SIZARG, START_RT_APIC_TIMERS, &arg);
 }
 
-RTAI_PROTO(void,rt_preempt_always_cpuid,(int yes_no, unsigned int cpuid))
+RTAI_PROTO(int, rt_hard_timer_tick_cpuid, (int cpuid))
 {
-	struct { int yes_no; unsigned int cpuid; } arg = { yes_no, cpuid };
-	rtai_lxrt(BIDX, SIZARG, PREEMPT_ALWAYS_CPUID, &arg);
+	struct { unsigned long cpuid; } arg = { cpuid };
+	return rtai_lxrt(BIDX, SIZARG, HARD_TIMER_COUNT_CPUID, &arg).i[LOW];
 }
 
 RTAI_PROTO(RTIME,count2nano_cpuid,(RTIME count, unsigned int cpuid))
 {
-	struct { RTIME count; unsigned int cpuid; } arg = { count, cpuid };
+	struct { RTIME count; unsigned long cpuid; } arg = { count, cpuid };
 	return rtai_lxrt(BIDX, SIZARG, COUNT2NANO_CPUID, &arg).rt;
 }
 
 RTAI_PROTO(RTIME,nano2count_cpuid,(RTIME nanos, unsigned int cpuid))
 {
-	struct { RTIME nanos; unsigned int cpuid; } arg = { nanos, cpuid };
+	struct { RTIME nanos; unsigned long cpuid; } arg = { nanos, cpuid };
 	return rtai_lxrt(BIDX, SIZARG, NANO2COUNT_CPUID, &arg).rt;
 }
 
 RTAI_PROTO(RTIME,rt_get_time_cpuid,(unsigned int cpuid))
 {
-	struct { unsigned int cpuid; } arg = { cpuid };
+	struct { unsigned long cpuid; } arg = { cpuid };
 	return rtai_lxrt(BIDX, SIZARG, GET_TIME_CPUID, &arg).rt;
 }
 
 RTAI_PROTO(RTIME,rt_get_time_ns_cpuid,(unsigned int cpuid))
 {
-	struct { unsigned int cpuid; } arg = { cpuid };
+	struct { unsigned long cpuid; } arg = { cpuid };
 	return rtai_lxrt(BIDX, SIZARG, GET_TIME_NS_CPUID, &arg).rt;
 }
 
 RTAI_PROTO(void,rt_boom,(void))
 {
-	struct { int dummy; } arg = { 0 };
+	struct { long dummy; } arg = { 0 };
 	rtai_lxrt(BIDX, SIZARG, RT_BOOM, &arg);
 }
 
 RTAI_PROTO(void,rt_mmgr_stats,(void))
 {
-	struct { int dummy; } arg = { 0 };
+	struct { long dummy; } arg = { 0 };
 	rtai_lxrt(BIDX, SIZARG, RT_MMGR_STATS, &arg);
 }
 
 RTAI_PROTO(void,rt_stomp,(void) )
 {
-	struct { int dummy; } arg = { 0 };
+	struct { long dummy; } arg = { 0 };
 	rtai_lxrt(BIDX, SIZARG, RT_STOMP, &arg);
 }
 
@@ -1143,14 +1205,15 @@ RTAI_PROTO(int,rt_set_linux_signal_handler,(RT_TASK *task, void (*handler)(int s
     return rtai_lxrt(BIDX, SIZARG, RT_SET_LINUX_SIGNAL_HANDLER, &arg).i[LOW];
 }
 
+#define VSNPRINTF_BUF_SIZE 256
 RTAI_PROTO(int,rtai_print_to_screen,(const char *format, ...))
 {
-	char display[256];
-	struct { const char *display; int nch; } arg = { display, 0 };
+	char display[VSNPRINTF_BUF_SIZE];
+	struct { const char *display; long nch; } arg = { display, 0 };
 	va_list args;
 
 	va_start(args, format);
-	arg.nch = vsprintf(display, format, args);
+	arg.nch = vsnprintf(display, VSNPRINTF_BUF_SIZE, format, args);
 	va_end(args);
 	rtai_lxrt(BIDX, SIZARG, PRINT_TO_SCREEN, &arg);
 	return arg.nch;
@@ -1158,12 +1221,12 @@ RTAI_PROTO(int,rtai_print_to_screen,(const char *format, ...))
 
 RTAI_PROTO(int,rt_printk,(const char *format, ...))
 {
-	char display[256];
-	struct { const char *display; int nch; } arg = { display, 0 };
+	char display[VSNPRINTF_BUF_SIZE];
+	struct { const char *display; long nch; } arg = { display, 0 };
 	va_list args;
 
 	va_start(args, format);
-	arg.nch = vsprintf(display, format, args);
+	arg.nch = vsnprintf(display, VSNPRINTF_BUF_SIZE, format, args);
 	va_end(args);
 	rtai_lxrt(BIDX, SIZARG, PRINTK, &arg);
 	return arg.nch;
@@ -1201,7 +1264,7 @@ RTAI_PROTO(void,rt_set_usp_flags_mask,(unsigned long flags_mask))
 
 RTAI_PROTO(RT_TASK *,rt_force_task_soft,(int pid))
 {
-	struct { int pid; } arg = { pid };
+	struct { long pid; } arg = { pid };
 	return (RT_TASK *)rtai_lxrt(BIDX, SIZARG, FORCE_TASK_SOFT, &arg).v[LOW];
 }
 
@@ -1213,6 +1276,12 @@ RTAI_PROTO(RT_TASK *,rt_agent,(void))
 
 #define rt_buddy() rt_agent()
 
+RTAI_PROTO(int, rt_gettid, (void))
+{
+	struct { unsigned long dummy; } arg;
+	return rtai_lxrt(BIDX, SIZARG, RT_GETTID, &arg).i[LOW];
+}
+
 /**
  * Give a Linux process, or pthread, hard real time execution capabilities 
  * allowing full kernel preemption.
@@ -1283,17 +1352,19 @@ RTAI_PROTO(int,rt_set_period,(RT_TASK *rt_task, RTIME new_period))
 
 RTAI_PROTO(void,rt_spv_RMS,(int cpuid))
 {
-	struct { int cpuid; } arg = { cpuid };
+	struct { long cpuid; } arg = { cpuid };
 	rtai_lxrt(BIDX, SIZARG, SPV_RMS, &arg);
 }
 
-RTAI_PROTO(int,rt_task_wakeup_sleeping,(RT_TASK *task))
+RTAI_PROTO(int, rt_task_masked_unblock,(RT_TASK *task, unsigned long mask))
 {
-	struct { RT_TASK *task; } arg = { task };
+	struct { RT_TASK *task; unsigned long mask; } arg = { task, mask };
 	return rtai_lxrt(BIDX, SIZARG, WAKEUP_SLEEPING, &arg).i[LOW];
 }
 
-RTAI_PROTO(void,rt_get_exectime,(RT_TASK *task, RTIME *exectime))
+#define rt_task_wakeup_sleeping(task, mask)  rt_task_masked_unblock(task, RT_SCHED_DELAYED)
+
+RTAI_PROTO(void, rt_get_exectime, (RT_TASK *task, RTIME *exectime))
 {
 	RTIME lexectime[] = { 0LL, 0LL, 0LL };
 	struct { RT_TASK *task; RTIME *lexectime; } arg = { task, lexectime };
@@ -1301,10 +1372,37 @@ RTAI_PROTO(void,rt_get_exectime,(RT_TASK *task, RTIME *exectime))
 	memcpy(exectime, lexectime, sizeof(lexectime));
 }
 
-RTAI_PROTO(void,rt_gettimeorig,(RTIME time_orig[]))
+RTAI_PROTO(void, rt_gettimeorig, (RTIME time_orig[]))
 {
-	struct { RTIME *time_orig; } arg = { time_orig };
+	RTIME lorig[2];
+	struct { RTIME *time_orig; } arg = { lorig };
 	rtai_lxrt(BIDX, SIZARG, GET_TIMEORIG, &arg);
+	memcpy(time_orig, lorig, sizeof(lorig));
+}
+
+RTAI_PROTO(RT_TASK *,ftask_init,(unsigned long name, int priority))
+{
+	struct { unsigned long name; long priority, stack_size, max_msg_size, cpus_allowed; } arg = { name, priority, 0, 0, 0 };
+	return (RT_TASK *)rtai_lxrt(BIDX, SIZARG, LXRT_TASK_INIT, &arg).v[LOW];
+}
+
+RTAI_PROTO(RTIME, start_ftimer,(long period, long ftick_freq))
+{
+	struct { long ftick_freq; void *handler; } arg = { ftick_freq, NULL };
+	if (!period) {
+		rtai_lxrt(BIDX, sizeof(long), SET_ONESHOT_MODE, &period);
+	} else {
+		rtai_lxrt(BIDX, sizeof(long), SET_PERIODIC_MODE, &period);
+	}
+	rtai_lxrt(BIDX, SIZARG, REQUEST_RTC, &arg);
+	return rtai_lxrt(BIDX, sizeof(long), START_TIMER, &period).rt;
+}
+
+RTAI_PROTO(RTIME, stop_ftimer,(void))
+{
+	struct { long dummy; } arg;
+	rtai_lxrt(BIDX, SIZARG, RELEASE_RTC, &arg);
+	return rtai_lxrt(BIDX, SIZARG, STOP_TIMER, &arg).rt;
 }
 
 #ifdef __cplusplus
