@@ -147,9 +147,7 @@ ACKNOWLEDGEMENTS:
 #include <linux/termios.h>
 #include <linux/tty_driver.h>
 #include <linux/console.h>
-#include <linux/config.h>
 #include <linux/slab.h>
-#include <linux/devfs_fs_kernel.h>
 #include <linux/stat.h>
 #include <linux/proc_fs.h>
 
@@ -213,7 +211,7 @@ typedef struct rt_fifo_struct {
 	int pol_asyn_pended;
 	wait_queue_head_t pollq;
 	struct fasync_struct *asynq;
-	int (*handler)(unsigned int arg);
+	rtf_handler_t handler;
 	F_SEM sem;
 	char name[RTF_NAMELEN+1];
 } FIFO;
@@ -230,9 +228,7 @@ static FIFO *fifo;
 static struct { int in, out; struct task_struct *task[MAXREQS]; } taskq;
 static struct { int in, out; FIFO *fifo[MAXREQS]; } pol_asyn_q;
 
-static RT_TASK *rt_base_linux_task;
-
-static int do_nothing(unsigned int arg) { return 0; }
+static int do_nothing(unsigned int arg, int rw) { return 0; }
 
 static inline void enqueue_blocked(LX_TASK *task, F_QUEUE *queue, int qtype, int priority)
 {
@@ -442,7 +438,7 @@ static inline int mbx_put(F_MBX *mbx, char **msg, int msg_size, int lnx)
 			tocpy = mbx->frbs;
 		}
 		if (lnx) {
-			copy_from_user(mbx->bufadr + mbx->lbyte, *msg, tocpy);
+			rt_copy_from_user(mbx->bufadr + mbx->lbyte, *msg, tocpy);
 		} else {
 			memcpy(mbx->bufadr + mbx->lbyte, *msg, tocpy);
 		}
@@ -475,7 +471,7 @@ static inline int mbx_ovrwr_put(F_MBX *mbx, char **msg, int msg_size, int lnx)
 				tocpy = mbx->frbs;
 			}
 			if (lnx) {
-				copy_from_user(mbx->bufadr + mbx->lbyte, *msg, tocpy);
+				rt_copy_from_user(mbx->bufadr + mbx->lbyte, *msg, tocpy);
 			} else {
 				memcpy(mbx->bufadr + mbx->lbyte, *msg, tocpy);
 			}
@@ -519,7 +515,7 @@ static inline int mbx_get(F_MBX *mbx, char **msg, int msg_size, int lnx)
 			tocpy = mbx->avbs;
 		}
 		if (lnx) {
-			copy_to_user(*msg, mbx->bufadr + mbx->fbyte, tocpy);
+			rt_copy_to_user(*msg, mbx->bufadr + mbx->fbyte, tocpy);
 		} else {
 			memcpy(*msg, mbx->bufadr + mbx->fbyte, tocpy);
 		}
@@ -548,7 +544,7 @@ static inline int mbx_evdrp(F_MBX *mbx, char **msg, int msg_size, int lnx)
 			tocpy = avbs;
 		}
 		if (lnx) {
-			copy_to_user(*msg, mbx->bufadr + fbyte, tocpy);
+			rt_copy_to_user(*msg, mbx->bufadr + fbyte, tocpy);
 		} else {
 			memcpy(*msg, mbx->bufadr + fbyte, tocpy);
 		}
@@ -595,7 +591,7 @@ static inline void mbx_init(F_MBX *mbx, int size, char *bufadr)
 	mbx->size = mbx->frbs = size;
 	mbx->fbyte = mbx->lbyte = mbx->avbs = 0;
 #ifdef CONFIG_SMP
-        mbx->buflock.lock = 0;
+        spin_lock_init(&mbx->buflock);
 #endif
 	spin_lock_init(&(mbx->buflock));
 }
@@ -733,7 +729,7 @@ static inline int mbx_receive_if(F_MBX *mbx, void *msg, int msg_size, int lnx)
 	unsigned long flags;
 
 	rtf_save_flags_and_cli(flags);
-	if (mbx->rcvsem.free && (mbx->frbs >= msg_size)) {
+	if (mbx->rcvsem.free && mbx->avbs >= msg_size) {
 		mbx->rcvsem.free = 0;
 		rtf_restore_flags(flags);
 		msg_size = mbx_get(mbx, (char **)(&msg), msg_size, lnx);
@@ -822,7 +818,7 @@ static void rtf_sysrq_handler(void)
  * @retval EINVAL if @a fd_fifo refers to a not opened fifo.
  * @retval EFAULT if the operation was unsuccessful.
  */
-int rtf_reset(unsigned int minor)
+RTAI_SYSCALL_MODE int rtf_reset(unsigned int minor)
 {
 	int semval;
 	F_MBX *mbx;
@@ -872,7 +868,7 @@ int rtf_reset(unsigned int minor)
  * @retval -EBUSY if @a size is smaller than actual content
  * size is unchanged.
  */
-int rtf_resize(unsigned int minor, int size)
+RTAI_SYSCALL_MODE int rtf_resize(unsigned int minor, int size)
 {
 	void *oldbuf, *newbuf;
 	int old_malloc_type, new_malloc_type, semval;
@@ -972,7 +968,7 @@ int rtf_resize(unsigned int minor, int size)
  * @retval ENOMEM if the necessary size could not be allocated for the RT-FIFO.
  *
  */
-int rtf_create(unsigned int minor, int size)
+RTAI_SYSCALL_MODE int rtf_create(unsigned int minor, int size)
 {
 	void *buf;
 
@@ -980,7 +976,7 @@ int rtf_create(unsigned int minor, int size)
 		return -ENODEV;
 	}
 	TRACE_RTAI_FIFO(TRACE_RTAI_EV_FIFO_CREATE, minor, size);
-	if (!atomic_cmpxchg(&fifo[minor].opncnt, 0, 1)) {
+	if (!atomic_cmpxchg((atomic_t *)&fifo[minor].opncnt, 0, 1)) {
 		if (size <= PAGE_SIZE*32) {
 			if (!(buf = kmalloc(size, GFP_KERNEL))) {
 				fifo[minor].opncnt = 0;
@@ -1031,7 +1027,7 @@ int rtf_create(unsigned int minor, int size)
  * @note The equivalent of rtf_destroy in user space is the standard UNIX
  * close.
  */
-int rtf_destroy(unsigned int minor)
+RTAI_SYSCALL_MODE int rtf_destroy(unsigned int minor)
 {
 	VALID_FIFO;
 
@@ -1078,7 +1074,7 @@ int rtf_destroy(unsigned int minor)
  * or not. The next call of rtf_create will uninstall the handler just
  * "installed".
  */
-int rtf_create_handler(unsigned int minor, int (*handler) (unsigned int fifo))
+int rtf_create_handler(unsigned int minor, void *handler)
 {
 	if (minor >= MAX_FIFOS || !handler) {
 		return -EINVAL;
@@ -1117,7 +1113,7 @@ int rtf_create_handler(unsigned int minor, int (*handler) (unsigned int fifo))
  * which can be either blocking or nonblocking according to how you opened the
  * related device.
  */
-int rtf_put(unsigned int minor, void *buf, int count)
+RTAI_SYSCALL_MODE int rtf_put(unsigned int minor, void *buf, int count)
 {
 	VALID_FIFO;
 
@@ -1127,13 +1123,13 @@ int rtf_put(unsigned int minor, void *buf, int count)
 	return count;
 }
 
-int rtf_ovrwr_put(unsigned int minor, void *buf, int count)
+RTAI_SYSCALL_MODE int rtf_ovrwr_put(unsigned int minor, void *buf, int count)
 {
 	VALID_FIFO;
 	return mbx_ovrwr_send(&(fifo[minor].mbx), buf, count, 0);
 }
 
-int rtf_put_if(unsigned int minor, void *buf, int count)
+RTAI_SYSCALL_MODE int rtf_put_if(unsigned int minor, void *buf, int count)
 {
 	VALID_FIFO;
 
@@ -1173,7 +1169,7 @@ int rtf_put_if(unsigned int minor, void *buf, int count)
  * which can be either blocking or nonblocking according to how you opened the
  * related device.
  */
-int rtf_get(unsigned int minor, void *buf, int count)
+RTAI_SYSCALL_MODE int rtf_get(unsigned int minor, void *buf, int count)
 {
 	VALID_FIFO;
 
@@ -1190,11 +1186,11 @@ int rtf_evdrp(unsigned int minor, void *msg, int msg_size)
 	return msg_size - mbx_evdrp(&(fifo[minor].mbx), (char **)(&msg), msg_size, 0);
 }
 
-int rtf_get_if(unsigned int minor, void *buf, int count)
+RTAI_SYSCALL_MODE int rtf_get_if(unsigned int minor, void *buf, int count)
 {
 	VALID_FIFO;
 
-	return count - mbx_send_if(&(fifo[minor].mbx), buf, count, 0);
+	return count - mbx_receive_if(&(fifo[minor].mbx), buf, count, 0);
 }
 
 /**
@@ -1218,7 +1214,7 @@ int rtf_get_if(unsigned int minor, void *buf, int count)
  * @retval 0 on success.
  * @retval EINVAL if @a fd_fifo refers to an invalid file descriptor or fifo.
  */
-int rtf_sem_init(unsigned int minor, int value)
+RTAI_SYSCALL_MODE int rtf_sem_init(unsigned int minor, int value)
 {
 	VALID_FIFO;
 
@@ -1246,7 +1242,7 @@ int rtf_sem_init(unsigned int minor, int value)
  * @retval 0 on success.
  * @retval EINVAL if @a fd_fifo refers to an invalid file descriptor or fifo.
  */
-int rtf_sem_post(unsigned int minor)
+RTAI_SYSCALL_MODE int rtf_sem_post(unsigned int minor)
 {
 	VALID_FIFO;
 
@@ -1275,7 +1271,7 @@ int rtf_sem_post(unsigned int minor)
  * @retval 0 on success.
  * @retval EINVAL if @a fd_fifo  refers to an invalid file descriptor or fifo.
  */
-int rtf_sem_trywait(unsigned int minor)
+RTAI_SYSCALL_MODE int rtf_sem_trywait(unsigned int minor)
 {
 	VALID_FIFO;
 
@@ -1302,7 +1298,7 @@ int rtf_sem_trywait(unsigned int minor)
  * @retval 0 on sucess.
  * @retval EINVAL if @a fd_fifo refers to an invalid file descriptor or fifo.
  */
-int rtf_sem_destroy(unsigned int minor)
+RTAI_SYSCALL_MODE int rtf_sem_destroy(unsigned int minor)
 {
 	VALID_FIFO;
 
@@ -1366,7 +1362,7 @@ static ssize_t rtf_read(struct file *filp, char *buf, size_t count, loff_t* ppos
 
 	if (count) {
 		inode->i_atime = CURRENT_TIME;
-		if ((handler_ret = ((int (*)(int, ...))(fifo[minor].handler))(minor, 'r')) < 0) {
+		if ((handler_ret = (fifo[minor].handler)(minor, 'r')) < 0) {
 			return handler_ret;
 		}
 	}
@@ -1392,7 +1388,7 @@ static ssize_t rtf_write(struct file *filp, const char *buf, size_t count, loff_
 	}
 
 	inode->i_ctime = inode->i_mtime = CURRENT_TIME;
-	if ((handler_ret = ((int (*)(int, ...))(fifo[minor].handler))(minor, 'w')) < 0) {
+	if ((handler_ret = (fifo[minor].handler)(minor, 'w')) < 0) {
 		return handler_ret;
 	}
 
@@ -1433,11 +1429,12 @@ static int rtf_ioctl(struct inode *inode, struct file *filp, unsigned int cmd, u
 			struct { char *buf; int count; } args;
 			int handler_ret;
 			TRACE_RTAI_FIFO(TRACE_RTAI_EV_FIFO_READ_ALLATONCE, 0, 0);
-			copy_from_user(&args, (void *)arg, sizeof(args));
+			rt_copy_from_user(&args, (void *)arg, sizeof(args));
 			args.count -= mbx_receive(&(fifop->mbx), args.buf, args.count, 1);
 			if (args.count) {
 				inode->i_atime = CURRENT_TIME;
-				if ((handler_ret = ((int (*)(int, ...))(fifo[minor].handler))(minor, 'r')) < 0) {
+				if ((handler_ret = (fifo[minor].handler)(minor,
+'r')) < 0) {
 					return handler_ret;
 				}
 				return args.count;
@@ -1446,13 +1443,13 @@ static int rtf_ioctl(struct inode *inode, struct file *filp, unsigned int cmd, u
 		}
 		case EAVESDROP: {
 			struct { char *buf; int count; } args;
-			copy_from_user(&args, (void *)arg, sizeof(args));
+			rt_copy_from_user(&args, (void *)arg, sizeof(args));
 			return args.count - mbx_evdrp(&(fifop->mbx), (char **)&args.buf, args.count, 1);
 		}
 		case READ_TIMED: {
 			struct { char *buf; int count, delay; } args;
 			int handler_ret;
-			copy_from_user(&args, (void *)arg, sizeof(args));
+			rt_copy_from_user(&args, (void *)arg, sizeof(args));
 			TRACE_RTAI_FIFO(TRACE_RTAI_EV_FIFO_READ_TIMED, args.count, DELAY(args.delay));
 			if (!args.delay) {
 				args.count -= mbx_receive_wp(&(fifop->mbx), args.buf, args.count, 1);
@@ -1465,7 +1462,7 @@ static int rtf_ioctl(struct inode *inode, struct file *filp, unsigned int cmd, u
 			if (args.count) {
 				inode->i_atime = CURRENT_TIME;
 //				if ((handler_ret = (fifop->handler)(minor)) < 0) {
-				if ((handler_ret = ((int (*)(int, ...))(fifo[minor].handler))(minor, 'r')) < 0) {
+				if ((handler_ret = (fifop->handler)(minor, 'r')) < 0) {
 					return handler_ret;
 				}
 				return args.count;
@@ -1475,11 +1472,11 @@ static int rtf_ioctl(struct inode *inode, struct file *filp, unsigned int cmd, u
 		case READ_IF: {
 			struct { char *buf; int count; } args;
 			int handler_ret;
-			copy_from_user(&args, (void *)arg, sizeof(args));
+			rt_copy_from_user(&args, (void *)arg, sizeof(args));
 			args.count -= mbx_receive_if(&(fifop->mbx), args.buf, args.count, 1);
 			if (args.count) {
 				inode->i_atime = CURRENT_TIME;
-				if ((handler_ret = ((int (*)(int, ...))(fifo[minor].handler))(minor, 'r')) < 0) {
+				if ((handler_ret = (fifo[minor].handler)(minor, 'r')) < 0) {
 					return handler_ret;
 				}
 				return args.count;
@@ -1489,7 +1486,7 @@ static int rtf_ioctl(struct inode *inode, struct file *filp, unsigned int cmd, u
 		case WRITE_TIMED: {
 			struct { char *buf; int count, delay; } args;
 			int handler_ret;
-			copy_from_user(&args, (void *)arg, sizeof(args));
+			rt_copy_from_user(&args, (void *)arg, sizeof(args));
 			TRACE_RTAI_FIFO(TRACE_RTAI_EV_FIFO_WRITE_TIMED, args.count, DELAY(args.delay));
 			if (!args.delay) {
 				args.count -= mbx_send_wp(&(fifop->mbx), args.buf, args.count, 1);
@@ -1501,7 +1498,7 @@ static int rtf_ioctl(struct inode *inode, struct file *filp, unsigned int cmd, u
 			}
 			inode->i_ctime = inode->i_mtime = CURRENT_TIME;
 //			if ((handler_ret = (fifop->handler)(minor)) < 0) {
-			if ((handler_ret = ((int (*)(int, ...))(fifo[minor].handler))(minor, 'w')) < 0) {
+			if ((handler_ret = (fifop->handler)(minor, 'w')) < 0) {
 				return handler_ret;
 			}
 			return args.count;
@@ -1509,7 +1506,7 @@ static int rtf_ioctl(struct inode *inode, struct file *filp, unsigned int cmd, u
 		case WRITE_IF: {
 			struct { char *buf; int count, delay; } args;
 			int handler_ret;
-			copy_from_user(&args, (void *)arg, sizeof(args));
+			rt_copy_from_user(&args, (void *)arg, sizeof(args));
 			if (args.count) {
 				args.count -= mbx_send_wp(&(fifop->mbx), args.buf, args.count, 1);
 				if (!args.count) {
@@ -1517,14 +1514,15 @@ static int rtf_ioctl(struct inode *inode, struct file *filp, unsigned int cmd, u
 				}
 			}
 			inode->i_ctime = inode->i_mtime = CURRENT_TIME;
-			if ((handler_ret = ((int (*)(int, ...))(fifo[minor].handler))(minor, 'w')) < 0) {
+			if ((handler_ret = (fifo[minor].handler)(minor, 'w')) <
+0) {
 				return handler_ret;
 			}
 			return args.count;
 		}
 		case OVRWRITE: {
 			struct { char *buf; int count; } args;
-			copy_from_user(&args, (void *)arg, sizeof(args));
+			rt_copy_from_user(&args, (void *)arg, sizeof(args));
 			return mbx_ovrwr_send(&(fifop->mbx), (char **)&args.buf, args.count, 1);
 		}
 		case RTF_SEM_INIT: {
@@ -1573,7 +1571,7 @@ static int rtf_ioctl(struct inode *inode, struct file *filp, unsigned int cmd, u
 			struct rt_fifo_get_info_struct req;
 			int i, n;
 
-			copy_from_user(&req, (void *)arg, sizeof(req));
+			rt_copy_from_user(&req, (void *)arg, sizeof(req));
 			for ( i = req.fifo, n = 0; 
 			      i < MAX_FIFOS && n < req.n; 
 			      i++, n++
@@ -1584,26 +1582,26 @@ static int rtf_ioctl(struct inode *inode, struct file *filp, unsigned int cmd, u
 				info.size        = fifo[i].mbx.size;
 				info.opncnt      = fifo[i].opncnt;
 				strncpy(info.name, fifo[i].name, RTF_NAMELEN+1);
-				copy_to_user(req.ptr + n, &info, sizeof(info));
+				rt_copy_to_user(req.ptr + n, &info, sizeof(info));
 			}
 			return n;
 		}
 		case RTF_NAMED_CREATE: {
 			struct { char name[RTF_NAMELEN+1]; int size; } args;
 
-			copy_from_user(&args, (void *)arg, sizeof(args));
+			rt_copy_from_user(&args, (void *)arg, sizeof(args));
 			return rtf_named_create(args.name, args.size);
 	        }
 		case RTF_CREATE_NAMED: {
 			char name[RTF_NAMELEN+1];
 
-			copy_from_user(name, (void *)arg, RTF_NAMELEN+1);
+			rt_copy_from_user(name, (void *)arg, RTF_NAMELEN+1);
 			return rtf_create_named(name);
 	        }
 		case RTF_NAME_LOOKUP: {
 			char name[RTF_NAMELEN+1];
 
-			copy_from_user(name, (void *)arg, RTF_NAMELEN+1);
+			rt_copy_from_user(name, (void *)arg, RTF_NAMELEN+1);
 			return rtf_getfifobyname(name);
 		}
 	        case TCGETS:
@@ -1663,7 +1661,11 @@ static devfs_handle_t devfs_handle;
 #endif
 
 static int MaxFifos = MAX_FIFOS;
-MODULE_PARM(MaxFifos, "i");
+RTAI_MODULE_PARM(MaxFifos, int);
+
+#define LXRTEXT  /* undef it to allow using fifos without any RTAI scheduler */
+
+#ifdef LXRTEXT
 
 static struct rt_fun_entry rtai_fifos_fun[] = {
 	[_CREATE]       = { 0, rtf_create },
@@ -1685,73 +1687,60 @@ static struct rt_fun_entry rtai_fifos_fun[] = {
 
 static int register_lxrt_fifos_support(void)
 {
-	RT_TASK *rt_linux_tasks[NR_RT_CPUS];
-	rt_base_linux_task = rt_get_base_linux_task(rt_linux_tasks);
-	if(rt_base_linux_task->task_trap_handler[0]) {
-		if(((int (*)(void *, int))rt_base_linux_task->task_trap_handler[0])(rtai_fifos_fun, FUN_FIFOS_LXRT_INDX)) {
-			printk("LXRT EXTENSION SLOT FOR FIFOS (%d) ALREADY USED\n", FUN_FIFOS_LXRT_INDX);
-			return -EACCES;
-		}
+	if (set_rt_fun_ext_index(rtai_fifos_fun, FUN_FIFOS_LXRT_INDX)) {
+		printk("LXRT EXTENSION SLOT FOR FIFOS (%d) ALREADY USED\n", FUN_FIFOS_LXRT_INDX);
+		return -EACCES;
 	}
-	return(0);
+	return 0;
 }
 
 static void unregister_lxrt_fifos_support(void)
 {
-	if(rt_base_linux_task->task_trap_handler[1]) {
-		((int (*)(void *, int))rt_base_linux_task->task_trap_handler[1])(rtai_fifos_fun, FUN_FIFOS_LXRT_INDX);
-	}
+	reset_rt_fun_ext_index(rtai_fifos_fun, FUN_FIFOS_LXRT_INDX);
 }
 
+#else
+
+static int register_lxrt_fifos_support(void) { return 0; }
+#define unregister_lxrt_fifos_support()
+
+#endif
+
+#define USE_UDEV_CLASS 1
+
+#if USE_UDEV_CLASS
+static class_t *fifo_class = NULL;
+#endif
+
 int __rtai_fifos_init(void)
 {
 	int minor;
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0) || !CONFIG_DEVFS_FS
-	if (register_chrdev(RTAI_FIFOS_MAJOR,"rtai_fifo",&rtf_fops)) {
-#else /* LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0) && CONFIG_DEVFS_FS */
-	if (devfs_register_chrdev(RTAI_FIFOS_MAJOR,"rtai_fifo",&rtf_fops)) {
-#endif  /* LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0) || !CONFIG_DEVFS_FS */
+	if (!(fifo = (FIFO *)kmalloc(MaxFifos*sizeof(FIFO), GFP_KERNEL))) {
+		printk("RTAI-FIFO: cannot allocate memory for FIFOS structure.\n");
+		return -ENOSPC;
+	}
+	memset(fifo, 0, MaxFifos*sizeof(FIFO));
+
+#if USE_UDEV_CLASS
+	if ((fifo_class = class_create(THIS_MODULE, "rtai_fifos")) == NULL) {
+		printk("RTAI-FIFO: cannot create class.\n");
+		return -EBUSY;
+	}
+	for (minor = 0; minor < MAX_FIFOS; minor++) {
+		if (CLASS_DEVICE_CREATE(fifo_class, MKDEV(RTAI_FIFOS_MAJOR, minor), NULL, "rtf%d", minor) == NULL) {
+			printk("RTAI-FIFO: cannot attach class.\n");
+			class_destroy(fifo_class);
+                        return -EBUSY;
+		}
+	}
+#endif
+
+	if (register_chrdev(RTAI_FIFOS_MAJOR, "rtai_fifo", &rtf_fops)) {
 		printk("RTAI-FIFO: cannot register major %d.\n", RTAI_FIFOS_MAJOR);
 		return -EIO;
 	}
 
-#ifdef CONFIG_DEVFS_FS
-
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
-	for (minor = 0; minor < MAX_FIFOS; minor++)
-	    if (devfs_mk_cdev(MKDEV(RTAI_FIFOS_MAJOR,minor),
-			      S_IFCHR|S_IRUSR|S_IWUSR,
-			      "rtf/%u",
-			      minor) < 0)
-		{
-		printk("RTAI-FIFO: cannot create devfs entry rtf/%u.\n",minor);
-		devfs_remove("rtf");
-		unregister_chrdev(RTAI_FIFOS_MAJOR,"rtai_fifo");
-		return -EIO;
-		}
-#else /* LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0) */
-	devfs_handle = devfs_mk_dir(NULL, "rtf", NULL);
-
-	if(!devfs_handle)
-	    {
-	    printk("RTAI-FIFO: cannot create devfs dir entry.\n");
-	    return -EIO;
-	    }
-
-	devfs_register_series(devfs_handle,
-			      "%u",
-			      MAX_FIFOS,
-			      DEVFS_FL_DEFAULT,
-			      RTAI_FIFOS_MAJOR,
-			      0,
-			      S_IFCHR|S_IRUGO|S_IWUGO,
-			      &rtf_fops,
-			      NULL);
-#endif /* LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0) */
-
-#endif /* CONFIG_DEVFS_FS */
-
 	if ((fifo_srq = rtf_request_srq(rtf_sysrq_handler)) < 0) {
 		printk("RTAI-FIFO: no srq available in rtai.\n");
 		return fifo_srq;
@@ -1759,12 +1748,6 @@ int __rtai_fifos_init(void)
 	taskq.in = taskq.out = pol_asyn_q.in = pol_asyn_q.out = 0;
 	async_sig = SIGIO;
 
-	if (!(fifo = (FIFO *)kmalloc(MaxFifos*sizeof(FIFO), GFP_KERNEL))) {
-		printk("RTAI-FIFO: cannot allocate memory for FIFOS structure.\n");
-		return -ENOSPC;
-	}
-	memset(fifo, 0, MaxFifos*sizeof(FIFO));
-
 	for (minor = 0; minor < MAX_FIFOS; minor++) {
 		fifo[minor].opncnt = fifo[minor].pol_asyn_pended = 0;
 		init_waitqueue_head(&fifo[minor].pollq);
@@ -1780,18 +1763,17 @@ int __rtai_fifos_init(void)
 void __rtai_fifos_exit(void)
 {
 	unregister_lxrt_fifos_support();
+	unregister_chrdev(RTAI_FIFOS_MAJOR, "rtai_fifo");
 
-#if CONFIG_DEVFS_FS
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
-	devfs_remove("rtf");
-	unregister_chrdev(RTAI_FIFOS_MAJOR,"rtai_fifo");
-#else /* LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0) */
-	devfs_unregister(devfs_handle);
-	devfs_unregister_chrdev(RTAI_FIFOS_MAJOR,"rtai_fifo");
-#endif  /* LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0) || !CONFIG_DEVFS_FS */
-#else /* !CONFIG_DEVFS_FS */
-	unregister_chrdev(RTAI_FIFOS_MAJOR,"rtai_fifo");
-#endif /* CONFIG_DEVFS_FS */
+#if USE_UDEV_CLASS
+{
+	int minor;
+	for (minor = 0; minor < MAX_FIFOS; minor++) {
+		class_device_destroy(fifo_class, MKDEV(RTAI_FIFOS_MAJOR, minor));
+	}
+	class_destroy(fifo_class);
+}
+#endif
 
 	if (rtf_free_srq(fifo_srq) < 0) {
 		printk("RTAI-FIFO: rtai srq %d illegal or already free.\n", fifo_srq);
@@ -1910,12 +1892,12 @@ int rtf_named_create(const char *name, int size)
 	return -EBUSY;
 }
 
-int rtf_create_named(const char *name)
+RTAI_SYSCALL_MODE int rtf_create_named(const char *name)
 {
 	return rtf_named_create(name, DEFAULT_SIZE);
 }
 
-int rtf_getfifobyname(const char *name)
+RTAI_SYSCALL_MODE int rtf_getfifobyname(const char *name)
 {
     	int minor;
 
