@@ -3,7 +3,7 @@
  * Mailbox functions.
  * @author Paolo Mantegazza
  *
- * @note Copyright (C) 1999-2003 Paolo Mantegazza
+ * @note Copyright (C) 1999-2006 Paolo Mantegazza
  * <mantegazza@aero.polimi.it> 
  *
  * This program is free software; you can redistribute it and/or
@@ -28,73 +28,43 @@
  * @defgroup mbx Mailbox functions
  *@{*/
 
+#include <linux/kernel.h>
+#include <linux/module.h>
+
 #include <asm/uaccess.h>
 
 #include <rtai_schedcore.h>
+#include <rtai_prinher.h>
 
 MODULE_LICENSE("GPL");
 
 /* +++++++++++++++++++++++++++++ MAIL BOXES ++++++++++++++++++++++++++++++++ */
 
+#define _mbx_signal(mbx, blckdon) \
+do { \
+	unsigned long flags; \
+	RT_TASK *task; \
+	flags = rt_global_save_flags_and_cli(); \
+	if ((task = mbx->waiting_task)) { \
+		rem_timed_task(task); \
+		task->blocked_on  = blckdon; \
+		mbx->waiting_task = NULL; \
+		if (task->state != RT_SCHED_READY && (task->state &= ~(RT_SCHED_MBXSUSP | RT_SCHED_DELAYED)) == RT_SCHED_READY) { \
+			enq_ready_task(task); \
+			RT_SCHEDULE(task, rtai_cpuid()); \
+		} \
+	} \
+	rt_global_restore_flags(flags); \
+} while (0)
+
+static void mbx_delete_signal(MBX *mbx)
+{
+	_mbx_signal(mbx, RTP_OBJREM);
+}
+
 static void mbx_signal(MBX *mbx)
 {
-	unsigned long flags;
-	RT_TASK *task;
-	int tosched;
-
-	flags = rt_global_save_flags_and_cli();
-	if ((task = mbx->waiting_task)) {
-		rem_timed_task(task);
-		mbx->waiting_task = NOTHING;
-		task->prio_passed_to = NOTHING;
-		if (task->state != RT_SCHED_READY && (task->state &= ~(RT_SCHED_MBXSUSP | RT_SCHED_DELAYED)) == RT_SCHED_READY) {
-			enq_ready_task(task);
-			if (mbx->sndsem.type <= 0) {
-				RT_SCHEDULE(task, rtai_cpuid());
-				rt_global_restore_flags(flags);
-				return;
-			}
-			tosched = 1;
-			goto res;
-		}
-	}
-	tosched = 0;
-res:	if (mbx->sndsem.type > 0) {
-		DECLARE_RT_CURRENT;
-		int sched;
-		ASSIGN_RT_CURRENT;
-		mbx->owndby = 0;
-		if (rt_current->owndres & SEMHLF) {
-			--rt_current->owndres;
-		}
-		if (!rt_current->owndres) {
-			sched = renq_current(rt_current, rt_current->base_priority);
-		} else if (!(rt_current->owndres & SEMHLF)) {
-			int priority;
-			sched = renq_current(rt_current, rt_current->base_priority > (priority = ((rt_current->msg_queue.next)->task)->priority) ? priority : rt_current->base_priority);
-		} else {
-			sched = 0;
-		}
-		if (rt_current->suspdepth) {
-			if (rt_current->suspdepth > 0) {
-				rt_current->state |= RT_SCHED_SUSPENDED;
-				rem_ready_current(rt_current);
-                        	sched = 1;
-			} else {
-				rt_task_delete(rt_current);
-			}
-		}
-		if (sched) {
-			if (tosched) {
-				RT_SCHEDULE_BOTH(task, cpuid);
-			} else {
-				rt_schedule();
-			}
-		} else if (tosched) {
-			RT_SCHEDULE(task, cpuid);
-		}
-	}
-	rt_global_restore_flags(flags);
+	_mbx_signal(mbx, NULL);
 }
 
 static int mbx_wait(MBX *mbx, int *fravbs, RT_TASK *rt_current)
@@ -103,25 +73,18 @@ static int mbx_wait(MBX *mbx, int *fravbs, RT_TASK *rt_current)
 
 	flags = rt_global_save_flags_and_cli();
 	if (!(*fravbs)) {
-		unsigned long schedmap;
-		if (mbx->sndsem.type > 0) {
-			schedmap = pass_prio(mbx->owndby, rt_current);
-		} else {
-			schedmap = 0;
-		}
+		unsigned long retval;
 		rt_current->state |= RT_SCHED_MBXSUSP;
 		rem_ready_current(rt_current);
+		rt_current->blocked_on = (void *)mbx;
 		mbx->waiting_task = rt_current;
-		RT_SCHEDULE_MAP_BOTH(schedmap);
-		if (mbx->waiting_task == rt_current || mbx->magic != RT_MBX_MAGIC) {
-			rt_current->prio_passed_to = NOTHING;
+		rt_schedule();
+		if (unlikely(retval = (unsigned long)rt_current->blocked_on)) {
+			mbx->waiting_task = NULL;
 			rt_global_restore_flags(flags);
-			return -1;
+			return retval;
 		}
 	}
-	if (mbx->sndsem.type > 0) {
-		(mbx->owndby = rt_current)->owndres++;
-	}
 	rt_global_restore_flags(flags);
 	return 0;
 }
@@ -132,33 +95,20 @@ static int mbx_wait_until(MBX *mbx, int *fravbs, RTIME time, RT_TASK *rt_current
 
 	flags = rt_global_save_flags_and_cli();
 	if (!(*fravbs)) {
+		void *retp;
+		rt_current->blocked_on = (void *)mbx;
 		mbx->waiting_task = rt_current;
 		if ((rt_current->resume_time = time) > rt_smp_time_h[rtai_cpuid()]) {
-			unsigned long schedmap;
-			if (mbx->sndsem.type > 0) {
-				schedmap = pass_prio(mbx->owndby, rt_current);
-			} else {
-				schedmap = 0;
-			}
 			rt_current->state |= (RT_SCHED_MBXSUSP | RT_SCHED_DELAYED);
 			rem_ready_current(rt_current);
 			enq_timed_task(rt_current);
-			RT_SCHEDULE_MAP_BOTH(schedmap);
+			rt_schedule();
 		}
-		if (mbx->magic != RT_MBX_MAGIC) {
-			rt_current->prio_passed_to = NOTHING;
+		if (unlikely((retp = rt_current->blocked_on) != NULL)) {
+			mbx->waiting_task = NULL;
 			rt_global_restore_flags(flags);
-			return -1;
+			return likely(retp > RTP_HIGERR) ? RTE_TIMOUT : (retp == RTP_UNBLKD ? RTE_UNBLKD : RTE_OBJREM);
 		}
-		if (mbx->waiting_task == rt_current) {
-			mbx->waiting_task = NOTHING;
-			rt_current->prio_passed_to = NOTHING;
-			rt_global_restore_flags(flags);
-			return -1;
-		}
-	}
-	if (mbx->sndsem.type > 0) {
-		(mbx->owndby = rt_current)->owndres++;
 	}
 	rt_global_restore_flags(flags);
 	return 0;
@@ -181,7 +131,7 @@ static int mbxput(MBX *mbx, char **msg, int msg_size, int space)
 		if (space) {
 			memcpy(mbx->bufadr + mbx->lbyte, *msg, tocpy);
 		} else {
-			copy_from_user(mbx->bufadr + mbx->lbyte, *msg, tocpy);
+			rt_copy_from_user(mbx->bufadr + mbx->lbyte, *msg, tocpy);
 		}
 		flags = rt_spin_lock_irqsave(&(mbx->lock));
 		mbx->frbs -= tocpy;
@@ -214,9 +164,9 @@ static int mbxovrwrput(MBX *mbx, char **msg, int msg_size, int space)
 			if (space) {
 				memcpy(mbx->bufadr + mbx->lbyte, *msg, tocpy);
 			} else {
-				copy_from_user(mbx->bufadr + mbx->lbyte, *msg, tocpy);
+				rt_copy_from_user(mbx->bufadr + mbx->lbyte, *msg, tocpy);
 			}
-	        	flags = rt_spin_lock_irqsave(&(mbx->lock));
+			flags = rt_spin_lock_irqsave(&(mbx->lock));
 			mbx->frbs -= tocpy;
 			mbx->avbs += tocpy;
         		rt_spin_unlock_irqrestore(flags, &(mbx->lock));
@@ -258,7 +208,7 @@ static int mbxget(MBX *mbx, char **msg, int msg_size, int space)
 		if (space) {
 			memcpy(*msg, mbx->bufadr + mbx->fbyte, tocpy);
 		} else {
-			copy_to_user(*msg, mbx->bufadr + mbx->fbyte, tocpy);
+			rt_copy_to_user(*msg, mbx->bufadr + mbx->fbyte, tocpy);
 		}
 		flags = rt_spin_lock_irqsave(&(mbx->lock));
 		mbx->frbs  += tocpy;
@@ -287,7 +237,7 @@ static int mbxevdrp(MBX *mbx, char **msg, int msg_size, int space)
 		if (space) {
 			memcpy(*msg, mbx->bufadr + fbyte, tocpy);
 		} else {
-			copy_to_user(*msg, mbx->bufadr + mbx->fbyte, tocpy);
+			rt_copy_to_user(*msg, mbx->bufadr + mbx->fbyte, tocpy);
 		}
 		avbs     -= tocpy;
 		msg_size -= tocpy;
@@ -316,14 +266,17 @@ static int mbxevdrp(MBX *mbx, char **msg, int msg_size, int space)
  *
  * @return The number of bytes not received is returned.
  */
-int _rt_mbx_evdrp(MBX *mbx, void *msg, int msg_size, int space)
+RTAI_SYSCALL_MODE int _rt_mbx_evdrp(MBX *mbx, void *msg, int msg_size, int space)
 {
 	return mbxevdrp(mbx, (char **)(&msg), msg_size, space);
 }
 
 
-#define CHK_MBX_MAGIC { if (mbx->magic != RT_MBX_MAGIC) { return -EINVAL; } }
+#define CHK_MBX_MAGIC { if (mbx->magic != RT_MBX_MAGIC) \
+	{ return CONFIG_RTAI_USE_NEWERR ? RTE_OBJINV : -EINVAL; } }
 
+#define MBX_RET(msg_size, retval) \
+	(CONFIG_RTAI_USE_NEWERR ? retval : msg_size)
 
 /**
  * @brief Initializes a fully typed mailbox queueing tasks
@@ -345,16 +298,16 @@ int _rt_mbx_evdrp(MBX *mbx, void *msg, int msg_size, int space)
  *
  * See also: notes under rt_mbx_init().
  */
-int rt_typed_mbx_init(MBX *mbx, int size, int type)
+RTAI_SYSCALL_MODE int rt_typed_mbx_init(MBX *mbx, int size, int type)
 {
-	if (!(mbx->bufadr = sched_malloc(size))) { 
+	if (!(mbx->bufadr = rt_malloc(size))) { 
 		return -ENOMEM;
 	}
 	rt_typed_sem_init(&(mbx->sndsem), 1, type & 3 ? type : BIN_SEM | type);
 	rt_typed_sem_init(&(mbx->rcvsem), 1, type & 3 ? type : BIN_SEM | type);
 	mbx->magic = RT_MBX_MAGIC;
 	mbx->size = mbx->frbs = size;
-	mbx->waiting_task = mbx->owndby = 0;
+	mbx->owndby = mbx->waiting_task = NULL;
 	mbx->fbyte = mbx->lbyte = mbx->avbs = 0;
         spin_lock_init(&(mbx->lock));
 	return 0;
@@ -414,7 +367,7 @@ int rt_mbx_init(MBX *mbx, int size)
  * - @b EINVAL: @e mbx points to an invalid mailbox.
  * - @b EFAULT: mailbox data were found in an invalid state.
  */
-int rt_mbx_delete(MBX *mbx)
+RTAI_SYSCALL_MODE int rt_mbx_delete(MBX *mbx)
 {
 	CHK_MBX_MAGIC;
 	mbx->magic = 0;
@@ -422,9 +375,9 @@ int rt_mbx_delete(MBX *mbx)
 		return -EFAULT;
 	}
 	while (mbx->waiting_task) {
-		mbx_signal(mbx);
+		mbx_delete_signal(mbx);
 	}
-	sched_free(mbx->bufadr); 
+	rt_free(mbx->bufadr); 
 	return 0;
 }
 
@@ -451,18 +404,19 @@ int rt_mbx_delete(MBX *mbx)
  *   in the queueing of all sending tasks.
  * - @b EINVAL: mbx points to an invalid mailbox.
  */
-int _rt_mbx_send(MBX *mbx, void *msg, int msg_size, int space)
+RTAI_SYSCALL_MODE int _rt_mbx_send(MBX *mbx, void *msg, int msg_size, int space)
 {
 	RT_TASK *rt_current = RT_CURRENT;
+	int retval;
 
 	CHK_MBX_MAGIC;
-	if (rt_sem_wait(&mbx->sndsem) > 1) {
-		return msg_size;
+	if ((retval = rt_sem_wait(&mbx->sndsem)) > 1) {
+		return MBX_RET(msg_size, retval);
 	}
 	while (msg_size) {
-		if (mbx_wait(mbx, &mbx->frbs, rt_current)) {
+		if ((retval = mbx_wait(mbx, &mbx->frbs, rt_current))) {
 			rt_sem_signal(&mbx->sndsem);
-			return msg_size;
+			return MBX_RET(msg_size, retval);
 		}
 		msg_size = mbxput(mbx, (char **)(&msg), msg_size, space);
 		mbx_signal(mbx);
@@ -488,17 +442,18 @@ int _rt_mbx_send(MBX *mbx, void *msg, int msg_size, int space)
  * failure a negative value is returned as described below:
  * - @b EINVAL: @e mbx points to an invalid mailbox.
  */
-int _rt_mbx_send_wp(MBX *mbx, void *msg, int msg_size, int space)
+RTAI_SYSCALL_MODE int _rt_mbx_send_wp(MBX *mbx, void *msg, int msg_size, int space)
 {
 	unsigned long flags;
 	RT_TASK *rt_current = RT_CURRENT;
 
 	CHK_MBX_MAGIC;
 	flags = rt_global_save_flags_and_cli();
-	if (mbx->sndsem.count && mbx->frbs) {
+	if (mbx->sndsem.count > 0 && mbx->frbs) {
 		mbx->sndsem.count = 0;
 		if (mbx->sndsem.type > 0) {
-			(mbx->sndsem.owndby = mbx->owndby = rt_current)->owndres += 2;
+			mbx->sndsem.owndby = rt_current;
+			enqueue_resqel(&mbx->sndsem.resq, rt_current);
 		}
 		rt_global_restore_flags(flags);
 		msg_size = mbxput(mbx, (char **)(&msg), msg_size, space);
@@ -524,17 +479,18 @@ int _rt_mbx_send_wp(MBX *mbx, void *msg, int msg_size, int space)
  * below:
  * - @b EINVAL: @e mbx points to an invalid mailbox.
  */
-int _rt_mbx_send_if(MBX *mbx, void *msg, int msg_size, int space)
+RTAI_SYSCALL_MODE int _rt_mbx_send_if(MBX *mbx, void *msg, int msg_size, int space)
 {
 	unsigned long flags;
 	RT_TASK *rt_current = RT_CURRENT;
 
 	CHK_MBX_MAGIC;
 	flags = rt_global_save_flags_and_cli();
-	if (mbx->sndsem.count && msg_size <= mbx->frbs) {
+	if (mbx->sndsem.count > 0 && msg_size <= mbx->frbs) {
 		mbx->sndsem.count = 0;
 		if (mbx->sndsem.type > 0) {
-			(mbx->sndsem.owndby = mbx->owndby = rt_current)->owndres += 2;
+			mbx->sndsem.owndby = rt_current;
+			enqueue_resqel(&mbx->sndsem.resq, rt_current);
 		}
 		rt_global_restore_flags(flags);
 		mbxput(mbx, (char **)(&msg), msg_size, space);
@@ -570,17 +526,19 @@ int _rt_mbx_send_if(MBX *mbx, void *msg, int msg_size, int space)
  *
  * See also: notes under @ref _rt_mbx_send_timed().
  */
-int _rt_mbx_send_until(MBX *mbx, void *msg, int msg_size, RTIME time, int space)
+RTAI_SYSCALL_MODE int _rt_mbx_send_until(MBX *mbx, void *msg, int msg_size, RTIME time, int space)
 {
 	RT_TASK *rt_current = RT_CURRENT;
+	int retval;
+
 	CHK_MBX_MAGIC;
-	if (rt_sem_wait_until(&mbx->sndsem, time) > 1) {
-		return msg_size;
+	if ((retval = rt_sem_wait_until(&mbx->sndsem, time)) > 1) {
+		return MBX_RET(msg_size, retval);
 	}
 	while (msg_size) {
-		if (mbx_wait_until(mbx, &mbx->frbs, time, rt_current)) {
+		if ((retval = mbx_wait_until(mbx, &mbx->frbs, time, rt_current))) {
 			rt_sem_signal(&mbx->sndsem);
-			return msg_size;
+			return MBX_RET(msg_size, retval);
 		}
 		msg_size = mbxput(mbx, (char **)(&msg), msg_size, space);
 		mbx_signal(mbx);
@@ -613,7 +571,7 @@ int _rt_mbx_send_until(MBX *mbx, void *msg, int msg_size, RTIME time, int space)
  *
  * See also: notes under @ref _rt_mbx_send_until().
  */
-int _rt_mbx_send_timed(MBX *mbx, void *msg, int msg_size, RTIME delay, int space)
+RTAI_SYSCALL_MODE int _rt_mbx_send_timed(MBX *mbx, void *msg, int msg_size, RTIME delay, int space)
 {
 	return _rt_mbx_send_until(mbx, msg, msg_size, get_time() + delay, space);
 }
@@ -638,17 +596,19 @@ int _rt_mbx_send_timed(MBX *mbx, void *msg, int msg_size, RTIME delay, int space
  *   in the queueing of all receiving tasks.
  * - @b EINVAL: mbx points to an invalid mailbox.
  */
-int _rt_mbx_receive(MBX *mbx, void *msg, int msg_size, int space)
+RTAI_SYSCALL_MODE int _rt_mbx_receive(MBX *mbx, void *msg, int msg_size, int space)
 {
 	RT_TASK *rt_current = RT_CURRENT;
+	int retval;
+
 	CHK_MBX_MAGIC;
-	if (rt_sem_wait(&mbx->rcvsem) > 1) {
+	if ((retval = rt_sem_wait(&mbx->rcvsem)) > 1) {
 		return msg_size;
 	}
 	while (msg_size) {
-		if (mbx_wait(mbx, &mbx->avbs, rt_current)) {
+		if ((retval = mbx_wait(mbx, &mbx->avbs, rt_current))) {
 			rt_sem_signal(&mbx->rcvsem);
-			return msg_size;
+			return MBX_RET(msg_size, retval);
 		}
 		msg_size = mbxget(mbx, (char **)(&msg), msg_size, space);
 		mbx_signal(mbx);
@@ -675,17 +635,18 @@ int _rt_mbx_receive(MBX *mbx, void *msg, int msg_size, int space)
  * failure a negative value is returned as described below:
  * - @b EINVAL: mbx points to not a valid mailbox.
  */
-int _rt_mbx_receive_wp(MBX *mbx, void *msg, int msg_size, int space)
+RTAI_SYSCALL_MODE int _rt_mbx_receive_wp(MBX *mbx, void *msg, int msg_size, int space)
 {
 	unsigned long flags;
 	RT_TASK *rt_current = RT_CURRENT;
 
 	CHK_MBX_MAGIC;
 	flags = rt_global_save_flags_and_cli();
-	if (mbx->rcvsem.count && mbx->avbs) {
+	if (mbx->rcvsem.count > 0 && mbx->avbs) {
 		mbx->rcvsem.count = 0;
 		if (mbx->rcvsem.type > 0) {
-			(mbx->rcvsem.owndby = mbx->owndby = rt_current)->owndres += 2;
+			mbx->rcvsem.owndby = rt_current;
+			enqueue_resqel(&mbx->rcvsem.resq, rt_current);
 		}
 		rt_global_restore_flags(flags);
 		msg_size = mbxget(mbx, (char **)(&msg), msg_size, space);
@@ -716,17 +677,18 @@ int _rt_mbx_receive_wp(MBX *mbx, void *msg, int msg_size, int space)
  * below:
  * - @b EINVAL: mbx points to an invalid mailbox.
  */
-int _rt_mbx_receive_if(MBX *mbx, void *msg, int msg_size, int space)
+RTAI_SYSCALL_MODE int _rt_mbx_receive_if(MBX *mbx, void *msg, int msg_size, int space)
 {
 	unsigned long flags;
 	RT_TASK *rt_current = RT_CURRENT;
 
 	CHK_MBX_MAGIC;
 	flags = rt_global_save_flags_and_cli();
-	if (mbx->rcvsem.count && msg_size <= mbx->avbs) {
+	if (mbx->rcvsem.count > 0 && msg_size <= mbx->avbs) {
 		mbx->rcvsem.count = 0;
 		if (mbx->rcvsem.type > 0) {
-			(mbx->rcvsem.owndby = mbx->owndby = rt_current)->owndres += 2;
+			mbx->rcvsem.owndby = rt_current;
+			enqueue_resqel(&mbx->rcvsem.resq, rt_current);
 		}
 		rt_global_restore_flags(flags);
 		mbxget(mbx, (char **)(&msg), msg_size, space);
@@ -762,17 +724,19 @@ int _rt_mbx_receive_if(MBX *mbx, void *msg, int msg_size, int space)
  *
  * See also: notes under rt_mbx_received_timed().
  */
-int _rt_mbx_receive_until(MBX *mbx, void *msg, int msg_size, RTIME time, int space)
+RTAI_SYSCALL_MODE int _rt_mbx_receive_until(MBX *mbx, void *msg, int msg_size, RTIME time, int space)
 {
 	RT_TASK *rt_current = RT_CURRENT;
+	int retval;
+
 	CHK_MBX_MAGIC;
-	if (rt_sem_wait_until(&mbx->rcvsem, time) > 1) {
-		return msg_size;
+	if ((retval = rt_sem_wait_until(&mbx->rcvsem, time)) > 1) {
+		return MBX_RET(msg_size, retval);
 	}
 	while (msg_size) {
-		if (mbx_wait_until(mbx, &mbx->avbs, time, rt_current)) {
+		if ((retval = mbx_wait_until(mbx, &mbx->avbs, time, rt_current))) {
 			rt_sem_signal(&mbx->rcvsem);
-			return msg_size;
+			return MBX_RET(msg_size, retval);
 		}
 		msg_size = mbxget(mbx, (char **)(&msg), msg_size, space);
 		mbx_signal(mbx);
@@ -805,7 +769,7 @@ int _rt_mbx_receive_until(MBX *mbx, void *msg, int msg_size, RTIME time, int spa
  *
  * See also: notes under rt_mbx_received_until().
  */
-int _rt_mbx_receive_timed(MBX *mbx, void *msg, int msg_size, RTIME delay, int space)
+RTAI_SYSCALL_MODE int _rt_mbx_receive_timed(MBX *mbx, void *msg, int msg_size, RTIME delay, int space)
 {
 	return _rt_mbx_receive_until(mbx, msg, msg_size, get_time() + delay, space);
 }
@@ -824,7 +788,7 @@ int _rt_mbx_receive_timed(MBX *mbx, void *msg, int msg_size, RTIME delay, int sp
  * is returned as described below:
  * - @b EINVAL: @e mbx points to an invalid mailbox.
  */
-int _rt_mbx_ovrwr_send(MBX *mbx, void *msg, int msg_size, int space)
+RTAI_SYSCALL_MODE int _rt_mbx_ovrwr_send(MBX *mbx, void *msg, int msg_size, int space)
 {
 	unsigned long flags;
 	RT_TASK *rt_current = RT_CURRENT;
@@ -832,10 +796,11 @@ int _rt_mbx_ovrwr_send(MBX *mbx, void *msg, int msg_size, int space)
 	CHK_MBX_MAGIC;
 
 	flags = rt_global_save_flags_and_cli();
-	if (mbx->sndsem.count) {
+	if (mbx->sndsem.count > 0) {
 		mbx->sndsem.count = 0;
 		if (mbx->sndsem.type > 0) {
-			(mbx->sndsem.owndby = mbx->owndby = rt_current)->owndres += 2;
+			mbx->sndsem.owndby = rt_current;
+			enqueue_resqel(&mbx->sndsem.resq, rt_current);
 		}
 		rt_global_restore_flags(flags);
 		msg_size = mbxovrwrput(mbx, (char **)(&msg), msg_size, space);
@@ -874,7 +839,7 @@ int _rt_mbx_ovrwr_send(MBX *mbx, void *msg, int msg_size, int space)
  *
  * See also: notes under rt_mbx_init() and rt_typed_mbx_init().
  */
-MBX *_rt_typed_named_mbx_init(unsigned long mbx_name, int size, int qtype)
+RTAI_SYSCALL_MODE MBX *_rt_typed_named_mbx_init(unsigned long mbx_name, int size, int qtype)
 {
 	MBX *mbx;
 
@@ -913,7 +878,7 @@ MBX *_rt_typed_named_mbx_init(unsigned long mbx_name, int size, int qtype)
  *
  * See also: notes under rt_mbx_delete().
  */
-int rt_named_mbx_delete(MBX *mbx)
+RTAI_SYSCALL_MODE int rt_named_mbx_delete(MBX *mbx)
 {
 	int ret;
 	if (!(ret = rt_drg_on_adr_cnt(mbx))) {
@@ -933,20 +898,20 @@ struct rt_native_fun_entry rt_mbx_entries[] = {
 
 	{ { 0, rt_typed_mbx_init }, 	      	TYPED_MBX_INIT },
 	{ { 0, rt_mbx_delete }, 	      	MBX_DELETE },
-	{ { 0, _rt_typed_named_mbx_init },  	NAMED_MBX_INIT },
-	{ { 0, rt_named_mbx_delete },		NAMED_MBX_DELETE },
 	{ { 1, _rt_mbx_send }, 	       		MBX_SEND },
 	{ { 1, _rt_mbx_send_wp },      		MBX_SEND_WP },
 	{ { 1, _rt_mbx_send_if },      	 	MBX_SEND_IF },
 	{ { 1, _rt_mbx_send_until },    	MBX_SEND_UNTIL },
 	{ { 1, _rt_mbx_send_timed },    	MBX_SEND_TIMED },
+	{ { 1, _rt_mbx_ovrwr_send },    	MBX_OVRWR_SEND },
+        { { 1, _rt_mbx_evdrp },         	MBX_EVDRP },
 	{ { 1, _rt_mbx_receive },       	MBX_RECEIVE },
 	{ { 1, _rt_mbx_receive_wp },    	MBX_RECEIVE_WP },
 	{ { 1, _rt_mbx_receive_if },    	MBX_RECEIVE_IF },
 	{ { 1, _rt_mbx_receive_until }, 	MBX_RECEIVE_UNTIL },
 	{ { 1, _rt_mbx_receive_timed }, 	MBX_RECEIVE_TIMED },
-	{ { 1, _rt_mbx_ovrwr_send },    	MBX_OVRWR_SEND },
-        { { 1, _rt_mbx_evdrp },         	MBX_EVDRP },
+	{ { 0, _rt_typed_named_mbx_init },  	NAMED_MBX_INIT },
+	{ { 0, rt_named_mbx_delete },		NAMED_MBX_DELETE },
 	{ { 0, 0 },  		      	       	000 }
 };
 
