@@ -35,12 +35,40 @@
  * @defgroup rpc Remote procedure call functions
  */
 
+#include <linux/kernel.h>
+#include <linux/module.h>
+#include <linux/errno.h>
+#include <linux/version.h>
+
 #include <rtai_schedcore.h>
+#include <rtai_prinher.h>
 
 MODULE_LICENSE("GPL");
 
 /* +++++++++++++++++++++++ SHORT INTERTASK MESSAGES +++++++++++++++++++++++++ */
 
+#define TASK_INVAL  ((RT_TASK *)RTE_OBJINV)
+
+#define msg_not_sent() \
+do { \
+	void *retp; \
+	if ((retp = rt_current->blocked_on) != RTP_OBJREM) { \
+		set_task_prio_from_resq(task); \
+		dequeue_blocked(rt_current); \
+		task = (void *)(long)(CONFIG_RTAI_USE_NEWERR ? ((likely(retp > RTP_HIGERR) ? RTE_TIMOUT : RTE_UNBLKD)) : 0); \
+	} else { \
+		rt_current->prio_passed_to = NULL; \
+		task = (void *)(long)(CONFIG_RTAI_USE_NEWERR ? RTE_OBJREM : 0); \
+	} \
+	rt_current->msg_queue.task = rt_current; \
+} while (0)
+
+#define msg_not_received() \
+do { \
+	rt_current->ret_queue.task = NULL; \
+	task = (void *)(long)(CONFIG_RTAI_USE_NEWERR ? (((void *)rt_current->blocked_on != RTP_UNBLKD) ? RTE_TIMOUT : RTE_UNBLKD) : 0); \
+} while (0)
+
 /* +++++++++++++++++++++++++++++ ASYNC SENDS ++++++++++++++++++++++++++++++++ */
 
 /**
@@ -73,13 +101,13 @@ MODULE_LICENSE("GPL");
  * CPUs RTAI runs on 0xFFFF is not an address that can be used by any
  * RTAI task, so it is should be safe always.
  */
-RT_TASK *rt_send(RT_TASK *task, unsigned int msg)
+RTAI_SYSCALL_MODE RT_TASK *rt_send(RT_TASK *task, unsigned long msg)
 {
 	DECLARE_RT_CURRENT;
 	unsigned long flags;
 
 	if (task->magic != RT_TASK_MAGIC) {
-		return MSG_ERR;
+		return TASK_INVAL;
 	}
 
 	flags = rt_global_save_flags_and_cli();
@@ -88,7 +116,7 @@ RT_TASK *rt_send(RT_TASK *task, unsigned int msg)
 	      (!task->msg_queue.task || task->msg_queue.task == rt_current)) {
 		task->msg = msg;
 		task->msg_queue.task = rt_current;
-		task->ret_queue.task = NOTHING;
+		task->ret_queue.task = NULL;
 		rem_timed_task(task);
 		if (task->state != RT_SCHED_READY && (task->state &= ~(RT_SCHED_RECEIVE | RT_SCHED_DELAYED)) == RT_SCHED_READY) {
 			enq_ready_task(task);
@@ -99,12 +127,13 @@ RT_TASK *rt_send(RT_TASK *task, unsigned int msg)
 		rt_current->msg_queue.task = task;
 		enqueue_blocked(rt_current, &task->msg_queue, 0);
 		rt_current->state |= RT_SCHED_SEND;
+		enqueue_resqtsk(task);
+		pass_prio(task, rt_current);
 		rem_ready_current(rt_current);
 		rt_schedule();
 	}
 	if (rt_current->msg_queue.task != rt_current) {
-		rt_current->msg_queue.task = rt_current;
-		task = (RT_TASK *)0;
+		msg_not_sent();
 	}
 	rt_global_restore_flags(flags);
 	return task;
@@ -138,13 +167,13 @@ RT_TASK *rt_send(RT_TASK *task, unsigned int msg)
  * CPUs RTAI runs on 0xFFFF is not an address that can be used by any
  * RTAI task, so it is should be safe always. (FIXME)
  */
-RT_TASK *rt_send_if(RT_TASK *task, unsigned int msg)
+RTAI_SYSCALL_MODE RT_TASK *rt_send_if(RT_TASK *task, unsigned long msg)
 {
 	DECLARE_RT_CURRENT;
 	unsigned long flags;
 
 	if (task->magic != RT_TASK_MAGIC) {
-		return MSG_ERR;
+		return TASK_INVAL;
 	}
 
 	flags = rt_global_save_flags_and_cli();
@@ -153,7 +182,7 @@ RT_TASK *rt_send_if(RT_TASK *task, unsigned int msg)
 	      (!task->msg_queue.task || task->msg_queue.task == rt_current)) {
 		task->msg = msg;
 		task->msg_queue.task = rt_current;
-		task->ret_queue.task = NOTHING;
+		task->ret_queue.task = NULL;
 		rem_timed_task(task);
 		if (task->state != RT_SCHED_READY && (task->state &= ~(RT_SCHED_RECEIVE | RT_SCHED_DELAYED)) == RT_SCHED_READY) {
 			enq_ready_task(task);
@@ -161,10 +190,10 @@ RT_TASK *rt_send_if(RT_TASK *task, unsigned int msg)
 		}
 		if (rt_current->msg_queue.task != rt_current) {
 			rt_current->msg_queue.task = rt_current;
-			task = (RT_TASK *)0;
+			task = NULL;
 		}
 	} else {
-		task = (RT_TASK *)0;
+		task = NULL;
 	}
 	rt_global_restore_flags(flags);
 	return task;
@@ -210,13 +239,13 @@ RT_TASK *rt_send_if(RT_TASK *task, unsigned int msg)
  * the CPUs RTAI runs on 0xFFFF is not an address that can be used by
  * any RTAI task, so it is should be safe always.
  */
-RT_TASK *rt_send_until(RT_TASK *task, unsigned int msg, RTIME time)
+RTAI_SYSCALL_MODE RT_TASK *rt_send_until(RT_TASK *task, unsigned long msg, RTIME time)
 {
 	DECLARE_RT_CURRENT;
 	unsigned long flags;
 
 	if (task->magic != RT_TASK_MAGIC) {
-		return MSG_ERR;
+		return TASK_INVAL;
 	}
 
 	flags = rt_global_save_flags_and_cli();
@@ -225,7 +254,7 @@ RT_TASK *rt_send_until(RT_TASK *task, unsigned int msg, RTIME time)
 	      (!task->msg_queue.task || task->msg_queue.task == rt_current)) {
 		task->msg = msg;
 		task->msg_queue.task = rt_current;
-		task->ret_queue.task = NOTHING;
+		task->ret_queue.task = NULL;
 		rem_timed_task(task);
 		if (task->state != RT_SCHED_READY && (task->state &= ~(RT_SCHED_RECEIVE | RT_SCHED_DELAYED)) == RT_SCHED_READY) {
 			enq_ready_task(task);
@@ -237,6 +266,8 @@ RT_TASK *rt_send_until(RT_TASK *task, unsigned int msg, RTIME time)
 			rt_current->msg = msg;
 			enqueue_blocked(rt_current, &task->msg_queue, 0);
 			rt_current->state |= (RT_SCHED_SEND | RT_SCHED_DELAYED);
+			enqueue_resqtsk(task);
+			pass_prio(task, rt_current);
 			rem_ready_current(rt_current);
 			enq_timed_task(rt_current);
 			rt_schedule();
@@ -245,9 +276,7 @@ RT_TASK *rt_send_until(RT_TASK *task, unsigned int msg, RTIME time)
 		}
 	}
 	if (rt_current->msg_queue.task != rt_current) {
-		dequeue_blocked(rt_current);
-		rt_current->msg_queue.task = rt_current;
-		task = (RT_TASK *)0;
+		msg_not_sent();
 	}
 	rt_global_restore_flags(flags);
 	return task;
@@ -293,7 +322,7 @@ RT_TASK *rt_send_until(RT_TASK *task, unsigned int msg, RTIME time)
  * any RTAI task, so it is should be safe always. (FIXME)
  */
 
-RT_TASK *rt_send_timed(RT_TASK *task, unsigned int msg, RTIME delay)
+RTAI_SYSCALL_MODE RT_TASK *rt_send_timed(RT_TASK *task, unsigned long msg, RTIME delay)
 {
 	return rt_send_until(task, msg, get_time() + delay);
 }
@@ -348,14 +377,14 @@ RT_TASK *rt_send_timed(RT_TASK *task, unsigned int msg, RTIME delay)
  * 	 a more truly QNX like way of inter task messaging use the support
  * 	 of the upper cased functions: rt_Send-rt_Recieve-rt_Reply.
  */
-RT_TASK *rt_rpc(RT_TASK *task, unsigned int to_do, unsigned int *result)
+RTAI_SYSCALL_MODE RT_TASK *rt_rpc(RT_TASK *task, unsigned long to_do, void *result)
 {
 
 	DECLARE_RT_CURRENT;
 	unsigned long flags;
 
 	if (task->magic != RT_TASK_MAGIC) {
-		return MSG_ERR;
+		return TASK_INVAL;
 	}
 
 	flags = rt_global_save_flags_and_cli();
@@ -364,7 +393,7 @@ RT_TASK *rt_rpc(RT_TASK *task, unsigned int to_do, unsigned int *result)
 		(!task->msg_queue.task || task->msg_queue.task == rt_current)) {
 		rt_current->msg = task->msg = to_do;
 		task->msg_queue.task = rt_current;
-		task->ret_queue.task = NOTHING;
+		task->ret_queue.task = NULL;
 		rem_timed_task(task);
 		if (task->state != RT_SCHED_READY && (task->state &= ~(RT_SCHED_RECEIVE | RT_SCHED_DELAYED)) == RT_SCHED_READY) {
 			enq_ready_task(task);
@@ -376,16 +405,15 @@ RT_TASK *rt_rpc(RT_TASK *task, unsigned int to_do, unsigned int *result)
                 enqueue_blocked(rt_current, &task->msg_queue, 0);
 		rt_current->state |= RT_SCHED_RPC;
 	}
-	task->owndres += RPCINC;
+	enqueue_resqtsk(task);
 	pass_prio(task, rt_current);
 	rem_ready_current(rt_current);
 	rt_current->msg_queue.task = task;
 	RT_SCHEDULE_BOTH(task, cpuid);
 	if (rt_current->msg_queue.task == rt_current) {
-		*result = rt_current->msg;
+		*(unsigned long *)result = rt_current->msg;
 	} else {
-		rt_current->msg_queue.task = rt_current;
-		task = (RT_TASK *)0;
+		msg_not_sent();
 	}
 	rt_global_restore_flags(flags);
 	return task;
@@ -431,13 +459,13 @@ RT_TASK *rt_rpc(RT_TASK *task, unsigned int to_do, unsigned int *result)
  * 	 all the CPUs RTAI runs on, 0xFFFF is not an address that can
  *  	 be used by any RTAI task, so it is should be safe always.
  */
-RT_TASK *rt_rpc_if(RT_TASK *task, unsigned int to_do, unsigned int *result)
+RTAI_SYSCALL_MODE RT_TASK *rt_rpc_if(RT_TASK *task, unsigned long to_do, void *result)
 {
 	DECLARE_RT_CURRENT;
 	unsigned long flags;
 
 	if (task->magic != RT_TASK_MAGIC) {
-		return MSG_ERR;
+		return TASK_INVAL;
 	}
 	flags = rt_global_save_flags_and_cli();
 	ASSIGN_RT_CURRENT;
@@ -445,26 +473,25 @@ RT_TASK *rt_rpc_if(RT_TASK *task, unsigned int to_do, unsigned int *result)
 	      (!task->msg_queue.task || task->msg_queue.task == rt_current)) {
 		rt_current->msg = task->msg = to_do;
 		task->msg_queue.task = rt_current;
-		task->ret_queue.task = NOTHING;
+		task->ret_queue.task = NULL;
 		rem_timed_task(task);
 		if (task->state != RT_SCHED_READY && (task->state &= ~(RT_SCHED_RECEIVE | RT_SCHED_DELAYED)) == RT_SCHED_READY) {
 			enq_ready_task(task);
 		}
 		enqueue_blocked(rt_current, &task->ret_queue, 0);
 		rt_current->state |= RT_SCHED_RETURN;
-		task->owndres += RPCINC;
+		enqueue_resqtsk(task);
 		pass_prio(task, rt_current);
 		rem_ready_current(rt_current);
 		rt_current->msg_queue.task = task;
 		RT_SCHEDULE_BOTH(task, cpuid);
 		if (rt_current->msg_queue.task == rt_current) {
-			*result = rt_current->msg;
+			*(unsigned long *)result = rt_current->msg;
 		} else {
-			rt_current->msg_queue.task = rt_current;
-			task = (RT_TASK *)0;
+			msg_not_sent();
 		}
 	} else {
-		task = (RT_TASK *)0;
+		task = NULL;
 	}
 	rt_global_restore_flags(flags);
 	return task;
@@ -509,13 +536,13 @@ RT_TASK *rt_rpc_if(RT_TASK *task, unsigned int to_do, unsigned int *result)
  * task, so it is should be always safe.<br>
  * See also the notes under @ref rt_rpc().
  */
-RT_TASK *rt_rpc_until(RT_TASK *task, unsigned int to_do, unsigned int *result, RTIME time)
+RTAI_SYSCALL_MODE RT_TASK *rt_rpc_until(RT_TASK *task, unsigned long to_do, void *result, RTIME time)
 {
 	DECLARE_RT_CURRENT;
 	unsigned long flags;
 
 	if (task->magic != RT_TASK_MAGIC) {
-		return MSG_ERR;
+		return TASK_INVAL;
 	}
 
 	flags = rt_global_save_flags_and_cli();
@@ -528,7 +555,7 @@ RT_TASK *rt_rpc_until(RT_TASK *task, unsigned int to_do, unsigned int *result, R
 	    (!task->msg_queue.task || task->msg_queue.task == rt_current)) {
 		rt_current->msg = task->msg = to_do;
 		task->msg_queue.task = rt_current;
-		task->ret_queue.task = NOTHING;
+		task->ret_queue.task = NULL;
 		rem_timed_task(task);
 		if (task->state != RT_SCHED_READY && (task->state &= ~(RT_SCHED_RECEIVE | RT_SCHED_DELAYED)) == RT_SCHED_READY) {
 			enq_ready_task(task);
@@ -539,18 +566,16 @@ RT_TASK *rt_rpc_until(RT_TASK *task, unsigned int to_do, unsigned int *result, R
 		rt_current->state |= (RT_SCHED_RPC | RT_SCHED_DELAYED);
 	}
 	enqueue_blocked(rt_current, &task->ret_queue, 0);
-	task->owndres += RPCINC;
+	enqueue_resqtsk(task);
 	pass_prio(task, rt_current);
 	rem_ready_current(rt_current);
 	rt_current->msg_queue.task = task;
 	enq_timed_task(rt_current);
 	RT_SCHEDULE_BOTH(task, cpuid);
 	if (rt_current->msg_queue.task == rt_current) {
-		*result = rt_current->msg;
+		*(unsigned long *)result = rt_current->msg;
 	} else {
-		dequeue_blocked(rt_current);
-		rt_current->msg_queue.task = rt_current;
-		task = (RT_TASK *)0;
+		msg_not_sent();
 	}
 	rt_global_restore_flags(flags);
 	return task;
@@ -595,7 +620,7 @@ RT_TASK *rt_rpc_until(RT_TASK *task, unsigned int to_do, unsigned int *result, R
  * task, so it is should be always safe.<br>
  * See also the notes under @ref rt_rpc().
  */
-RT_TASK *rt_rpc_timed(RT_TASK *task, unsigned int to_do, unsigned int *result, RTIME delay)
+RTAI_SYSCALL_MODE RT_TASK *rt_rpc_timed(RT_TASK *task, unsigned long to_do, void *result, RTIME delay)
 {
 	return rt_rpc_until(task, to_do, result, get_time() + delay);
 }
@@ -626,7 +651,7 @@ RT_TASK *rt_rpc_timed(RT_TASK *task, unsigned int to_do, unsigned int *result, R
  * task which is not waiting for it. Therefore using rt_isrpc might not
  * be necessary. 
  */
-int rt_isrpc(RT_TASK *task)
+RTAI_SYSCALL_MODE int rt_isrpc(RT_TASK *task)
 {
 	return task->state & RT_SCHED_RETURN;
 }
@@ -667,13 +692,13 @@ int rt_isrpc(RT_TASK *task)
  *
  * See also: notes under @ref rt_rpc().
  */
-RT_TASK *rt_return(RT_TASK *task, unsigned int result)
+RTAI_SYSCALL_MODE RT_TASK *rt_return(RT_TASK *task, unsigned long result)
 {
 	DECLARE_RT_CURRENT;
 	unsigned long flags;
 
 	if (task->magic != RT_TASK_MAGIC) {
-		return MSG_ERR;
+		return TASK_INVAL;
 	}
 
 	flags = rt_global_save_flags_and_cli();
@@ -681,17 +706,7 @@ RT_TASK *rt_return(RT_TASK *task, unsigned int result)
 	if ((task->state & RT_SCHED_RETURN) && task->msg_queue.task == rt_current) {
 		int sched;
 		dequeue_blocked(task);
-		if (rt_current->owndres & RPCHLF) {
-			rt_current->owndres -= RPCINC;
-		}
-		if (!rt_current->owndres) {
-			sched = renq_current(rt_current, rt_current->base_priority);
-		} else if (!(rt_current->owndres & SEMHLF)) {
-			int priority;
-			sched = renq_current(rt_current, rt_current->base_priority > (priority = ((rt_current->msg_queue.next)->task)->priority) ? priority : rt_current->base_priority);
-		} else {
-			sched = 0;
-		}
+		sched = set_current_prio_from_resq(rt_current);
 		task->msg = result;
 		task->msg_queue.task = task;
 		rem_timed_task(task);
@@ -706,7 +721,7 @@ RT_TASK *rt_return(RT_TASK *task, unsigned int result)
                         rt_schedule();
                 }
 	} else {
-		task = (RT_TASK *)0;
+		task = NULL;
 	}
 	rt_global_restore_flags(flags);
 	return task;
@@ -742,20 +757,20 @@ RT_TASK *rt_return(RT_TASK *task, unsigned int result)
  * the CPUs RTAI runs on 0xFFFF is not an address that can be used by
  * any RTAI task, so it is should be always safe.
  */
-RT_TASK *rt_evdrp(RT_TASK *task, unsigned int *msg)
+RTAI_SYSCALL_MODE RT_TASK *rt_evdrp(RT_TASK *task, void *msg)
 {
 	DECLARE_RT_CURRENT;
 
 	if (task && task->magic != RT_TASK_MAGIC) {
-		return MSG_ERR;
+		return TASK_INVAL;
 	}
 
 	ASSIGN_RT_CURRENT;
 	if (!task) task = (rt_current->msg_queue.next)->task;
 	if ((task->state & (RT_SCHED_SEND | RT_SCHED_RPC)) && task->msg_queue.task == rt_current) {
-		*msg = task->msg;
+		*(unsigned long *)msg = task->msg;
 	} else {
-		task = (RT_TASK *)0;
+		task = NULL;
 	}
 	return task;
 }
@@ -796,13 +811,13 @@ RT_TASK *rt_evdrp(RT_TASK *task, unsigned int *msg)
  * the CPUs RTAI runs on 0xFFFF is not an address that can be used by
  * any RTAI task, so it is should be always safe.
  */
-RT_TASK *rt_receive(RT_TASK *task, unsigned int *msg)
+RTAI_SYSCALL_MODE RT_TASK *rt_receive(RT_TASK *task, void *msg)
 {
 	DECLARE_RT_CURRENT;
 	unsigned long flags;
 
 	if (task && task->magic != RT_TASK_MAGIC) {
-		return MSG_ERR;
+		return TASK_INVAL;
 	}
 
 	flags = rt_global_save_flags_and_cli();
@@ -811,35 +826,40 @@ RT_TASK *rt_receive(RT_TASK *task, unsigned int *msg)
 	if ((task->state & (RT_SCHED_SEND | RT_SCHED_RPC)) && task->msg_queue.task == rt_current) {
 		dequeue_blocked(task);
 		rem_timed_task(task);
-		*msg = task->msg;
+		*(unsigned long *)msg = task->msg;
 		rt_current->msg_queue.task = task;
 		if (task->state & RT_SCHED_SEND) {
+			int sched;
 			task->msg_queue.task = task;
+			sched = set_current_prio_from_resq(rt_current);
 			if (task->state != RT_SCHED_READY && (task->state &= ~(RT_SCHED_SEND | RT_SCHED_DELAYED)) == RT_SCHED_READY) {
 				enq_ready_task(task);
-				RT_SCHEDULE(task, cpuid);
+				if (sched) {
+					RT_SCHEDULE_BOTH(task, cpuid);
+				} else {
+					RT_SCHEDULE(task, cpuid);
+				}
 			}
 		} else if (task->state & RT_SCHED_RPC) {
                         enqueue_blocked(task, &rt_current->ret_queue, 0);
 			task->state = (task->state & ~(RT_SCHED_RPC | RT_SCHED_DELAYED)) | RT_SCHED_RETURN;
 		}
 	} else {
-		rt_current->ret_queue.task = SOMETHING;
+		rt_current->ret_queue.task = RTP_HIGERR;
 		rt_current->state |= RT_SCHED_RECEIVE;
 		rem_ready_current(rt_current);
 		rt_current->msg_queue.task = task != rt_current ? task : (RT_TASK *)0;
 		rt_schedule();
-		*msg = rt_current->msg;
+		*(unsigned long *)msg = rt_current->msg;
 	}
 	if (rt_current->ret_queue.task) {
-		rt_current->ret_queue.task = NOTHING;
-		task = (RT_TASK *)0;
+		msg_not_received();
 	} else {
 		task = rt_current->msg_queue.task;
 	}
 	rt_current->msg_queue.task = rt_current;
 	rt_global_restore_flags(flags);
-	if (task && (struct proxy_t *)task->stack_bottom) {
+	if ((unsigned long)task > RTE_HIGERR && (struct proxy_t *)task->stack_bottom) {
 		if (((struct proxy_t *)task->stack_bottom)->receiver == rt_current) {
 			rt_return(task, 0);
 		}
@@ -879,13 +899,13 @@ RT_TASK *rt_receive(RT_TASK *task, unsigned int *msg)
  * RTAI runs on 0xFFFF is not an address that can be used by any RTAI
  * task, so it is should be always safe.
  */
-RT_TASK *rt_receive_if(RT_TASK *task, unsigned int *msg)
+RTAI_SYSCALL_MODE RT_TASK *rt_receive_if(RT_TASK *task, void *msg)
 {
 	DECLARE_RT_CURRENT;
 	unsigned long flags;
 
 	if (task && task->magic != RT_TASK_MAGIC) {
-		return MSG_ERR;
+		return TASK_INVAL;
 	}
 
 	flags = rt_global_save_flags_and_cli();
@@ -894,30 +914,36 @@ RT_TASK *rt_receive_if(RT_TASK *task, unsigned int *msg)
 	if ((task->state & (RT_SCHED_SEND | RT_SCHED_RPC)) && task->msg_queue.task == rt_current) {
 		dequeue_blocked(task);
 		rem_timed_task(task);
-		*msg = task->msg;
+		*(unsigned long *)msg = task->msg;
 		rt_current->msg_queue.task = task;
 		if (task->state & RT_SCHED_SEND) {
+			int sched;
 			task->msg_queue.task = task;
+			sched = set_current_prio_from_resq(rt_current);
 			if (task->state != RT_SCHED_READY && (task->state &= ~(RT_SCHED_SEND | RT_SCHED_DELAYED)) == RT_SCHED_READY) {
 				enq_ready_task(task);
-				RT_SCHEDULE(task, cpuid);
+				if (sched) {
+					RT_SCHEDULE_BOTH(task, cpuid);
+				} else {
+					RT_SCHEDULE(task, cpuid);
+				}
 			}
 		} else if (task->state & RT_SCHED_RPC) {
 			enqueue_blocked(task, &rt_current->ret_queue, 0);
 			task->state = (task->state & ~(RT_SCHED_RPC | RT_SCHED_DELAYED)) | RT_SCHED_RETURN;
 		}
 		if (rt_current->ret_queue.task) {
-			rt_current->ret_queue.task = NOTHING;
-			task = (RT_TASK *)0;
+			rt_current->ret_queue.task = NULL;
+			task = NULL;
 		} else {
 			task = rt_current->msg_queue.task;
 		}
 		rt_current->msg_queue.task = rt_current;
 	} else {
-		task = (RT_TASK *)0;
+		task = NULL;
 	}
 	rt_global_restore_flags(flags);
-	if (task && (struct proxy_t *)task->stack_bottom) {
+	if ((unsigned long)task > RTE_HIGERR && (struct proxy_t *)task->stack_bottom) {
 		if (((struct proxy_t *)task->stack_bottom)->receiver == rt_current) {
 			rt_return(task, 0);
 		}
@@ -967,13 +993,13 @@ RT_TASK *rt_receive_if(RT_TASK *task, unsigned int *msg)
  *
  * See also: @ref rt_receive_timed().
  */
-RT_TASK *rt_receive_until(RT_TASK *task, unsigned int *msg, RTIME time)
+RTAI_SYSCALL_MODE RT_TASK *rt_receive_until(RT_TASK *task, void *msg, RTIME time)
 {
 	DECLARE_RT_CURRENT;
 	unsigned long flags;
 
 	if (task && task->magic != RT_TASK_MAGIC) {
-		return MSG_ERR;
+		return TASK_INVAL;
 	}
 
 	flags = rt_global_save_flags_and_cli();
@@ -982,38 +1008,43 @@ RT_TASK *rt_receive_until(RT_TASK *task, unsigned int *msg, RTIME time)
 	if ((task->state & (RT_SCHED_SEND | RT_SCHED_RPC)) && task->msg_queue.task == rt_current) {
 		dequeue_blocked(task);
 		rem_timed_task(task);
-		*msg = task->msg;
+		*(unsigned long *)msg = task->msg;
 		rt_current->msg_queue.task = task;
 		if (task->state & RT_SCHED_SEND) {
+			int sched;
 			task->msg_queue.task = task;
+			sched = set_current_prio_from_resq(rt_current);
 			if (task->state != RT_SCHED_READY && (task->state &= ~(RT_SCHED_SEND | RT_SCHED_DELAYED)) == RT_SCHED_READY) {
 				enq_ready_task(task);
-				RT_SCHEDULE(task, cpuid);
+				if (sched) {
+					RT_SCHEDULE_BOTH(task, cpuid);
+				} else {
+					RT_SCHEDULE(task, cpuid);
+				}
 			}
 		} else if (task->state & RT_SCHED_RPC) {
 			enqueue_blocked(task, &rt_current->ret_queue, 0);
 			task->state = (task->state & ~(RT_SCHED_RPC | RT_SCHED_DELAYED)) | RT_SCHED_RETURN;
 		}
 	} else {
-		rt_current->ret_queue.task = SOMETHING;
+		rt_current->ret_queue.task = RTP_HIGERR;
 		if ((rt_current->resume_time = time) > rt_time_h) {
 			rt_current->state |= (RT_SCHED_RECEIVE | RT_SCHED_DELAYED);
 			rem_ready_current(rt_current);
 			rt_current->msg_queue.task = task != rt_current ? task : (RT_TASK *)0;
 			enq_timed_task(rt_current);
 			rt_schedule();
-			*msg = rt_current->msg;
+			*(unsigned long *)msg = rt_current->msg;
 		}
 	}
 	if (rt_current->ret_queue.task) {
-		rt_current->ret_queue.task = NOTHING;
-		task = (RT_TASK *)0;
+		msg_not_received();
 	} else {
 		task = rt_current->msg_queue.task;
 	}
 	rt_current->msg_queue.task = rt_current;
 	rt_global_restore_flags(flags);
-	if (task && (struct proxy_t *)task->stack_bottom) {
+	if ((unsigned long)task > RTE_HIGERR && (struct proxy_t *)task->stack_bottom) {
 		if (((struct proxy_t *)task->stack_bottom)->receiver == rt_current) {
 			rt_return(task, 0);
 		}
@@ -1064,7 +1095,7 @@ RT_TASK *rt_receive_until(RT_TASK *task, unsigned int *msg, RTIME time)
  *
  * See also: @ref rt_receive_until().
  */
-RT_TASK *rt_receive_timed(RT_TASK *task, unsigned int *msg, RTIME delay)
+RTAI_SYSCALL_MODE RT_TASK *rt_receive_timed(RT_TASK *task, void *msg, RTIME delay)
 {
 	return rt_receive_until(task, msg, get_time() + delay);
 }
@@ -1083,7 +1114,7 @@ COPYRIGHT (C) 2003  Pierre Cloutier  (pcloutier@poseidoncontrols.com)
 	} while (0)
 
 /**
- * @ingroup rpcx
+ * @ingroup rpc
  * @anchor rt_rpcx
  * @brief Make an extended remote procedure call
  *
@@ -1118,19 +1149,19 @@ COPYRIGHT (C) 2003  Pierre Cloutier  (pcloutier@poseidoncontrols.com)
  * 	 found in QNX. For a even greater compatibility see
  *       rt_Send-rt_Receive-rt_Reply.
  */
-RT_TASK *rt_rpcx(RT_TASK *task, void *smsg, void *rmsg, int ssize, int rsize)
+RTAI_SYSCALL_MODE RT_TASK *rt_rpcx(RT_TASK *task, void *smsg, void *rmsg, int ssize, int rsize)
 {
 	if (task) {
 		struct mcb_t mcb;
 		SET_RPC_MCB();
-		return rt_rpc(task, (unsigned int)&mcb, &mcb.rbytes);
+		return rt_rpc(task, (unsigned long)&mcb, &mcb.rbytes);
 	}
 	return 0;
 }
 
 
 /**
- * @ingroup rpcx
+ * @ingroup rpc
  * @anchor rt_rpcx_if
  * @brief Make an extended remote procedure call, only if the calling task 
  * will not be blocked.
@@ -1167,12 +1198,12 @@ RT_TASK *rt_rpcx(RT_TASK *task, void *smsg, void *rmsg, int ssize, int rsize)
  * 	 all the CPUs RTAI runs on, 0xFFFF is not an address that can
  *  	 be used by any RTAI task, so it is should be always safe.
  */
-RT_TASK *rt_rpcx_if(RT_TASK *task, void *smsg, void *rmsg, int ssize, int rsize)
+RTAI_SYSCALL_MODE RT_TASK *rt_rpcx_if(RT_TASK *task, void *smsg, void *rmsg, int ssize, int rsize)
 {
 	if (task) {
 		struct mcb_t mcb;
 		SET_RPC_MCB();
-		return rt_rpc_if(task, (unsigned int)&mcb, &mcb.rbytes);
+		return rt_rpc_if(task, (unsigned long)&mcb, &mcb.rbytes);
 	}
 	return 0;
 }
@@ -1221,12 +1252,12 @@ RT_TASK *rt_rpcx_if(RT_TASK *task, void *smsg, void *rmsg, int ssize, int rsize)
  * task, so it is should be always safe.<br>
  * See also the notes under @ref rt_rpc().
  */
-RT_TASK *rt_rpcx_until(RT_TASK *task, void *smsg, void *rmsg, int ssize, int rsize, RTIME time)
+RTAI_SYSCALL_MODE RT_TASK *rt_rpcx_until(RT_TASK *task, void *smsg, void *rmsg, int ssize, int rsize, RTIME time)
 {
 	if (task) {
 		struct mcb_t mcb;
 		SET_RPC_MCB();
-		return rt_rpc_until(task, (unsigned int)&mcb, &mcb.rbytes, time);
+		return rt_rpc_until(task, (unsigned long)&mcb, &mcb.rbytes, time);
 	}
 	return 0;
 }
@@ -1275,12 +1306,12 @@ RT_TASK *rt_rpcx_until(RT_TASK *task, void *smsg, void *rmsg, int ssize, int rsi
  * task, so it is should be always safe.<br>
  * See also the notes under @ref rt_rpc().
  */
-RT_TASK *rt_rpcx_timed(RT_TASK *task, void *smsg, void *rmsg, int ssize, int rsize, RTIME delay)
+RTAI_SYSCALL_MODE RT_TASK *rt_rpcx_timed(RT_TASK *task, void *smsg, void *rmsg, int ssize, int rsize, RTIME delay)
 {
 	if (task) {
 		struct mcb_t mcb;
 		SET_RPC_MCB();
-		return rt_rpc_timed(task, (unsigned int)&mcb, &mcb.rbytes, delay);
+		return rt_rpc_timed(task, (unsigned long)&mcb, &mcb.rbytes, delay);
 	}
 	return 0;
 }
@@ -1327,11 +1358,11 @@ RT_TASK *rt_rpcx_timed(RT_TASK *task, void *smsg, void *rmsg, int ssize, int rsi
  * CPUs RTAI runs on 0xFFFF is not an address that can be used by any
  * RTAI task, so it is should be safe always.
  */
-RT_TASK *rt_sendx(RT_TASK *task, void *msg, int size) 
+RTAI_SYSCALL_MODE RT_TASK *rt_sendx(RT_TASK *task, void *msg, int size) 
 {
 	if (task) {
 		SET_SEND_MCB();
-		return rt_send(task, (unsigned int)&task_mcb);
+		return rt_send(task, (unsigned long)&task_mcb);
 	}
 	return 0;
 }
@@ -1368,11 +1399,11 @@ RT_TASK *rt_sendx(RT_TASK *task, void *msg, int size)
  * CPUs RTAI runs on 0xFFFF is not an address that can be used by any
  * RTAI task, so it is should be safe always.
  */
-RT_TASK *rt_sendx_if(RT_TASK *task, void *msg, int size)
+RTAI_SYSCALL_MODE RT_TASK *rt_sendx_if(RT_TASK *task, void *msg, int size)
 {
 	if (task) {
 		SET_SEND_MCB();
-		return rt_send_if(task, (unsigned int)&task_mcb);
+		return rt_send_if(task, (unsigned long)&task_mcb);
 	}
 	return 0;
 }
@@ -1419,11 +1450,11 @@ RT_TASK *rt_sendx_if(RT_TASK *task, void *msg, int size)
  * CPUs RTAI runs on 0xFFFF is not an address that can be used by any
  * RTAI task, so it is should be safe always.
  */
-RT_TASK *rt_sendx_until(RT_TASK *task, void *msg, int size, RTIME time)
+RTAI_SYSCALL_MODE RT_TASK *rt_sendx_until(RT_TASK *task, void *msg, int size, RTIME time)
 {
 	if (task) {
 		SET_SEND_MCB();
-		return rt_send_until(task, (unsigned int)&task_mcb, time);
+		return rt_send_until(task, (unsigned long)&task_mcb, time);
 	}
 	return 0;
 }
@@ -1470,11 +1501,11 @@ RT_TASK *rt_sendx_until(RT_TASK *task, void *msg, int size, RTIME time)
  * CPUs RTAI runs on 0xFFFF is not an address that can be used by any
  * RTAI task, so it is should be safe always.
  */
-RT_TASK *rt_sendx_timed(RT_TASK *task, void *msg, int size, RTIME delay)
+RTAI_SYSCALL_MODE RT_TASK *rt_sendx_timed(RT_TASK *task, void *msg, int size, RTIME delay)
 {
 	if (task) {
 		SET_SEND_MCB();
-		return rt_send_timed(task, (unsigned int)&task_mcb, delay);
+		return rt_send_timed(task, (unsigned long)&task_mcb, delay);
 	}
 	return 0;
 }
@@ -1515,7 +1546,7 @@ RT_TASK *rt_sendx_timed(RT_TASK *task, void *msg, int size, RTIME delay)
  *
  * See also: notes under @ref rt_rpcx().
  */
-RT_TASK *rt_returnx(RT_TASK *task, void *msg, int size)
+RTAI_SYSCALL_MODE RT_TASK *rt_returnx(RT_TASK *task, void *msg, int size)
 {
 	if (task) {
 		struct mcb_t *mcb;
@@ -1570,10 +1601,10 @@ RT_TASK *rt_returnx(RT_TASK *task, void *msg, int size)
  * the CPUs RTAI runs on 0xFFFF is not an address that can be used by
  * any RTAI task, so it is should be always safe.
  */
-RT_TASK *rt_evdrpx(RT_TASK *task, void *msg, int size, int *len)
+RTAI_SYSCALL_MODE RT_TASK *rt_evdrpx(RT_TASK *task, void *msg, int size, long *len)
 {
 	struct mcb_t *mcb;
-	if ((task = rt_evdrp(task, (unsigned int *)&mcb))) {
+	if ((task = rt_evdrp(task, (unsigned long *)&mcb))) {
 		DO_RCV_MSG();
 		return task;
 	}
@@ -1621,10 +1652,10 @@ RT_TASK *rt_evdrpx(RT_TASK *task, void *msg, int size, int *len)
  * the CPUs RTAI runs on 0xFFFF is not an address that can be used by
  * any RTAI task, so it is should be always safe.
  */
-RT_TASK *rt_receivex(RT_TASK *task, void *msg, int size, int *len)
+RTAI_SYSCALL_MODE RT_TASK *rt_receivex(RT_TASK *task, void *msg, int size, long *len)
 {
 	struct mcb_t *mcb;
-	if ((task = rt_receive(task, (unsigned int *)&mcb))) {
+	if ((task = rt_receive(task, (unsigned long *)&mcb))) {
 		DO_RCV_MSG();
 		return task;
 	}
@@ -1672,10 +1703,10 @@ RT_TASK *rt_receivex(RT_TASK *task, void *msg, int size, int *len)
  * the CPUs RTAI runs on 0xFFFF is not an address that can be used by
  * any RTAI task, so it is should be always safe.
  */
-RT_TASK *rt_receivex_if(RT_TASK *task, void *msg, int size, int *len)
+RTAI_SYSCALL_MODE RT_TASK *rt_receivex_if(RT_TASK *task, void *msg, int size, long *len)
 {
 	struct mcb_t *mcb;
-	if ((task = rt_receive_if(task, (unsigned int *)&mcb))) {
+	if ((task = rt_receive_if(task, (unsigned long *)&mcb))) {
 		DO_RCV_MSG();
 		return task;
 	}
@@ -1729,10 +1760,10 @@ RT_TASK *rt_receivex_if(RT_TASK *task, void *msg, int size, int *len)
  * the CPUs RTAI runs on 0xFFFF is not an address that can be used by
  * any RTAI task, so it is should be always safe.
  */
-RT_TASK *rt_receivex_until(RT_TASK *task, void *msg, int size, int *len, RTIME time)
+RTAI_SYSCALL_MODE RT_TASK *rt_receivex_until(RT_TASK *task, void *msg, int size, long *len, RTIME time)
 {
 	struct mcb_t *mcb;
-	if ((task = rt_receive_until(task, (unsigned int *)&mcb, time))) {
+	if ((task = rt_receive_until(task, (unsigned long *)&mcb, time))) {
 		DO_RCV_MSG();
 		return task;
 	}
@@ -1786,10 +1817,10 @@ RT_TASK *rt_receivex_until(RT_TASK *task, void *msg, int size, int *len, RTIME t
  * the CPUs RTAI runs on 0xFFFF is not an address that can be used by
  * any RTAI task, so it is should be always safe.
  */
-RT_TASK *rt_receivex_timed(RT_TASK *task, void *msg, int size, int *len, RTIME delay)
+RTAI_SYSCALL_MODE RT_TASK *rt_receivex_timed(RT_TASK *task, void *msg, int size, long *len, RTIME delay)
 {
 	struct mcb_t *mcb;
-	if ((task = rt_receive_timed(task, (unsigned int *)&mcb, delay))) {
+	if ((task = rt_receive_timed(task, (unsigned long *)&mcb, delay))) {
 		DO_RCV_MSG();
 		return task;
 	}
@@ -1802,20 +1833,20 @@ RT_TASK *rt_receivex_timed(RT_TASK *task, void *msg, int size, int *len, RTIME d
 static void proxy_task(RT_TASK *me)
 {
 	struct proxy_t *my;
-	unsigned int ret;
+	unsigned long ret;
 
 	my = (struct proxy_t *)me->stack_bottom;        	
 	while (1) {
 		while (my->nmsgs) {
 		 	atomic_dec((atomic_t *)&my->nmsgs);
-			rt_rpc(my->receiver, *((unsigned int *)my->msg), &ret);
+			rt_rpc(my->receiver, *((unsigned long *)my->msg), &ret);
 		}
 		rt_task_suspend(me);
 	}
 }
 
 // Create a raw proxy agent task.
-RT_TASK *__rt_proxy_attach(void (*agent)(int), RT_TASK *task, void *msg, int nbytes, int priority)
+RT_TASK *__rt_proxy_attach(void (*agent)(long), RT_TASK *task, void *msg, int nbytes, int priority)
 {
 	RT_TASK *proxy, *rt_current;
 	struct proxy_t *my;
@@ -1836,7 +1867,7 @@ RT_TASK *__rt_proxy_attach(void (*agent)(int), RT_TASK *task, void *msg, int nby
 	if (priority == -1 && (priority = rt_current->base_priority) == RT_SCHED_LINUX_PRIORITY) {
 		priority = RT_SCHED_LOWEST_PRIORITY;
 	}
-	if (rt_kthread_init(proxy, agent, (int)proxy, PROXY_MIN_STACK_SIZE + nbytes + sizeof(struct proxy_t), priority, 0, 0)) {
+	if (rt_kthread_init(proxy, agent, (long)proxy, PROXY_MIN_STACK_SIZE + nbytes + sizeof(struct proxy_t), priority, 0, 0)) {
 		rt_free(proxy);
 		return 0;
 	}
@@ -1855,14 +1886,14 @@ RT_TASK *__rt_proxy_attach(void (*agent)(int), RT_TASK *task, void *msg, int nby
 }
 
 // Create a raw proxy task.
-RT_TASK *rt_proxy_attach(RT_TASK *task, void *msg, int nbytes, int prio)
+RTAI_SYSCALL_MODE RT_TASK *rt_proxy_attach(RT_TASK *task, void *msg, int nbytes, int prio)
 {
 	return __rt_proxy_attach((void *)proxy_task, task, msg, nbytes, prio);
 }
 
 // Delete a proxy task (a simplified specific rt_task_delete).
 // Note: a self delete will not do the rt_free() call.
-int rt_proxy_detach(RT_TASK *proxy)
+RTAI_SYSCALL_MODE int rt_proxy_detach(RT_TASK *proxy)
 {
 	if (!rt_task_delete(proxy)) {
 		rt_free(proxy);
@@ -1872,7 +1903,7 @@ int rt_proxy_detach(RT_TASK *proxy)
 }
 
 // Trigger a proxy.
-RT_TASK *rt_trigger(RT_TASK *proxy)
+RTAI_SYSCALL_MODE RT_TASK *rt_trigger(RT_TASK *proxy)
 {
 	struct proxy_t *his;
 	
@@ -1885,7 +1916,7 @@ RT_TASK *rt_trigger(RT_TASK *proxy)
 	return (RT_TASK *)0;
 }
 
-#ifdef CONFIG_RTAI_INTERNAL_LXRT_SUPPORT
+#if 1 //def CONFIG_RTAI_INTERNAL_LXRT_SUPPORT
 
 /* ++++++++++++ ANOTHER API SET FOR EXTENDED INTERTASK MESSAGES +++++++++++++++
 COPYRIGHT (C) 2003  Pierre Cloutier  (pcloutier@poseidoncontrols.com)
@@ -1897,19 +1928,19 @@ COPYRIGHT (C) 2003  Pierre Cloutier  (pcloutier@poseidoncontrols.com)
 #include "rtai_registry.h"
 #include "rtai_msg.h"
 
-int rt_Send(pid_t pid, void *smsg, void *rmsg, size_t ssize, size_t rsize)
+RTAI_SYSCALL_MODE int rt_Send(pid_t pid, void *smsg, void *rmsg, size_t ssize, size_t rsize)
 {
 	RT_TASK *task;
 	if ((task = pid2rttask(pid))) {
 		MSGCB cb;
 		RT_TASK *replier;
-		unsigned int replylen;
+		unsigned long replylen;
 		cb.cmd    = SYNCMSG;
 		cb.sbuf   = smsg;
 		cb.sbytes = ssize; 
 		cb.rbuf   = rmsg;
 		cb.rbytes = rsize;
-		if (!(replier = rt_rpc(task, (unsigned int)&cb, &replylen))) {
+		if (!(replier = rt_rpc(task, (unsigned long)&cb, &replylen))) {
 			return -EINVAL;
 		} else if (replier != task) {
 			return -ESRCH;
@@ -1919,7 +1950,7 @@ int rt_Send(pid_t pid, void *smsg, void *rmsg, size_t ssize, size_t rsize)
 	return -ESRCH;
 }
 
-pid_t rt_Receive(pid_t pid, void *msg, size_t maxsize, size_t *msglen)
+RTAI_SYSCALL_MODE pid_t rt_Receive(pid_t pid, void *msg, size_t maxsize, size_t *msglen)
 {
 	RT_TASK *task;
 	MSGCB *cb;
@@ -1936,7 +1967,7 @@ pid_t rt_Receive(pid_t pid, void *msg, size_t maxsize, size_t *msglen)
 	return -EINVAL;
 }
 
-pid_t rt_Creceive(pid_t pid, void *msg, size_t maxsize, size_t *msglen, RTIME delay)
+RTAI_SYSCALL_MODE pid_t rt_Creceive(pid_t pid, void *msg, size_t maxsize, size_t *msglen, RTIME delay)
 {
 	RT_TASK *task;
 	MSGCB *cb;
@@ -1959,13 +1990,13 @@ pid_t rt_Creceive(pid_t pid, void *msg, size_t maxsize, size_t *msglen, RTIME de
 	return 0;
 }
 
-int rt_Reply(pid_t pid, void *msg, size_t size)
+RTAI_SYSCALL_MODE int rt_Reply(pid_t pid, void *msg, size_t size)
 {
 	RT_TASK *task;
 	if ((task = pid2rttask(pid))) {
 		MSGCB *cb;
 		if ((cb = (MSGCB *)task->msg)->cmd == SYNCMSG) {
-			unsigned int retlen;
+			unsigned long retlen;
 			RT_TASK *retask;
 			if ((retlen = size <= cb->rbytes ? size : cb->rbytes)) {
 				memcpy(cb->rbuf, msg, retlen);
@@ -1986,7 +2017,7 @@ static void Proxy_Task(RT_TASK *me)
 {
         struct proxy_t *my;
 	MSGCB cb;
-        unsigned int replylen;
+        unsigned long replylen;
         my = (struct proxy_t *)me->stack_bottom;
 	cb.cmd    = PROXY;
 	cb.sbuf   = my->msg;
@@ -1996,19 +2027,19 @@ static void Proxy_Task(RT_TASK *me)
         while(1) {
 		while (my->nmsgs) {
 			atomic_dec((atomic_t *)&my->nmsgs);
-                        rt_rpc(my->receiver, (unsigned int)(&cb), &replylen);
+                        rt_rpc(my->receiver, (unsigned long)(&cb), &replylen);
 		}
 		rt_task_suspend(me);
         }
 }
 
-pid_t rt_Proxy_attach(pid_t pid, void *msg, int nbytes, int prio)
+RTAI_SYSCALL_MODE pid_t rt_Proxy_attach(pid_t pid, void *msg, int nbytes, int prio)
 {
 	RT_TASK *task;
 	return (task = __rt_proxy_attach((void *)Proxy_Task, pid ? pid2rttask(pid) : 0, msg, nbytes, prio)) ? (task->lnxtsk)->pid : -ENOMEM;
 }
 
-int rt_Proxy_detach(pid_t pid)
+RTAI_SYSCALL_MODE int rt_Proxy_detach(pid_t pid)
 {
 	RT_TASK *proxy;
 	if (!rt_task_delete(proxy = pid2rttask(pid))) {
@@ -2018,7 +2049,7 @@ int rt_Proxy_detach(pid_t pid)
 	return -EINVAL;
 }
 
-pid_t rt_Trigger(pid_t pid)
+RTAI_SYSCALL_MODE pid_t rt_Trigger(pid_t pid)
 {
 	RT_TASK *proxy;
        	struct proxy_t *his;
@@ -2035,20 +2066,20 @@ pid_t rt_Trigger(pid_t pid)
 }
 
 
-pid_t rt_Name_attach(const char *argname)
+RTAI_SYSCALL_MODE pid_t rt_Name_attach(const char *argname)
 {
 	RT_TASK *task;
-	task = current->rtai_tskext(0) ? (RT_TASK *)current->rtai_tskext(0) : _rt_whoami();
+	task = current->rtai_tskext(TSKEXT0) ? (RT_TASK *)current->rtai_tskext(TSKEXT0) : _rt_whoami();
 	if (current->comm[0] != 'U' && current->comm[1] != ':') {
-	    	strncpy_from_user(task->task_name, argname, MAX_NAME_LENGTH);
+	    	rt_strncpy_from_user(task->task_name, argname, RTAI_MAX_NAME_LENGTH);
 	} else {
-	    	strncpy(task->task_name, argname, MAX_NAME_LENGTH);
+	    	strncpy(task->task_name, argname, RTAI_MAX_NAME_LENGTH);
 	}
-    	task->task_name[MAX_NAME_LENGTH - 1] = 0;
-	return strnlen(task->task_name, MAX_NAME_LENGTH) > (MAX_NAME_LENGTH - 1) ? -EINVAL : task->lnxtsk ? ((struct task_struct *)current->rtai_tskext(1))->pid : (pid_t)task;
+    	task->task_name[RTAI_MAX_NAME_LENGTH - 1] = 0;
+	return strnlen(task->task_name, RTAI_MAX_NAME_LENGTH) > (RTAI_MAX_NAME_LENGTH - 1) ? -EINVAL : task->lnxtsk ? ((struct task_struct *)current->rtai_tskext(TSKEXT1))->pid : (long)task;
 }
 
-pid_t rt_Name_locate(const char *arghost, const char *argname)
+RTAI_SYSCALL_MODE pid_t rt_Name_locate(const char *arghost, const char *argname)
 {
 	extern RT_TASK rt_smp_linux_task[];
 	int cpuid;
@@ -2056,8 +2087,8 @@ pid_t rt_Name_locate(const char *arghost, const char *argname)
         for (cpuid = 0; cpuid < num_online_cpus(); cpuid++) {
                 task = &rt_smp_linux_task[cpuid];
                 while ((task = task->next)) {
-			if (!strncmp(argname, task->task_name, MAX_NAME_LENGTH - 1)) {
-				return (struct task_struct *)(task->lnxtsk) ?  ((struct task_struct *)(task->lnxtsk)->rtai_tskext(1))->pid : (pid_t)task;
+			if (!strncmp(argname, task->task_name, RTAI_MAX_NAME_LENGTH - 1)) {
+				return (struct task_struct *)(task->lnxtsk) ?  ((struct task_struct *)(task->lnxtsk)->rtai_tskext(TSKEXT1))->pid : (long)task;
 
 			}
 		}
@@ -2065,15 +2096,15 @@ pid_t rt_Name_locate(const char *arghost, const char *argname)
 	return strlen(argname) <= 6 && (task = rt_get_adr(nam2num(argname))) ? rttask2pid(task) : 0;
 }
 
-int rt_Name_detach(pid_t pid)
+RTAI_SYSCALL_MODE int rt_Name_detach(pid_t pid)
 {
 	if (pid <= PID_MAX_LIMIT) {
-	 	if (pid != ((struct task_struct *)current->rtai_tskext(1))->pid ) {
+	 	if (pid != ((struct task_struct *)current->rtai_tskext(TSKEXT1))->pid ) {
 			return -EINVAL;
 		}
-	    	((RT_TASK *)current->rtai_tskext(0))->task_name[0] = 0;
+	    	((RT_TASK *)current->rtai_tskext(TSKEXT0))->task_name[0] = 0;
 	} else {
-	    	((RT_TASK *)pid)->task_name[0] = 0;
+	    	((RT_TASK *)(long)pid)->task_name[0] = 0;
 	}
 	return 0;
 }
@@ -2087,6 +2118,7 @@ struct rt_native_fun_entry rt_msg_entries[] = {
 	{ { 1, rt_send_if },				SEND_IF },
 	{ { 1, rt_send_until },				SEND_UNTIL },
 	{ { 1, rt_send_timed },				SEND_TIMED },
+	{ { UW1(2, 0), rt_evdrp },			EVDRP },
 	{ { UW1(2, 0), rt_receive },			RECEIVEMSG },
 	{ { UW1(2, 0), rt_receive_if },			RECEIVE_IF },
 	{ { UW1(2, 0), rt_receive_until },		RECEIVE_UNTIL },
@@ -2095,7 +2127,6 @@ struct rt_native_fun_entry rt_msg_entries[] = {
 	{ { UW1(3, 0), rt_rpc_if },			RPC_IF },
 	{ { UW1(3, 0), rt_rpc_until },			RPC_UNTIL },
 	{ { UW1(3, 0), rt_rpc_timed },			RPC_TIMED },
-	{ { UW1(2, 0), rt_evdrp },			EVDRP },
 	{ { 0, rt_isrpc }, 		 		ISRPC },
 	{ { 1, rt_return },				RETURNMSG },
 	{ { UR1(2, 4) | UW1(3, 5), rt_rpcx },		RPCX },
@@ -2107,15 +2138,14 @@ struct rt_native_fun_entry rt_msg_entries[] = {
 	{ { UR1(2, 3), rt_sendx_until },		SENDX_UNTIL },
 	{ { UR1(2, 3), rt_sendx_timed },		SENDX_TIMED },
 	{ { UR1(2, 3), rt_returnx },			RETURNX },
+	{ { UW1(2, 3) | UW2(4, 0), rt_evdrpx },		EVDRPX },
 	{ { UW1(2, 3) | UW2(4, 0), rt_receivex },	RECEIVEX },
 	{ { UW1(2, 3) | UW2(4, 0), rt_receivex_if },	RECEIVEX_IF },
 	{ { UW1(2, 3) | UW2(4, 0), rt_receivex_until }, RECEIVEX_UNTIL },
 	{ { UW1(2, 3) | UW2(4, 0), rt_receivex_timed },	RECEIVEX_TIMED },
-	{ { UW1(2, 3) | UW2(4, 0), rt_evdrpx },		EVDRPX },
 	{ { UR1(2, 3), rt_proxy_attach },         	PROXY_ATTACH },
 	{ { 1, rt_proxy_detach },                 	PROXY_DETACH },
 	{ { 1, rt_trigger },                      	PROXY_TRIGGER },
-#ifdef CONFIG_RTAI_INTERNAL_LXRT_SUPPORT
 	{ { UR1(2, 4) | UW1(3, 5), rt_Send },	 	RT_SEND },
 	{ { UW1(2, 3) | UW2(4, 0), rt_Receive },	RT_RECEIVE },
 	{ { UW1(2, 3) | UW2(4, 0), rt_Creceive }, 	RT_CRECEIVE },
@@ -2124,9 +2154,8 @@ struct rt_native_fun_entry rt_msg_entries[] = {
 	{ { 1, rt_Proxy_detach },		  	RT_PROXY_DETACH },
 	{ { 1, rt_Trigger },			  	RT_TRIGGER },
 	{ { 1, rt_Name_attach },		  	RT_NAME_ATTACH },
-	{ { 1, rt_Name_detach },		  	RT_NAME_DETACH },
 	{ { 0, rt_Name_locate },		  	RT_NAME_LOCATE },
-#endif /* CONFIG_RTAI_INTERNAL_LXRT_SUPPORT */
+	{ { 1, rt_Name_detach },		  	RT_NAME_DETACH },
 	{ { 0, 0 },  		      			000 }
 };
 
