@@ -29,13 +29,21 @@
  *
  *@{*/
 
+#include <linux/kernel.h>
+#include <linux/module.h>
+
+#include <asm/uaccess.h>
+
 #include <rtai_schedcore.h>
+#include <rtai_prinher.h>
 #include <rtai_sem.h>
 #include <rtai_rwl.h>
 #include <rtai_spl.h>
 
 MODULE_LICENSE("GPL");
 
+extern struct epoch_struct boot_epoch;
+
 /* +++++++++++++++++++++ ALL SEMAPHORES TYPES SUPPORT +++++++++++++++++++++++ */
 
 /**
@@ -90,15 +98,18 @@ MODULE_LICENSE("GPL");
  * posed on it is just registered. An owner task will go into suspend
  * state only when it releases all the owned resources.
  *
- * @note RTAI counting semaphores assume that their counter will never
- *	 exceed 0xFFFF, such a number being used to signal returns in
- *	 error. Thus also the initial count value cannot be greater
- *	 than 0xFFFF. To be used only with RTAI24.x.xx (FIXME).
+ * @note if the legacy error return values scheme is used RTAI counting 
+ *       semaphores assume that their counter will never exceed 0xFFFF, 
+ *       such a number being used to signal returns in error. Thus also 
+ *       the initial count value cannot be greater 0xFFFF. The new error
+ *       return scheme allows counts in the order of billions instead.
+ *
  */
-void rt_typed_sem_init(SEM *sem, int value, int type)
+RTAI_SYSCALL_MODE void rt_typed_sem_init(SEM *sem, int value, int type)
 {
 	sem->magic = RT_SEM_MAGIC;
 	sem->count = value;
+	sem->restype = 0;
 	if ((type & RES_SEM) == RES_SEM) {
 		sem->qtype = 0;
 	} else {
@@ -109,10 +120,14 @@ void rt_typed_sem_init(SEM *sem, int value, int type)
 		sem->count = 1;
 	} else if (type > 0) {
 		sem->type = sem->count = 1;
+		sem->restype = value;
 	}
 	sem->queue.prev = &(sem->queue);
 	sem->queue.next = &(sem->queue);
-	sem->queue.task = sem->owndby = 0;
+	sem->queue.task = sem->owndby = NULL;
+
+	sem->resq.prev = sem->resq.next = &sem->resq;
+	sem->resq.task = (void *)&sem->queue;
 }
 
 
@@ -138,7 +153,7 @@ void rt_typed_sem_init(SEM *sem, int value, int type)
  *	 exceed 0xFFFF, such a number being used to signal returns in
  *	 error. Thus also the initial count value cannot be greater
  *	 than 0xFFFF.
- *	 This is an old legacy function. RTAI 24.1.xx has also 
+ *	 This is an old legacy functioni, there is also 
  *	 @ref rt_typed_sem_init(), allowing to
  *	 choose among counting, binary and resource
  *	 semaphores. Resource semaphores have priority inherithance. 
@@ -172,7 +187,7 @@ void rt_sem_init(SEM *sem, int value)
  *	 up to such number of events, in any case avoid counting up 
  *	 to 0xFFFF. 
  */
-int rt_sem_delete(SEM *sem)
+RTAI_SYSCALL_MODE int rt_sem_delete(SEM *sem)
 {
 	unsigned long flags;
 	RT_TASK *task;
@@ -180,7 +195,7 @@ int rt_sem_delete(SEM *sem)
 	QUEUE *q;
 
 	if (sem->magic != RT_SEM_MAGIC) {
-		return SEM_ERR;
+		return RTE_OBJINV;
 	}
 
 	schedmap = 0;
@@ -190,6 +205,7 @@ int rt_sem_delete(SEM *sem)
 	while ((q = q->next) != &(sem->queue) && (task = q->task)) {
 		rem_timed_task(task);
 		if (task->state != RT_SCHED_READY && (task->state &= ~(RT_SCHED_SEMAPHORE | RT_SCHED_DELAYED)) == RT_SCHED_READY) {
+			task->blocked_on = RTP_OBJREM;
 			enq_ready_task(task);
 			set_bit(task->runnable_on_cpus & 0x1F, &schedmap);
 		}
@@ -197,21 +213,13 @@ int rt_sem_delete(SEM *sem)
 	sched = schedmap;
 	clear_bit(rtai_cpuid(), &schedmap);
 	if ((task = sem->owndby) && sem->type > 0) {
-		if (task->owndres & SEMHLF) {
-			--task->owndres;
-		}
-		if (!task->owndres) {
-			sched |= renq_ready_task(task, task->base_priority);
-		} else if (!(task->owndres & SEMHLF)) {
-			int priority;
-                        sched |= renq_ready_task(task, task->base_priority > (priority = ((task->msg_queue.next)->task)->priority) ? priority : task->base_priority);
-		}
+		sched |= dequeue_resqel_reset_task_priority(&sem->resq, task);
 		if (task->suspdepth) {
 			if (task->suspdepth > 0) {
 				task->state |= RT_SCHED_SUSPENDED;
 				rem_ready_task(task);
 				sched = 1;
-			} else {
+			} else if (task->suspdepth == RT_RESEM_SUSPDEL) {
 				rt_task_delete(task);
 			}
 		}
@@ -228,7 +236,7 @@ int rt_sem_delete(SEM *sem)
 }
 
 
-int rt_sem_count(SEM *sem)
+RTAI_SYSCALL_MODE int rt_sem_count(SEM *sem)
 {
 	return sem->count;
 }
@@ -258,18 +266,22 @@ int rt_sem_count(SEM *sem)
  * 	 0xFFFF.
  *	 See @ref rt_sem_wait() notes for some curiosities.
  */
-int rt_sem_signal(SEM *sem)
+RTAI_SYSCALL_MODE int rt_sem_signal(SEM *sem)
 {
 	unsigned long flags;
 	RT_TASK *task;
 	int tosched;
 
 	if (sem->magic != RT_SEM_MAGIC) {
-		return SEM_ERR;
+		return RTE_OBJINV;
 	}
 
 	flags = rt_global_save_flags_and_cli();
 	if (sem->type) {
+		if (sem->restype && (!sem->owndby || sem->owndby != RT_CURRENT)) {
+			rt_global_restore_flags(flags);
+			return RTE_PERM;
+		}
 		if (sem->type > 1) {
 			sem->type--;
 			rt_global_restore_flags(flags);
@@ -301,23 +313,13 @@ res:	if (sem->type > 0) {
 		int sched;
 		ASSIGN_RT_CURRENT;
 		sem->owndby = 0;
-		if (rt_current->owndres & SEMHLF) {
-			--rt_current->owndres;
-		}
-		if (!rt_current->owndres) {
-			sched = renq_current(rt_current, rt_current->base_priority);
-		} else if (!(rt_current->owndres & SEMHLF)) {
-			int priority;
-			sched = renq_current(rt_current, rt_current->base_priority > (priority = ((rt_current->msg_queue.next)->task)->priority) ? priority : rt_current->base_priority);
-		} else {
-			sched = 0;
-		}
+		sched = dequeue_resqel_reset_current_priority(&sem->resq, rt_current);
 		if (rt_current->suspdepth) {
 			if (rt_current->suspdepth > 0) {
 				rt_current->state |= RT_SCHED_SUSPENDED;
 				rem_ready_current(rt_current);
                         	sched = 1;
-			} else {
+			} else if (task->suspdepth == RT_RESEM_SUSPDEL) {
 				rt_task_delete(rt_current);
 			}
 		}
@@ -351,24 +353,26 @@ res:	if (sem->type > 0) {
  * 
  * @returns 0 always.
  */
-int rt_sem_broadcast(SEM *sem)
+RTAI_SYSCALL_MODE int rt_sem_broadcast(SEM *sem)
 {
 	unsigned long flags, schedmap;
 	RT_TASK *task;
 	QUEUE *q;
 
 	if (sem->magic != RT_SEM_MAGIC) {
-		return SEM_ERR;
+		return RTE_OBJINV;
 	}
 	schedmap = 0;
 	q = &(sem->queue);
 	flags = rt_global_save_flags_and_cli();
 	while ((q = q->next) != &(sem->queue)) {
-		dequeue_blocked(task = q->task);
-		rem_timed_task(task);
-		if (task->state != RT_SCHED_READY && (task->state &= ~(RT_SCHED_SEMAPHORE | RT_SCHED_DELAYED)) == RT_SCHED_READY) {
-			enq_ready_task(task);
-			set_bit(task->runnable_on_cpus & 0x1F, &schedmap);
+		if ((task = q->task)) {
+			dequeue_blocked(task = q->task);
+			rem_timed_task(task);
+			if (task->state != RT_SCHED_READY && (task->state &= ~(RT_SCHED_SEMAPHORE | RT_SCHED_DELAYED)) == RT_SCHED_READY) {
+				enq_ready_task(task);
+				set_bit(task->runnable_on_cpus & 0x1F, &schedmap);
+			}
 		}
 		rt_global_restore_flags(flags);
 		flags = rt_global_save_flags_and_cli();
@@ -428,25 +432,30 @@ int rt_sem_broadcast(SEM *sem)
  *	 traffic light, so that semaphores have an intuitive appeal
  * 	 and their use and meaning is easily understood.
  */
-int rt_sem_wait(SEM *sem)
+RTAI_SYSCALL_MODE int rt_sem_wait(SEM *sem)
 {
 	RT_TASK *rt_current;
 	unsigned long flags;
 	int count;
 
 	if (sem->magic != RT_SEM_MAGIC) {
-		return SEM_ERR;
+		return RTE_OBJINV;
 	}
 
 	flags = rt_global_save_flags_and_cli();
 	rt_current = RT_CURRENT;
 	if ((count = sem->count) <= 0) {
+		void *retp;
 		unsigned long schedmap;
 		if (sem->type > 0) {
-			if (sem->owndby == rt_current) {
-				count = sem->type++;
+			if (sem->restype && sem->owndby == rt_current) {
+				if (sem->restype > 0) {
+					count = sem->type++;
+					rt_global_restore_flags(flags);
+					return count + 1;
+				}
 				rt_global_restore_flags(flags);
-				return count + 1;
+				return RTE_DEADLOK;
 			}
 			schedmap = pass_prio(sem->owndby, rt_current);
 		} else {
@@ -457,18 +466,30 @@ int rt_sem_wait(SEM *sem)
 		rem_ready_current(rt_current);
 		enqueue_blocked(rt_current, &sem->queue, sem->qtype);
 		RT_SCHEDULE_MAP_BOTH(schedmap);
-		if (rt_current->blocked_on || sem->magic != RT_SEM_MAGIC) {
-			rt_current->prio_passed_to = NOTHING;
-			rt_global_restore_flags(flags);
-			return SEM_ERR;
-		} else { 
+		if (likely(!(retp = rt_current->blocked_on))) { 
 			count = sem->count;
+		} else {
+			if (likely(retp != RTP_OBJREM)) { 
+				dequeue_blocked(rt_current);
+				if (++sem->count > 1 && sem->type) {
+					sem->count = 1;
+				}
+				if (sem->owndby && sem->type > 0) {
+					set_task_prio_from_resq(sem->owndby);
+				}
+				rt_global_restore_flags(flags);
+				return RTE_UNBLKD;
+			} else {
+				rt_current->prio_passed_to = NULL;
+				rt_global_restore_flags(flags);
+				return RTE_OBJREM;
+			}
 		}
 	} else {
 		sem->count--;
 	}
 	if (sem->type > 0) {
-		(sem->owndby = rt_current)->owndres++;
+		enqueue_resqel(&sem->resq, sem->owndby = rt_current);
 	}
 	rt_global_restore_flags(flags);
 	return count;
@@ -497,26 +518,30 @@ int rt_sem_wait(SEM *sem)
  *	 up to such number  of events, in any case avoid counting up
  *	 to 0xFFFF.
  */
-int rt_sem_wait_if(SEM *sem)
+RTAI_SYSCALL_MODE int rt_sem_wait_if(SEM *sem)
 {
 	int count;
 	unsigned long flags;
 
 	if (sem->magic != RT_SEM_MAGIC) {
-		return SEM_ERR;
+		return RTE_OBJINV;
 	}
 
 	flags = rt_global_save_flags_and_cli();
 	if ((count = sem->count) <= 0) {
-		if (sem->type > 0 && sem->owndby == RT_CURRENT) {
-			count = sem->type++;
+		if (sem->restype && sem->owndby == RT_CURRENT) {
+			if (sem->restype > 0) {
+				count = sem->type++;
+				rt_global_restore_flags(flags);
+				return count + 1;
+			}
 			rt_global_restore_flags(flags);
-			return count + 1;
+			return RTE_DEADLOK;
 		}
 	} else {
 		sem->count--;
 		if (sem->type > 0) {
-			(sem->owndby = RT_CURRENT)->owndres++;
+			enqueue_resqel(&sem->resq, sem->owndby = RT_CURRENT);
 		}
 	}
 	rt_global_restore_flags(flags);
@@ -558,27 +583,34 @@ int rt_sem_wait_if(SEM *sem)
  *	 up to such number of events, in any case avoid counting up to
  *	 0xFFFF.
  */
-int rt_sem_wait_until(SEM *sem, RTIME time)
+RTAI_SYSCALL_MODE int rt_sem_wait_until(SEM *sem, RTIME time)
 {
 	DECLARE_RT_CURRENT;
 	int count;
 	unsigned long flags;
 
 	if (sem->magic != RT_SEM_MAGIC) {
-		return SEM_ERR;
+		return RTE_OBJINV;
 	}
 
+	REALTIME2COUNT(time);
+
 	flags = rt_global_save_flags_and_cli();
 	ASSIGN_RT_CURRENT;
 	if ((count = sem->count) <= 0) {
+		void *retp;
 		rt_current->blocked_on = &sem->queue;
 		if ((rt_current->resume_time = time) > rt_time_h) {
 			unsigned long schedmap;
 			if (sem->type > 0) {
-				if (sem->owndby == rt_current) {
-					count = sem->type++;
+				if (sem->restype && sem->owndby == rt_current) {
+					if (sem->restype > 0) {
+						count = sem->type++;
+						rt_global_restore_flags(flags);
+						return count + 1;
+					}
 					rt_global_restore_flags(flags);
-					return count + 1;
+					return RTE_DEADLOK;
 				}
 				schedmap = pass_prio(sem->owndby, rt_current);
 			} else {
@@ -594,27 +626,28 @@ int rt_sem_wait_until(SEM *sem, RTIME time)
 			sem->count--;
 			rt_current->queue.prev = rt_current->queue.next = &rt_current->queue;
 		}
-		if (sem->magic != RT_SEM_MAGIC) {
-			rt_current->prio_passed_to = NOTHING;
-			rt_global_restore_flags(flags);
-			return SEM_ERR;
-		} else {
-			if (rt_current->blocked_on) {
-				dequeue_blocked(rt_current);
-				if(++sem->count > 1 && sem->type) {
-					sem->count = 1;
-				}
-				rt_global_restore_flags(flags);
-				return SEM_TIMOUT;
-			} else {
-				count = sem->count;
+		if (likely(!(retp = rt_current->blocked_on))) { 
+			count = sem->count;
+		} else if (likely(retp != RTP_OBJREM)) { 
+			dequeue_blocked(rt_current);
+			if (++sem->count > 1 && sem->type) {
+				sem->count = 1;
 			}
+			if (sem->owndby && sem->type > 0) {
+				set_task_prio_from_resq(sem->owndby);
+			}
+			rt_global_restore_flags(flags);
+			return likely(retp > RTP_HIGERR) ? RTE_TIMOUT : RTE_UNBLKD;
+		} else {
+			rt_current->prio_passed_to = NULL;
+			rt_global_restore_flags(flags);
+			return RTE_OBJREM;
 		}
 	} else {
 		sem->count--;
 	}
 	if (sem->type > 0) {
-		(sem->owndby = rt_current)->owndres++;
+		enqueue_resqel(&sem->resq, sem->owndby = rt_current);
 	}
 	rt_global_restore_flags(flags);
 	return count;
@@ -655,7 +688,7 @@ int rt_sem_wait_until(SEM *sem, RTIME time)
  *	 up to such number of events, in any case avoid counting up to
  *	 0xFFFF.
  */
-int rt_sem_wait_timed(SEM *sem, RTIME delay)
+RTAI_SYSCALL_MODE int rt_sem_wait_timed(SEM *sem, RTIME delay)
 {
 	return rt_sem_wait_until(sem, get_time() + delay);
 }
@@ -671,26 +704,28 @@ int rt_sem_wait_timed(SEM *sem, RTIME delay)
  * a request will be blocked till a number of tasks equal to the semaphore
  * count set at rt_sem_init is reached.
  *
- * @returns 0 always.
+ * @returns -1 for tasks that waited on the barrier, 0 for the tasks that 
+ * completed the barrier count.
  */
-int rt_sem_wait_barrier(SEM *sem)
+RTAI_SYSCALL_MODE int rt_sem_wait_barrier(SEM *sem)
 {
 	unsigned long flags;
 
 	if (sem->magic != RT_SEM_MAGIC) {
-		return SEM_ERR;
+		return RTE_OBJINV;
 	}
 
 	flags = rt_global_save_flags_and_cli();
 	if (!sem->owndby) {
-		sem->owndby = (void *)(sem->count < 1 ? 1 : sem->count);
+		sem->owndby = (void *)(long)(sem->count < 1 ? 1 : sem->count);
 		sem->count = sem->type = 0;
 	}
-	if ((1 - sem->count) < (int)sem->owndby) {
+	if ((1 - sem->count) < (long)sem->owndby) {
 		rt_sem_wait(sem);
-	} else {
-		rt_sem_broadcast(sem);
+		rt_global_restore_flags(flags);
+		return -1;
 	}
+	rt_sem_broadcast(sem);
 	rt_global_restore_flags(flags);
 	return 0;
 }
@@ -712,13 +747,13 @@ int rt_sem_wait_barrier(SEM *sem)
  * @returns 0
  *
  */
-int rt_cond_signal(CND *cnd)
+RTAI_SYSCALL_MODE int rt_cond_signal(CND *cnd)
 {
 	unsigned long flags;
 	RT_TASK *task;
 
 	if (cnd->magic != RT_SEM_MAGIC) {
-		return SEM_ERR;
+		return RTE_OBJINV;
 	}
 	flags = rt_global_save_flags_and_cli();
 	if ((task = (cnd->queue.next)->task)) {
@@ -738,8 +773,9 @@ static inline int rt_cndmtx_signal(SEM *mtx, RT_TASK *rt_current)
 	int type;
 	RT_TASK *task;
 
-	type = mtx->type;
-	mtx->type = 1;
+	if ((type = mtx->type) > 1) {
+		mtx->type = 1;
+	}
 	if (++mtx->count > 1) {
 		mtx->count = 1;
 	}
@@ -749,17 +785,9 @@ static inline int rt_cndmtx_signal(SEM *mtx, RT_TASK *rt_current)
 		if (task->state != RT_SCHED_READY && (task->state &= ~(RT_SCHED_SEMAPHORE | RT_SCHED_DELAYED)) == RT_SCHED_READY) {
 			enq_ready_task(task);
 		}
-	} else {
-		task = 0;
 	}
 	mtx->owndby = 0;
-	if (rt_current->owndres & SEMHLF) {
-		--rt_current->owndres;
-	}
-	if (!(rt_current->owndres & SEMHLF)) {
-		int priority;
-		rt_current->priority = rt_current->base_priority > (priority = ((rt_current->msg_queue.next)->task)->priority) ? priority : rt_current->base_priority;
-	}
+	dequeue_resqel_reset_current_priority(&mtx->resq, rt_current);
 	if (task) {
 		 RT_SCHEDULE_BOTH(task, rtai_cpuid());
 	} else {
@@ -788,28 +816,40 @@ static inline int rt_cndmtx_signal(SEM *mtx, RT_TASK *rt_current)
  * @return 0 on succes, SEM_ERR in case of error.
  *
  */
-int rt_cond_wait(CND *cnd, SEM *mtx)
+RTAI_SYSCALL_MODE int rt_cond_wait(CND *cnd, SEM *mtx)
 {
 	RT_TASK *rt_current;
 	unsigned long flags;
+	void *retp;
 	int retval, type;
 
 	if (cnd->magic != RT_SEM_MAGIC || mtx->magic != RT_SEM_MAGIC) {
-		return SEM_ERR;
+		return RTE_OBJINV;
 	}
-	retval = 0;
 	flags = rt_global_save_flags_and_cli();
 	rt_current = RT_CURRENT;
+	if (mtx->owndby != rt_current) {
+		rt_global_restore_flags(flags);
+		return RTE_PERM;
+	}
 	rt_current->state |= RT_SCHED_SEMAPHORE;
 	rem_ready_current(rt_current);
 	enqueue_blocked(rt_current, &cnd->queue, cnd->qtype);
 	type = rt_cndmtx_signal(mtx, rt_current);
-	if (rt_current->blocked_on || cnd->magic != RT_SEM_MAGIC) {
-		retval = SEM_ERR;
+	if (likely((retp = rt_current->blocked_on) != RTP_OBJREM)) { 
+		if (unlikely(retp != NULL)) {
+			dequeue_blocked(rt_current);
+                        retval = RTE_UNBLKD;
+		} else {
+			retval = 0;
+		}
+	} else {
+		retval = RTE_OBJREM;
 	}
 	rt_global_restore_flags(flags);
-	rt_sem_wait(mtx);
-	mtx->type = type;
+	if (rt_sem_wait(mtx) < RTE_LOWERR) {
+		mtx->type = type;
+	}
 	return retval;
 }
 
@@ -837,40 +877,47 @@ int rt_cond_wait(CND *cnd, SEM *mtx)
  * if the task has been resumed because of any other action (likely cnd
  * was deleted).
  */
-int rt_cond_wait_until(CND *cnd, SEM *mtx, RTIME time)
+RTAI_SYSCALL_MODE int rt_cond_wait_until(CND *cnd, SEM *mtx, RTIME time)
 {
 	DECLARE_RT_CURRENT;
 	unsigned long flags;
+	void *retp;
 	int retval, type;
 
 	if (cnd->magic != RT_SEM_MAGIC && mtx->magic != RT_SEM_MAGIC) {
-		return SEM_ERR;
+		return RTE_OBJINV;
 	}
-	retval = SEM_TIMOUT;
+
+	REALTIME2COUNT(time);
+
 	flags = rt_global_save_flags_and_cli();
 	ASSIGN_RT_CURRENT;
+	if (mtx->owndby != rt_current) {
+		rt_global_restore_flags(flags);
+		return RTE_PERM;
+	}
 	if ((rt_current->resume_time = time) > rt_time_h) {
 		rt_current->state |= (RT_SCHED_SEMAPHORE | RT_SCHED_DELAYED);
 		rem_ready_current(rt_current);
 		enqueue_blocked(rt_current, &cnd->queue, cnd->qtype);
 		enq_timed_task(rt_current);
 		type = rt_cndmtx_signal(mtx, rt_current);
-		if (cnd->magic != RT_SEM_MAGIC) {
-			retval = SEM_ERR;
+		if (unlikely((retp = rt_current->blocked_on) == RTP_OBJREM)) { 
+                        retval = RTE_OBJREM;
+		} else if (unlikely(retp != NULL)) {
+			dequeue_blocked(rt_current);
+			retval = likely(retp > RTP_HIGERR) ? RTE_TIMOUT : RTE_UNBLKD;
 		} else {
-			if (rt_current->blocked_on) {
-				dequeue_blocked(rt_current);
-			} else {
-				retval = 0;
-			}
+			retval = 0;
+		}
+		rt_global_restore_flags(flags);
+		if (rt_sem_wait(mtx) < RTE_LOWERR) {
+			mtx->type = type;
 		}
 	} else {
+		retval = RTE_TIMOUT;
 		rt_global_restore_flags(flags);
-		return retval;
 	}
-	rt_global_restore_flags(flags);
-	rt_sem_wait(mtx);
-	mtx->type = type;
 	return retval;
 }
 
@@ -899,7 +946,7 @@ int rt_cond_wait_until(CND *cnd, SEM *mtx, RTIME time)
  * if the task has been resumed because of any other action (likely cnd
  * was deleted).
  */
-int rt_cond_wait_timed(CND *cnd, SEM *mtx, RTIME delay)
+RTAI_SYSCALL_MODE int rt_cond_wait_timed(CND *cnd, SEM *mtx, RTIME delay)
 {
 	return rt_cond_wait_until(cnd, mtx, get_time() + delay);
 }
@@ -925,11 +972,11 @@ int rt_cond_wait_timed(CND *cnd, SEM *mtx, RTIME delay)
  *
  */
 
-int rt_rwl_init(RWL *rwl)
+RTAI_SYSCALL_MODE int rt_typed_rwl_init(RWL *rwl, int type)
 {
-	rt_typed_sem_init(&rwl->wrmtx, 1, RES_SEM);
-	rt_typed_sem_init(&rwl->wrsem, 0, CNT_SEM);
-	rt_typed_sem_init(&rwl->rdsem, 0, CNT_SEM);
+	rt_typed_sem_init(&rwl->wrmtx, type, RES_SEM);
+	rt_typed_sem_init(&rwl->wrsem, 0, CNT_SEM | PRIO_Q);
+	rt_typed_sem_init(&rwl->rdsem, 0, CNT_SEM | PRIO_Q);
 	return 0;
 }
 
@@ -945,14 +992,14 @@ int rt_rwl_init(RWL *rwl)
  *
  */
 
-int rt_rwl_delete(RWL *rwl)
+RTAI_SYSCALL_MODE int rt_rwl_delete(RWL *rwl)
 {
 	int ret;
 
 	ret  =  rt_sem_delete(&rwl->rdsem);
-	ret |= !rt_sem_delete(&rwl->wrsem);
-	ret |= !rt_sem_delete(&rwl->wrmtx);
-	return ret ? 0 : SEM_ERR;
+	ret |= rt_sem_delete(&rwl->wrsem);
+	ret |= rt_sem_delete(&rwl->wrmtx);
+	return !ret ? 0 : RTE_OBJINV;
 }
 
 /**
@@ -970,7 +1017,7 @@ int rt_rwl_delete(RWL *rwl)
  *
  */
 
-int rt_rwl_rdlock(RWL *rwl)
+RTAI_SYSCALL_MODE int rt_rwl_rdlock(RWL *rwl)
 {
 	unsigned long flags;
 	RT_TASK *wtask, *rt_current;
@@ -981,14 +1028,14 @@ int rt_rwl_rdlock(RWL *rwl)
 		int ret;
 		if (rwl->wrmtx.owndby == rt_current) {
 			rt_global_restore_flags(flags);
-			return SEM_ERR + 1;
+			return RTE_RWLINV;
 		}
-		if ((ret = rt_sem_wait(&rwl->rdsem)) >= SEM_TIMOUT) {
+		if ((ret = rt_sem_wait(&rwl->rdsem)) >= RTE_LOWERR) {
 			rt_global_restore_flags(flags);
 			return ret;
 		}
 	}
-	((int *)&rwl->rdsem.owndby)[0]++;
+	((volatile int *)&rwl->rdsem.owndby)[0]++;
 	rt_global_restore_flags(flags);
 	return 0;
 }
@@ -1007,14 +1054,14 @@ int rt_rwl_rdlock(RWL *rwl)
  *
  */
 
-int rt_rwl_rdlock_if(RWL *rwl)
+RTAI_SYSCALL_MODE int rt_rwl_rdlock_if(RWL *rwl)
 {
 	unsigned long flags;
 	RT_TASK *wtask;
 
 	flags = rt_global_save_flags_and_cli();
 	if (!rwl->wrmtx.owndby && (!(wtask = (rwl->wrsem.queue.next)->task) || wtask->priority > RT_CURRENT->priority)) {
-		((int *)&rwl->rdsem.owndby)[0]++;
+		((volatile int *)&rwl->rdsem.owndby)[0]++;
 		rt_global_restore_flags(flags);
 		return 0;
 	}
@@ -1040,7 +1087,7 @@ int rt_rwl_rdlock_if(RWL *rwl)
  *
  */
 
-int rt_rwl_rdlock_until(RWL *rwl, RTIME time)
+RTAI_SYSCALL_MODE int rt_rwl_rdlock_until(RWL *rwl, RTIME time)
 {
 	unsigned long flags;
 	RT_TASK *wtask, *rt_current;
@@ -1051,14 +1098,14 @@ int rt_rwl_rdlock_until(RWL *rwl, RTIME time)
 		int ret;
 		if (rwl->wrmtx.owndby == rt_current) {
 			rt_global_restore_flags(flags);
-			return SEM_ERR + 1;
+			return RTE_RWLINV;
 		}
-		if ((ret = rt_sem_wait_until(&rwl->rdsem, time)) >= SEM_TIMOUT) {
+		if ((ret = rt_sem_wait_until(&rwl->rdsem, time)) >= RTE_LOWERR) {
 			rt_global_restore_flags(flags);
 			return ret;
 		}
 	}
-	((int *)&rwl->rdsem.owndby)[0]++;
+	((volatile int *)&rwl->rdsem.owndby)[0]++;
 	rt_global_restore_flags(flags);
 	return 0;
 }
@@ -1082,7 +1129,7 @@ int rt_rwl_rdlock_until(RWL *rwl, RTIME time)
  *
  */
 
-int rt_rwl_rdlock_timed(RWL *rwl, RTIME delay)
+RTAI_SYSCALL_MODE int rt_rwl_rdlock_timed(RWL *rwl, RTIME delay)
 {
 	return rt_rwl_rdlock_until(rwl, get_time() + delay);
 }
@@ -1101,19 +1148,19 @@ int rt_rwl_rdlock_timed(RWL *rwl, RTIME delay)
  *
  */
 
-int rt_rwl_wrlock(RWL *rwl)
+RTAI_SYSCALL_MODE int rt_rwl_wrlock(RWL *rwl)
 {
 	unsigned long flags;
 	int ret;
 
 	flags = rt_global_save_flags_and_cli();
 	while (rwl->rdsem.owndby) {
-		if ((ret = rt_sem_wait(&rwl->wrsem)) >= SEM_TIMOUT) {
+		if ((ret = rt_sem_wait(&rwl->wrsem)) >= RTE_LOWERR) {
 			rt_global_restore_flags(flags);
 			return ret;
 		}
 	}
-	if ((ret = rt_sem_wait(&rwl->wrmtx)) >= SEM_TIMOUT) {
+	if ((ret = rt_sem_wait(&rwl->wrmtx)) >= RTE_LOWERR) {
 		rt_global_restore_flags(flags);
 		return ret;
 	}
@@ -1134,12 +1181,13 @@ int rt_rwl_wrlock(RWL *rwl)
  *
  */
 
-int rt_rwl_wrlock_if(RWL *rwl)
+RTAI_SYSCALL_MODE int rt_rwl_wrlock_if(RWL *rwl)
 {
 	unsigned long flags;
+	int ret;
 
 	flags = rt_global_save_flags_and_cli();
-	if (!rwl->rdsem.owndby && rt_sem_wait_if(&rwl->wrmtx) >= 0) {
+	if (!rwl->rdsem.owndby && (ret = rt_sem_wait_if(&rwl->wrmtx)) > 0 && ret  < RTE_LOWERR) {
 		rt_global_restore_flags(flags);
 		return 0;
 	}
@@ -1165,19 +1213,19 @@ int rt_rwl_wrlock_if(RWL *rwl)
  *
  */
 
-int rt_rwl_wrlock_until(RWL *rwl, RTIME time)
+RTAI_SYSCALL_MODE int rt_rwl_wrlock_until(RWL *rwl, RTIME time)
 {
 	unsigned long flags;
 	int ret;
 
 	flags = rt_global_save_flags_and_cli();
 	while (rwl->rdsem.owndby) {
-		if ((ret = rt_sem_wait_until(&rwl->wrsem, time)) >= SEM_TIMOUT) {
+		if ((ret = rt_sem_wait_until(&rwl->wrsem, time)) >= RTE_LOWERR) {
 			rt_global_restore_flags(flags);
 			return ret;
 		};
 	}
-	if ((ret = rt_sem_wait_until(&rwl->wrmtx, time)) >= SEM_TIMOUT) {
+	if ((ret = rt_sem_wait_until(&rwl->wrmtx, time)) >= RTE_LOWERR) {
 		rt_global_restore_flags(flags);
 		return ret;
 	};
@@ -1204,7 +1252,7 @@ int rt_rwl_wrlock_until(RWL *rwl, RTIME time)
  *
  */
 
-int rt_rwl_wrlock_timed(RWL *rwl, RTIME delay)
+RTAI_SYSCALL_MODE int rt_rwl_wrlock_timed(RWL *rwl, RTIME delay)
 {
 	return rt_rwl_wrlock_until(rwl, get_time() + delay);
 }
@@ -1224,15 +1272,18 @@ int rt_rwl_wrlock_timed(RWL *rwl, RTIME delay)
  *
  */
 
-int rt_rwl_unlock(RWL *rwl)
+RTAI_SYSCALL_MODE int rt_rwl_unlock(RWL *rwl)
 {
 	unsigned long flags;
 
 	flags = rt_global_save_flags_and_cli();
-	if (rwl->wrmtx.owndby) {
+	if (rwl->wrmtx.owndby == RT_CURRENT) {
 		rt_sem_signal(&rwl->wrmtx);
 	} else if (rwl->rdsem.owndby) {
-	           rwl->rdsem.owndby = (struct rt_task_struct *)((char *)rwl->rdsem.owndby - 1);
+		((volatile int *)&rwl->rdsem.owndby)[0]--;
+	} else {
+		rt_global_restore_flags(flags);
+		return RTE_PERM;
 	}
 	rt_global_restore_flags(flags);
 	flags = rt_global_save_flags_and_cli();
@@ -1241,15 +1292,15 @@ int rt_rwl_unlock(RWL *rwl)
 		wtask = (rwl->wrsem.queue.next)->task;
 		rtask = (rwl->rdsem.queue.next)->task;
 		if (wtask && rtask) {
-			if (wtask->priority < rtask->priority) {
+			if (wtask->priority <= rtask->priority) {
 				rt_sem_signal(&rwl->wrsem);
 			} else {
-				rt_sem_signal(&rwl->rdsem);
+				rt_sem_broadcast(&rwl->rdsem);
 			}
 		} else if (wtask) {
 			rt_sem_signal(&rwl->wrsem);
 		} else if (rtask) {
-			rt_sem_signal(&rwl->rdsem);
+			rt_sem_broadcast(&rwl->rdsem);
 		}
         }
 	rt_global_restore_flags(flags);
@@ -1277,7 +1328,7 @@ int rt_rwl_unlock(RWL *rwl)
  *
  */
 
-int rt_spl_init(SPL *spl)
+RTAI_SYSCALL_MODE int rt_spl_init(SPL *spl)
 {
 	spl->owndby = 0;
 	spl->count  = 0;
@@ -1296,7 +1347,7 @@ int rt_spl_init(SPL *spl)
  *
  */
 
-int rt_spl_delete(SPL *spl)
+RTAI_SYSCALL_MODE int rt_spl_delete(SPL *spl)
 {
         return 0;
 }
@@ -1317,7 +1368,7 @@ int rt_spl_delete(SPL *spl)
  *
  */
 
-int rt_spl_lock(SPL *spl)
+RTAI_SYSCALL_MODE int rt_spl_lock(SPL *spl)
 {
 	unsigned long flags;
 	RT_TASK *rt_current;
@@ -1326,9 +1377,10 @@ int rt_spl_lock(SPL *spl)
 	if (spl->owndby == (rt_current = RT_CURRENT)) {
 		spl->count++;
 	} else {
-		while (atomic_cmpxchg(&spl->owndby, 0, rt_current));
+		while (cmpxchg(&spl->owndby, 0L, rt_current));
 		spl->flags = flags;
 	}
+	rtai_restore_flags(flags);
 	return 0;
 }
 
@@ -1347,7 +1399,7 @@ int rt_spl_lock(SPL *spl)
  *
  */
 
-int rt_spl_lock_if(SPL *spl)
+RTAI_SYSCALL_MODE int rt_spl_lock_if(SPL *spl)
 {
 	unsigned long flags;
 	RT_TASK *rt_current;
@@ -1356,12 +1408,13 @@ int rt_spl_lock_if(SPL *spl)
 	if (spl->owndby == (rt_current = RT_CURRENT)) {
 		spl->count++;
 	} else {
-		if (atomic_cmpxchg(&spl->owndby, 0, rt_current)) {
+		if (cmpxchg(&spl->owndby, 0L, rt_current)) {
 			rtai_restore_flags(flags);
 			return -1;
 		}
 		spl->flags = flags;
 	}
+	rtai_restore_flags(flags);
 	return 0;
 }
 
@@ -1386,7 +1439,7 @@ int rt_spl_lock_if(SPL *spl)
  *
  */
 
-int rt_spl_lock_timed(SPL *spl, unsigned long ns)
+RTAI_SYSCALL_MODE int rt_spl_lock_timed(SPL *spl, unsigned long ns)
 {
 	unsigned long flags;
 	RT_TASK *rt_current;
@@ -1396,15 +1449,16 @@ int rt_spl_lock_timed(SPL *spl, unsigned long ns)
 		spl->count++;
 	} else {
 		RTIME end_time;
-		void *locked;
+		long locked;
 		end_time = rdtsc() + imuldiv(ns, tuned.cpu_freq, 1000000000);
-		while ((locked = atomic_cmpxchg(&spl->owndby, 0, rt_current)) && rdtsc() < end_time);
+		while ((locked = (long)cmpxchg(&spl->owndby, 0L, rt_current)) && rdtsc() < end_time);
 		if (locked) {
 			rtai_restore_flags(flags);
 			return -1;
 		}
 		spl->flags = flags;
 	}
+	rtai_restore_flags(flags);
 	return 0;
 }
 
@@ -1425,7 +1479,7 @@ int rt_spl_lock_timed(SPL *spl, unsigned long ns)
  *
  */
 
-int rt_spl_unlock(SPL *spl)
+RTAI_SYSCALL_MODE int rt_spl_unlock(SPL *spl)
 {
 	unsigned long flags;
 	RT_TASK *rt_current;
@@ -1437,8 +1491,8 @@ int rt_spl_unlock(SPL *spl)
 		} else {
 			spl->owndby = 0;
 			spl->count  = 0;
-			rtai_restore_flags(spl->flags);
 		}
+		rtai_restore_flags(spl->flags);
 		return 0;
 	}
 	rtai_restore_flags(flags);
@@ -1490,11 +1544,18 @@ int rt_spl_unlock(SPL *spl)
  *
  */
 
-SEM *_rt_typed_named_sem_init(unsigned long sem_name, int value, int type)
+RTAI_SYSCALL_MODE SEM *_rt_typed_named_sem_init(unsigned long sem_name, int value, int type, unsigned long *handle)
 {
 	SEM *sem;
 
 	if ((sem = rt_get_adr_cnt(sem_name))) {
+		if (handle) {
+			if ((unsigned long)handle > PAGE_OFFSET) {
+				*handle = 1;
+			} else {
+				rt_copy_to_user(handle, sem, sizeof(SEM *));
+			}
+		}
 		return sem;
 	}
 	if ((sem = rt_malloc(sizeof(SEM)))) {
@@ -1529,7 +1590,7 @@ SEM *_rt_typed_named_sem_init(unsigned long sem_name, int value, int type)
  *
  */
 
-int rt_named_sem_delete(SEM *sem)
+RTAI_SYSCALL_MODE int rt_named_sem_delete(SEM *sem)
 {
 	int ret;
 	if (!(ret = rt_drg_on_adr_cnt(sem))) {
@@ -1537,7 +1598,7 @@ int rt_named_sem_delete(SEM *sem)
 			rt_free(sem);
 			return 0;
 		} else {
-			return SEM_ERR;
+			return RTE_OBJINV;
 		}
 	}
 	return ret;
@@ -1572,7 +1633,7 @@ int rt_named_sem_delete(SEM *sem)
  *
  */
 
-RWL *_rt_named_rwl_init(unsigned long rwl_name)
+RTAI_SYSCALL_MODE RWL *_rt_named_rwl_init(unsigned long rwl_name)
 {
 	RWL *rwl;
 
@@ -1610,7 +1671,7 @@ RWL *_rt_named_rwl_init(unsigned long rwl_name)
  *
  */
 
-int rt_named_rwl_delete(RWL *rwl)
+RTAI_SYSCALL_MODE int rt_named_rwl_delete(RWL *rwl)
 {
 	int ret;
 	if (!(ret = rt_drg_on_adr_cnt(rwl))) {
@@ -1618,7 +1679,7 @@ int rt_named_rwl_delete(RWL *rwl)
 			rt_free(rwl);
 			return 0;
 		} else {
-			return SEM_ERR;
+			return RTE_OBJINV;
 		}
 	}
 	return ret;
@@ -1652,7 +1713,7 @@ int rt_named_rwl_delete(RWL *rwl)
  *
  */
 
-SPL *_rt_named_spl_init(unsigned long spl_name)
+RTAI_SYSCALL_MODE SPL *_rt_named_spl_init(unsigned long spl_name)
 {
 	SPL *spl;
 
@@ -1689,7 +1750,7 @@ SPL *_rt_named_spl_init(unsigned long spl_name)
  *
  */
 
-int rt_named_spl_delete(SPL *spl)
+RTAI_SYSCALL_MODE int rt_named_spl_delete(SPL *spl)
 {
 	int ret;
 	if (!(ret = rt_drg_on_adr_cnt(spl))) {
@@ -1708,17 +1769,18 @@ struct rt_native_fun_entry rt_sem_entries[] = {
 	{ { 0, _rt_typed_named_sem_init }, NAMED_SEM_INIT },
 	{ { 0, rt_named_sem_delete },      NAMED_SEM_DELETE },
 	{ { 1, rt_sem_signal },            SEM_SIGNAL },
+	{ { 1, rt_sem_broadcast },         SEM_BROADCAST },
 	{ { 1, rt_sem_wait },              SEM_WAIT },
 	{ { 1, rt_sem_wait_if },           SEM_WAIT_IF },
 	{ { 1, rt_sem_wait_until },        SEM_WAIT_UNTIL },
 	{ { 1, rt_sem_wait_timed },        SEM_WAIT_TIMED },
-	{ { 1, rt_sem_broadcast },         SEM_BROADCAST },
 	{ { 1, rt_sem_wait_barrier },      SEM_WAIT_BARRIER },
 	{ { 1, rt_sem_count },             SEM_COUNT },
+        { { 1, rt_cond_signal}, 	   COND_SIGNAL },
         { { 1, rt_cond_wait },             COND_WAIT },
         { { 1, rt_cond_wait_until },       COND_WAIT_UNTIL },
         { { 1, rt_cond_wait_timed },       COND_WAIT_TIMED },
-        { { 0, rt_rwl_init },              RWL_INIT },
+        { { 0, rt_typed_rwl_init },        RWL_INIT },
         { { 0, rt_rwl_delete },            RWL_DELETE },
 	{ { 0, _rt_named_rwl_init },	   NAMED_RWL_INIT },
 	{ { 0, rt_named_rwl_delete },      NAMED_RWL_DELETE },
@@ -1739,7 +1801,6 @@ struct rt_native_fun_entry rt_sem_entries[] = {
         { { 1, rt_spl_lock_if },           SPL_LOCK_IF },
         { { 1, rt_spl_lock_timed },        SPL_LOCK_TIMED },
         { { 1, rt_spl_unlock },            SPL_UNLOCK },
-        { { 1, rt_cond_signal}, 	   COND_SIGNAL },
 	{ { 0, 0 },  		           000 }
 };
 
@@ -1785,7 +1846,7 @@ EXPORT_SYMBOL(rt_cond_wait);
 EXPORT_SYMBOL(rt_cond_wait_until);
 EXPORT_SYMBOL(rt_cond_wait_timed);
 
-EXPORT_SYMBOL(rt_rwl_init);
+EXPORT_SYMBOL(rt_typed_rwl_init);
 EXPORT_SYMBOL(rt_rwl_delete);
 EXPORT_SYMBOL(rt_rwl_rdlock);
 EXPORT_SYMBOL(rt_rwl_rdlock_if);
