@@ -33,8 +33,8 @@
 
 #include <linux/version.h>
 #include <linux/module.h>
-#include <linux/config.h>
 #include <linux/errno.h>
+#include <linux/fs.h>
 #include <linux/mm.h>
 #include <linux/miscdevice.h>
 
@@ -133,12 +133,12 @@ void *rt_shm_alloc(unsigned long name, int size, int suprt)
 	return _rt_shm_alloc(name, size, suprt);
 }
 
-static int rt_shm_alloc_usp(unsigned long name, int size, int suprt)
+static RTAI_SYSCALL_MODE int rt_shm_alloc_usp(unsigned long name, int size, int suprt)
 {
 	TRACE_RTAI_SHM(TRACE_RTAI_EV_SHM_MALLOC, name, size, current->pid);
 
 	if (_rt_shm_alloc(name, size, suprt)) {
-		((current->mm)->mmap)->vm_private_data = (void *)name;
+		current->rtai_tskext(TSKEXT1) = (void *)name;
 		return abs(rt_get_type(name));
 	}
 	return 0;
@@ -164,13 +164,13 @@ static int rt_shm_alloc_usp(unsigned long name, int size, int suprt)
  *
  */
 
-int rt_shm_free(unsigned long name)
+RTAI_SYSCALL_MODE int rt_shm_free(unsigned long name)
 {
 	TRACE_RTAI_SHM(TRACE_RTAI_EV_SHM_KFREE, name, 0, 0);
 	return _rt_shm_free(name, rt_get_type(name));
 }
 
-static int rt_shm_size(unsigned long *arg)
+static RTAI_SYSCALL_MODE int rt_shm_size(unsigned long *arg)
 {
 	int size;
 	struct vm_area_struct *vma;
@@ -200,14 +200,16 @@ static struct vm_operations_struct rtai_shm_vm_ops = {
 	close: 	rtai_shm_vm_close
 };
 
-static void rt_set_heap(unsigned long, void *);
+#ifdef CONFIG_RTAI_MALLOC
+static RTAI_SYSCALL_MODE void rt_set_heap(unsigned long, void *);
+#endif
 
 static int rtai_shm_f_ioctl(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg)
 {
 	switch (cmd) {
 		case SHM_ALLOC: {
 			TRACE_RTAI_SHM(TRACE_RTAI_EV_SHM_MALLOC, ((unsigned long *)arg)[0], cmd, current->pid);
-			return rt_shm_alloc_usp(((unsigned long *)arg)[0], ((int *)arg)[1], ((int *)arg)[2]);
+			return rt_shm_alloc_usp(((unsigned long *)arg)[0], ((long *)arg)[1], ((long *)arg)[2]);
 		}
 		case SHM_FREE: {
 			TRACE_RTAI_SHM(TRACE_RTAI_EV_SHM_FREE, arg, cmd, current->pid);
@@ -217,10 +219,12 @@ static int rtai_shm_f_ioctl(struct inode *inode, struct file *file, unsigned int
 			TRACE_RTAI_SHM(TRACE_RTAI_EV_SHM_GET_SIZE, arg, cmd, current->pid);
 			return rt_shm_size((unsigned long *)((unsigned long *)arg)[0]);
 		}
+#ifdef CONFIG_RTAI_MALLOC
 		case HEAP_SET: {
 			rt_set_heap(((unsigned long *)arg)[0], (void *)((unsigned long *)arg)[1]);
 			return 0;
 		}
+#endif
 	}
 	return 0;
 }
@@ -232,8 +236,8 @@ static int rtai_shm_f_mmap(struct file *file, struct vm_area_struct *vma)
 	if (!vma->vm_ops) {
 		vma->vm_ops = &rtai_shm_vm_ops;
 		vma->vm_flags |= VM_LOCKED;
-		name = (unsigned long)(vma->vm_private_data = ((current->mm)->mmap)->vm_private_data);
-		((current->mm)->mmap)->vm_private_data = NULL;
+		name = (unsigned long)(vma->vm_private_data = current->rtai_tskext(TSKEXT1));
+		current->rtai_tskext(TSKEXT1) = current->rtai_tskext(TSKEXT0) ? current : NULL;
 		return (size = rt_get_type(name)) < 0 ? rkmmap(ALIGN2PAGE(rt_get_adr(name)), -size, vma) : rvmmap(rt_get_adr(name), size, vma);
 	}
 	return -EFAULT;
@@ -247,6 +251,8 @@ static struct file_operations rtai_shm_fops = {
 static struct miscdevice rtai_shm_dev = 
 	{ RTAI_SHM_MISC_MINOR, "RTAI_SHM", &rtai_shm_fops };
 
+#ifdef CONFIG_RTAI_MALLOC
+
 static inline void *_rt_halloc(int size, struct rt_heap_t *heap)
 {
 	void *mem_ptr = NULL;
@@ -373,7 +379,7 @@ void rt_named_free(void *adr)
 #define RTAI_TASK(return_instr) \
 do { \
 	if (!(task = _rt_whoami())->is_hard) { \
-		if (!(task = current->rtai_tskext(0))) { \
+		if (!(task = current->rtai_tskext(TSKEXT0))) { \
 			return_instr; \
 		} \
 	} \
@@ -445,7 +451,7 @@ static inline void rt_named_hfree_typed(void *adr, int htype)
  *
  */
 
-void *rt_halloc(int size)
+RTAI_SYSCALL_MODE void *rt_halloc(int size)
 {
 	return rt_halloc_typed(size, SPECIFIC);
 }
@@ -462,7 +468,7 @@ void *rt_halloc(int size)
  *
  */
 
-void rt_hfree(void *adr)
+RTAI_SYSCALL_MODE void rt_hfree(void *adr)
 {
 	rt_hfree_typed(adr, SPECIFIC);
 }
@@ -498,7 +504,7 @@ void rt_hfree(void *adr)
  *
  */
 
-void *rt_named_halloc(unsigned long name, int size)
+RTAI_SYSCALL_MODE void *rt_named_halloc(unsigned long name, int size)
 {
 	return rt_named_halloc_typed(name, size, SPECIFIC);
 }
@@ -522,7 +528,7 @@ void *rt_named_halloc(unsigned long name, int size)
  *
  */
 
-void rt_named_hfree(void *adr)
+RTAI_SYSCALL_MODE void rt_named_hfree(void *adr)
 {
 	rt_named_hfree_typed(adr, SPECIFIC);
 }
@@ -531,31 +537,31 @@ extern rtheap_t  rtai_global_heap;
 extern void     *rtai_global_heap_adr;
 extern int       rtai_global_heap_size;
 
-static void *rt_malloc_usp(int size)
+static RTAI_SYSCALL_MODE void *rt_malloc_usp(int size)
 {
 	return rtai_global_heap_adr ? rt_halloc_typed(size, GLOBAL) : NULL;
 }
 
-static void rt_free_usp(void *adr)
+static RTAI_SYSCALL_MODE void rt_free_usp(void *adr)
 {
 	if (rtai_global_heap_adr) {
 		rt_hfree_typed(adr, GLOBAL);
 	}
 }
 
-static void *rt_named_malloc_usp(unsigned long name, int size)
+static RTAI_SYSCALL_MODE void *rt_named_malloc_usp(unsigned long name, int size)
 {
 	return rtai_global_heap_adr ? rt_named_halloc_typed(name, size, GLOBAL) : NULL;
 }
 
-static void rt_named_free_usp(void *adr)
+static RTAI_SYSCALL_MODE void rt_named_free_usp(void *adr)
 {
 	if (rtai_global_heap_adr) {
 		rt_named_hfree_typed(adr, GLOBAL);
 	}
 }
 
-static void rt_set_heap(unsigned long name, void *adr)
+static RTAI_SYSCALL_MODE void rt_set_heap(unsigned long name, void *adr)
 {
 	void *heap, *hptr;
 	int size;
@@ -565,8 +571,8 @@ static void rt_set_heap(unsigned long name, void *adr)
 	hptr = ALIGN2PAGE(heap);
 	size = ((abs(rt_get_type(name)) - sizeof(rtheap_t) - (hptr - heap)) & PAGE_MASK);
 	heap = hptr + size;
-	if (!atomic_cmpxchg((int *)hptr, 0, name)) {
-		rtheap_init(heap, hptr, size, PAGE_SIZE);
+	if (!atomic_cmpxchg((atomic_t *)hptr, 0, name)) {
+		rtheap_init(heap, hptr, size, PAGE_SIZE, 0);
 	}
 	RTAI_TASK(return);
 	if (name == GLOBAL_HEAP_ID) {
@@ -625,10 +631,13 @@ void *rt_heap_open(unsigned long name, int size, int suprt)
 	return 0;
 }
 
+#endif
+
 struct rt_native_fun_entry rt_shm_entries[] = {
         { { 0, rt_shm_alloc_usp },		SHM_ALLOC },
         { { 0, rt_shm_free },			SHM_FREE },
         { { 0, rt_shm_size },			SHM_SIZE },
+#ifdef CONFIG_RTAI_MALLOC
         { { 0, rt_set_heap },			HEAP_SET},
         { { 0, rt_halloc },			HEAP_ALLOC },
         { { 0, rt_hfree },			HEAP_FREE },
@@ -638,37 +647,61 @@ struct rt_native_fun_entry rt_shm_entries[] = {
         { { 0, rt_free_usp },			FREE },
         { { 0, rt_named_malloc_usp },		NAMED_MALLOC },
         { { 0, rt_named_free_usp },		NAMED_FREE },
+#endif
         { { 0, 0 },				000 }
 };
 
 extern int set_rt_fun_entries(struct rt_native_fun_entry *entry);
 extern void reset_rt_fun_entries(struct rt_native_fun_entry *entry);
 
+#define USE_UDEV_CLASS 1
+
+#if USE_UDEV_CLASS
+static class_t *shm_class = NULL;
+#endif
+
 int __rtai_shm_init (void)
 {
+#if USE_UDEV_CLASS
+	if ((shm_class = class_create(THIS_MODULE, "rtai_shm")) == NULL) {
+		printk("RTAI-SHM: cannot create class.\n");
+		return -EBUSY;
+	}
+	if (CLASS_DEVICE_CREATE(shm_class, MKDEV(MISC_MAJOR, RTAI_SHM_MISC_MINOR), NULL, "rtai_shm") == NULL) {
+		printk("RTAI-SHM: cannot attach class.\n");
+		class_destroy(shm_class);
+		return -EBUSY;
+	}
+#endif
+
 	if (misc_register(&rtai_shm_dev) < 0) {
 		printk("***** UNABLE TO REGISTER THE SHARED MEMORY DEVICE (miscdev minor: %d) *****\n", RTAI_SHM_MISC_MINOR);
 		return -EBUSY;
 	}
-#ifndef CONFIG_RTAI_MALLOC_VMALLOC
-	printk("***** WARNING: GLOBAL HEAP NEITHER SHARABLE NOR USABLE FROM USER SPACE (use the vmalloc option for RTAI malloc) *****\n");
-#else
+#ifdef CONFIG_RTAI_MALLOC
+#ifdef CONFIG_RTAI_MALLOC_VMALLOC
 	rt_register(GLOBAL_HEAP_ID, rtai_global_heap_adr, rtai_global_heap_size, 0);
 	rt_smp_linux_task->heap[GLOBAL].heap = &rtai_global_heap;
 	rt_smp_linux_task->heap[GLOBAL].kadr =
 	rt_smp_linux_task->heap[GLOBAL].uadr = rtai_global_heap_adr;
+#else
+	printk("***** WARNING: GLOBAL HEAP NEITHER SHARABLE NOR USABLE FROM USER SPACE (use the vmalloc option for RTAI malloc) *****\n");
+#endif
 #endif
 	return set_rt_fun_entries(rt_shm_entries);
 }
 
 void __rtai_shm_exit (void)
 {
+	extern int max_slots;
         int slot;
-        struct rt_registry_entry_struct entry;
+        struct rt_registry_entry entry;
 
+#ifdef CONFIG_RTAI_MALLOC_VMALLOC
 	rt_drg_on_name_cnt(GLOBAL_HEAP_ID);
-	for (slot = 1; slot <= MAX_SLOTS; slot++) {
-		if (rt_get_registry_slot(slot, &entry) && entry.adr) {
+#endif
+	for (slot = 1; slot <= max_slots; slot++) {
+		if (rt_get_registry_slot(slot, &entry)) {
 			if (abs(entry.type) >= PAGE_SIZE) {
         			char name[8];
 				while (_rt_shm_free(entry.name, entry.type));
@@ -679,6 +712,10 @@ void __rtai_shm_exit (void)
 	}
 	reset_rt_fun_entries(rt_shm_entries);
 	misc_deregister(&rtai_shm_dev);
+#if USE_UDEV_CLASS
+	class_device_destroy(shm_class, MKDEV(MISC_MAJOR, RTAI_SHM_MISC_MINOR));
+	class_destroy(shm_class);
+#endif
 	return;
 }
 
@@ -692,6 +729,7 @@ module_exit(__rtai_shm_exit);
 #ifdef CONFIG_KBUILD
 EXPORT_SYMBOL(rt_shm_alloc);
 EXPORT_SYMBOL(rt_shm_free);
+#ifdef CONFIG_RTAI_MALLOC
 EXPORT_SYMBOL(rt_named_malloc);
 EXPORT_SYMBOL(rt_named_free);
 EXPORT_SYMBOL(rt_halloc);
@@ -699,4 +737,5 @@ EXPORT_SYMBOL(rt_hfree);
 EXPORT_SYMBOL(rt_named_halloc);
 EXPORT_SYMBOL(rt_named_hfree);
 EXPORT_SYMBOL(rt_heap_open);
+#endif
 #endif /* CONFIG_KBUILD */
