@@ -2,11 +2,11 @@
  * @ingroup tasklets
  * @file
  *
- * Implementation of the @ref tasklets "mini LXRT RTAI tasklets module".
+ * Implementation of the @ref tasklets "RTAI tasklets module".
  *
  * @author Paolo Mantegazza
  *
- * @note Copyright &copy;1999-2003 Paolo Mantegazza <mantegazza@aero.polimi.it>
+ * @note Copyright &copy; 1999-2006 Paolo Mantegazza <mantegazza@aero.polimi.it>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,75 +24,53 @@
  */
 
 /**
- * @defgroup tasklets mini RTAI LXRT tasklets module
+ * @defgroup tasklets module
  *
- * The MINI_RTAI_LXRT tasklets module adds an interesting new feature along the
- * line, pioneered by RTAI, of a symmetric usage of all its services inter-intra
- * kernel and user space, both for soft and hard real time applications.   In
- * such a way you have opened a whole spectrum of development and implementation
+ * The tasklets module adds an interesting feature along the line, pioneered
+ * by RTAI, of a symmetric usage of all its services inter-intra kernel and 
+ * user space, both for soft and hard real time applications.   In such a way 
+ * you have opened a whole spectrum of development and implementation
  * lanes, allowing maximum flexibility with uncompromized performances.
  *
- * The new services provided can be useful when you have many tasks, both in
- * kernel and user space, that must execute in soft/hard real time but do not
- * need any RTAI scheduler service that could lead to a task block. Such tasks
- * are here called tasklets and can be of two kinds: normal tasklets and timed
- * tasklets (timers).
+ * The new services provided can be useful when you have many tasks, both 
+ * in kernel and user space, that must execute simple, often ripetitive, 
+ * functions, both in soft and hard real time, asynchronously within their 
+ * parent application. Such tasks are here called tasklets and can be of 
+ * two kinds: normal tasklets and timed tasklets (timers).
  *
  * It must be noted that only timers should need to be made available both in
  * user and kernel space.   In fact normal tasklets in kernel space are nothing
  * but standard functions that can be directly executed by calling them, so
  * there would be no need for any special treatment.   However to maintain full
- * usage symmetry, and to ease any possible porting from one address space to
- * the other, also normal tasklet functions can be used in whatever address
+ * usage symmetry and to ease any possible porting from one address space to
+ * the other, plain tasklets can be used in the same way from whatever address 
  * space.
  *
- * Note that if, at this point, you are reminded to similar Linux kernel
- * services you are not totally wrong.  They are not exactly the same, because
- * of their symmetric availability in kernel and user space, but the basic idea
- * behind them  is clearly fairly similar.
- *
- * Tasklets should be used whenever the standard hard real time tasks available
- * with RTAI and LXRT schedulers can be a waist of resources and the execution
- * of simple, possibly timed, functions could often be more than
- * enough. Instances of such applications are timed polling and simple
- * Programmable Logic Controllers (PLC) like sequences of services.   Obviously
- * there are many others instances that can make it sufficient the use of
- * tasklets, either normal or timers.   In general such an approach can be a
- * very useful complement to fully featured tasks in controlling complex
+ * Tasklets should be used where and whenever the standard hard real time 
+ * RTAI tasks are used.  Instances of such applications are timed polling and 
+ * simple Programmable Logic Controllers (PLC) like sequences of services.   
+ * Obviously there are many others instances that can make it sufficient the 
+ * use of tasklets, either normal or timers.   In general such an approach can 
+ * be a very useful complement to fully featured tasks in controlling complex
  * machines and systems, both for basic and support services.
  *
  * It is remarked that the implementation found here for timed tasklets rely on
- * a server support task that executes the related timer functions, either in
- * oneshot or periodic mode, on the base of their time deadline and according to
- * their, user assigned, priority. Instead, as told above, plain tasklets are
- * just functions executed from kernel space; their execution needs no server
- * and is simply triggered by calling a given service function at due time,
- * either from a kernel task or interrupt handler requiring, or in charge of,
- * their execution when they are needed. Once more it is important to recall
- * that all non blocking RTAI scheduler services can be used in any tasklet
- * function.   Blocking services must absolutely be avoided.   They will
- * deadlock the timers server task, executing task or interrupt handler,
- * whichever applies, so that no more tasklet functions will be executed.
- *
- * User and kernel space MINI_RTAI_LXRT applications can cooperate and
- * synchronize by using shared memory. It has been called MINI_RTAI_LXRT because
- * it is a kind of light soft/hard real time server that can partially
- * substitute RTAI and LXRT in simple applications, i.e. if the constraints
- * hinted above are wholly satisfied. So MINI_RTAI_LXRT can be used in kernel
- * and user space, with any RTAI scheduler. Its implementations has been very
- * easy, as it is nothing but what its name implies.   LXRT made all the needed
- * tools already available.   In fact it duplicates a lot of LXRT so that its
- * final production version will be fully integrated with it, ASAP.   However,
- * at the moment, it cannot work with LXRT yet.
+ * server support tasks, one per cpu, that execute the related timer functions, 
+ * either in oneshot or periodic mode, on the base of their time deadline and 
+ * according to their, user assigned, priority. Instead, as told above, plain 
+ * tasklets are just functions executed from kernel space; their execution 
+ * needs no server and is simply triggered by calling a given service function 
+ * at due time, either from a kernel task or interrupt handler requiring, or in
+ * charge of, their execution whenever they are needed.
  *
  * Note that in user space you run within the memory of the process owning the
- * tasklet function so you MUST lock all of your processes memory in core, by
- * using mlockall, to prevent it being swapped out.   Also abundantly pre grow
- * your stack to the largest size needed during the execution of your
- * application, see mlockall usage in Linux manuals.
+ * tasklet function so you MUST lock all of your tasks memory in core, by
+ * using mlockall, to prevent it being swapped out.   Pre grow also your stack 
+ * to the largest size needed during the execution of your application, see 
+ * mlockall usage in Linux mans.
  *
  * The RTAI distribution contains many useful examples that demonstrate the use
- * of most services, both in kernel and user space.
+ * of most tasklets services, both in kernel and user space.
  *
  *@{*/
 
@@ -112,24 +90,35 @@
 
 MODULE_LICENSE("GPL");
 
-#ifdef CONFIG_RTAI_MALLOC
-#define sched_malloc(size)      rt_malloc((size))
-#define sched_free(adr)         rt_free((adr))
-#else
-#define sched_malloc(size)      kmalloc((size), GFP_KERNEL)
-#define sched_free(adr)         kfree((adr))
-#endif
+extern struct epoch_struct boot_epoch;
 
 DEFINE_LINUX_CR0
 
-static struct rt_tasklet_struct timers_list =
-{ &timers_list, &timers_list, RT_SCHED_LOWEST_PRIORITY, 0, RT_TIME_END, 0LL, 0, 0, 0 };
+#ifdef CONFIG_SMP
+#define NUM_CPUS           RTAI_NR_CPUS
+#define TIMED_TIMER_CPUID  (timed_timer->cpuid)
+#define TIMER_CPUID        (timer->cpuid)
+#define LIST_CPUID         (cpuid)
+#else
+#define NUM_CPUS           1
+#define TIMED_TIMER_CPUID  (0)
+#define TIMER_CPUID        (0)
+#define LIST_CPUID         (0)
+#endif
+
+
+static struct rt_tasklet_struct timers_list[NUM_CPUS] =
+{ { &timers_list[0], &timers_list[0], RT_SCHED_LOWEST_PRIORITY, 0, 0, RT_TIME_END, 0LL, NULL, 0UL, 0UL, 0, NULL, NULL, 0, 
+#ifdef  CONFIG_RTAI_LONG_TIMED_LIST
+{ NULL } 
+#endif
+}, };
 
 static struct rt_tasklet_struct tasklets_list =
 { &tasklets_list, &tasklets_list, };
 
+static spinlock_t timers_lock[NUM_CPUS] = { SPIN_LOCK_UNLOCKED, };
 static spinlock_t tasklets_lock = SPIN_LOCK_UNLOCKED;
-static spinlock_t timers_lock   = SPIN_LOCK_UNLOCKED;
 
 static struct rt_fun_entry rt_tasklet_fun[]  __attribute__ ((__unused__));
 
@@ -150,8 +139,74 @@ static struct rt_fun_entry rt_tasklet_fun[] = {
 	{ 0, rt_wait_tasklet_is_hard },	   	//  13
 	{ 0, rt_set_tasklet_priority },  	//  14
 	{ 0, rt_register_task },	  	//  15
+	{ 0, rt_get_timer_times },		//  16	
+	{ 0, rt_get_timer_overrun },		//  17	
+		
+/* Posix timers support */	
+
+	{ 0, rt_ptimer_create },		//  18
+	{ 0, rt_ptimer_settime },		//  19
+	{ 0, rt_ptimer_overrun },		//  20
+	{ 0, rt_ptimer_gettime },		//  21
+	{ 0, rt_ptimer_delete }			//  22	
+	
+/* End Posix timers support */
+	
 };
 
+#ifdef CONFIG_RTAI_LONG_TIMED_LIST
+
+/* BINARY TREE */
+static inline void enq_timer(struct rt_tasklet_struct *timed_timer)
+{
+	struct rt_tasklet_struct *timerh, *tmrnxt, *timer;
+	rb_node_t **rbtn, *rbtpn = NULL;
+	timer = timerh = &timers_list[TIMED_TIMER_CPUID];
+	rbtn = &timerh->rbr.rb_node;
+
+	while (*rbtn) {
+		rbtpn = *rbtn;
+		tmrnxt = rb_entry(rbtpn, struct rt_tasklet_struct, rbn);
+		if (timer->firing_time > tmrnxt->firing_time) {
+			rbtn = &(rbtpn)->rb_right;
+		} else {
+			rbtn = &(rbtpn)->rb_left;
+			timer = tmrnxt;
+		}
+	}
+	rb_link_node(&timed_timer->rbn, rbtpn, rbtn);
+	rb_insert_color(&timed_timer->rbn, &timerh->rbr);
+	timer->prev = (timed_timer->prev = timer->prev)->next = timed_timer;
+	timed_timer->next = timer;
+}
+
+#define rb_erase_timer(timer) \
+rb_erase(&(timer)->rbn, &timers_list[NUM_CPUS > 1 ? (timer)->cpuid : 0].rbr)
+
+#else /* !CONFIG_RTAI_LONG_TIMED_LIST */
+
+/* LINEAR */
+static inline void enq_timer(struct rt_tasklet_struct *timed_timer)
+{
+	struct rt_tasklet_struct *timer;
+	timer = &timers_list[TIMED_TIMER_CPUID];
+        while (timed_timer->firing_time > (timer = timer->next)->firing_time);
+	timer->prev = (timed_timer->prev = timer->prev)->next = timed_timer;
+	timed_timer->next = timer;
+}
+
+#define rb_erase_timer(timer)
+
+#endif /* CONFIG_RTAI_LONG_TIMED_LIST */
+
+static inline void rem_timer(struct rt_tasklet_struct *timer)
+{
+	(timer->next)->prev = timer->prev;
+	(timer->prev)->next = timer->next;
+	timer->next = timer->prev = timer;
+	rb_erase_timer(timer);
+}
+
 /**
  * Insert a tasklet in the list of tasklets to be processed.
  *
@@ -181,9 +236,9 @@ static struct rt_fun_entry rt_tasklet_fun[] = {
  * @return a negative number to indicate that an invalid handler address has
  * been passed.
  *
- * @note To be used only with RTAI24.x.xx.
  */
-int rt_insert_tasklet(struct rt_tasklet_struct *tasklet, int priority, void (*handler)(unsigned long), unsigned long data, unsigned long id, int pid)
+
+RTAI_SYSCALL_MODE int rt_insert_tasklet(struct rt_tasklet_struct *tasklet, int priority, void (*handler)(unsigned long), unsigned long data, unsigned long id, int pid)
 {
 	unsigned long flags;
 
@@ -200,7 +255,7 @@ int rt_insert_tasklet(struct rt_tasklet_struct *tasklet, int priority, void (*ha
 		tasklet->task = 0;
 	} else {
 		(tasklet->task)->priority = priority;
-		copy_to_user(tasklet->usptasklet, tasklet, sizeof(struct rt_tasklet_struct));
+		rt_copy_to_user(tasklet->usptasklet, tasklet, sizeof(struct rt_tasklet_struct));
 	}
 // tasklet insertion tasklets_list
 	flags = rt_spin_lock_irqsave(&tasklets_lock);
@@ -220,9 +275,9 @@ int rt_insert_tasklet(struct rt_tasklet_struct *tasklet, int priority, void (*ha
  * @param tasklet is the pointer to the tasklet structure to be used to manage
  * the tasklet at hand.
  *
- * @note To be used only with RTAI24.x.xx.
  */
-void rt_remove_tasklet(struct rt_tasklet_struct *tasklet)
+
+RTAI_SYSCALL_MODE void rt_remove_tasklet(struct rt_tasklet_struct *tasklet)
 {
 	if (tasklet->next != tasklet && tasklet->prev != tasklet) {
 		unsigned long flags;
@@ -246,8 +301,8 @@ void rt_remove_tasklet(struct rt_tasklet_struct *tasklet)
  * @retval 0 to indicate that @a id is not a valid identifier so that the
  * related tasklet was not found.
  *
- * @note To be used only with RTAI24.x.xx.
  */
+
 struct rt_tasklet_struct *rt_find_tasklet_by_id(unsigned long id)
 {
 	struct rt_tasklet_struct *tasklet;
@@ -279,9 +334,9 @@ struct rt_tasklet_struct *rt_find_tasklet_by_id(unsigned long id)
  * calling rt_find_tasklet_by_id() to get the tasklet address to be used
  * in rt_tasklet_exec().
  *
- * @note To be used only with RTAI24.x.xx.
  */
-int rt_exec_tasklet(struct rt_tasklet_struct *tasklet)
+
+RTAI_SYSCALL_MODE int rt_exec_tasklet(struct rt_tasklet_struct *tasklet)
 {
 	if (tasklet && tasklet->next != tasklet && tasklet->prev != tasklet) {
 		if (!tasklet->task) {
@@ -294,7 +349,7 @@ int rt_exec_tasklet(struct rt_tasklet_struct *tasklet)
 	return -EINVAL;
 }
 
-void rt_set_tasklet_priority(struct rt_tasklet_struct *tasklet, int priority)
+RTAI_SYSCALL_MODE void rt_set_tasklet_priority(struct rt_tasklet_struct *tasklet, int priority)
 {
 	tasklet->priority = priority;
 	if (tasklet->task) {
@@ -302,55 +357,59 @@ void rt_set_tasklet_priority(struct rt_tasklet_struct *tasklet, int priority)
 	}
 }
 
-int rt_set_tasklet_handler(struct rt_tasklet_struct *tasklet, void (*handler)(unsigned long))
+RTAI_SYSCALL_MODE int rt_set_tasklet_handler(struct rt_tasklet_struct *tasklet, void (*handler)(unsigned long))
 {
 	if (!handler) {
 		return -EINVAL;
 	}
 	tasklet->handler = handler;
 	if (tasklet->task) {
-		copy_to_user(tasklet->usptasklet, tasklet, sizeof(struct rt_tasklet_struct));
+		rt_copy_to_user(tasklet->usptasklet, tasklet, sizeof(struct rt_tasklet_struct));
 	}
 	return 0;
 }
 
-void rt_set_tasklet_data(struct rt_tasklet_struct *tasklet, unsigned long data)
+RTAI_SYSCALL_MODE void rt_set_tasklet_data(struct rt_tasklet_struct *tasklet, unsigned long data)
 {
 	tasklet->data = data;
 	if (tasklet->task) {
-		copy_to_user(tasklet->usptasklet, tasklet, sizeof(struct rt_tasklet_struct));
+		rt_copy_to_user(tasklet->usptasklet, tasklet, sizeof(struct rt_tasklet_struct));
 	}
 }
 
-RT_TASK *rt_tasklet_use_fpu(struct rt_tasklet_struct *tasklet, int use_fpu)
+RTAI_SYSCALL_MODE RT_TASK *rt_tasklet_use_fpu(struct rt_tasklet_struct *tasklet, int use_fpu)
 {
 	tasklet->uses_fpu = use_fpu ? 1 : 0;
 	return tasklet->task;
 }
 
-static RT_TASK timers_manager;
+static RT_TASK timers_manager[NUM_CPUS];
 
-static inline void asgn_min_prio(void)
+static inline void asgn_min_prio(int cpuid)
 {
 // find minimum priority in timers_struct 
-	struct rt_tasklet_struct *timer;
+	RT_TASK *timer_manager;
+	struct rt_tasklet_struct *timer, *timerl;
+	spinlock_t *lock;
 	unsigned long flags;
 	int priority;
 
-	priority = (timer = timers_list.next)->priority;
-	flags = rt_spin_lock_irqsave(&timers_lock);
-	while ((timer = timer->next) != &timers_list) {
+	priority = (timer = (timerl = &timers_list[LIST_CPUID])->next)->priority;
+	flags = rt_spin_lock_irqsave(lock = &timers_lock[LIST_CPUID]);
+	while ((timer = timer->next) != timerl) {
 		if (timer->priority < priority) {
 			priority = timer->priority;
 		}
+		rt_spin_unlock_irqrestore(flags, lock);
+		flags = rt_spin_lock_irqsave(lock);
 	}
-	rt_spin_unlock_irqrestore(flags, &timers_lock);
+	rt_spin_unlock_irqrestore(flags, lock);
 	flags = rt_global_save_flags_and_cli();
-	if (timers_manager.priority > priority) {
-		timers_manager.priority = priority;
-		if (timers_manager.state == RT_SCHED_READY) {
-			rem_ready_task(&timers_manager);
-			enq_ready_task(&timers_manager);
+	if ((timer_manager = &timers_manager[LIST_CPUID])->priority > priority) {
+		timer_manager->priority = priority;
+		if (timer_manager->state == RT_SCHED_READY) {
+			rem_ready_task(timer_manager);
+			enq_ready_task(timer_manager);
 		}
 	}
 	rt_global_restore_flags(flags);
@@ -359,20 +418,14 @@ static inline void asgn_min_prio(void)
 static inline void set_timer_firing_time(struct rt_tasklet_struct *timer, RTIME firing_time)
 {
 	if (timer->next != timer && timer->prev != timer) {
+		spinlock_t *lock;
 		unsigned long flags;
-		struct rt_tasklet_struct *tmr;
 
-		tmr = &timers_list;
 		timer->firing_time = firing_time;
-		flags = rt_spin_lock_irqsave(&timers_lock);
-		(timer->next)->prev = timer->prev;
-		(timer->prev)->next = timer->next;
-		while (firing_time >= (tmr = tmr->next)->firing_time);
-		timer->next     = tmr;
-		timer->prev     = tmr->prev;
-		(tmr->prev)->next = timer;
-		tmr->prev         = timer;
-		rt_spin_unlock_irqrestore(flags, &timers_lock);
+		flags = rt_spin_lock_irqsave(lock = &timers_lock[TIMER_CPUID]);
+		rem_timer(timer);
+		enq_timer(timer);
+		rt_spin_unlock_irqrestore(flags, lock);
 	}
 }
 
@@ -411,48 +464,57 @@ static inline void set_timer_firing_time(struct rt_tasklet_struct *timer, RTIME
  * @retval 0 on success
  * @retval EINVAL if @a handler is an invalid handler address
  *
- * @note To be used only with RTAI24.x.xx.
  */
-int rt_insert_timer(struct rt_tasklet_struct *timer, int priority, RTIME firing_time, RTIME period, void (*handler)(unsigned long), unsigned long data, int pid)
+
+RTAI_SYSCALL_MODE int rt_insert_timer(struct rt_tasklet_struct *timer, int priority, RTIME firing_time, RTIME period, void (*handler)(unsigned long), unsigned long data, int pid)
 {
-	unsigned long flags;
-	struct rt_tasklet_struct *tmr;
+	spinlock_t *lock;
+	unsigned long flags, cpuid;
+	RT_TASK *timer_manager;
 
 // timer initialization
-	if (!handler) {
-		return -EINVAL;
-	}
 	timer->uses_fpu    = 0;
-	timer->priority    = priority;
+	
+	if (pid >= 0) {
+		if (!handler) {
+			return -EINVAL;
+		}
+		timer->handler   = handler;	
+		timer->data 			 = data;
+	} else {
+		if (timer->handler != NULL || timer->handler == (void *)1) {
+			timer->handler = (void *)1;	
+			timer->data    = data;
+		}		
+	}
+	
+	timer->priority    = priority;	
+	REALTIME2COUNT(firing_time)
 	timer->firing_time = firing_time;
 	timer->period      = period;
-	timer->handler     = handler;
-	timer->data        = data;
+	
 	if (!pid) {
 		timer->task = 0;
+		timer->cpuid = cpuid = NUM_CPUS > 1 ? rtai_cpuid() : 0;
 	} else {
+		timer->cpuid = cpuid = NUM_CPUS > 1 ? (timer->task)->runnable_on_cpus : 0;
 		(timer->task)->priority = priority;
-		copy_to_user(timer->usptasklet, timer, sizeof(struct rt_tasklet_struct));
+		rt_copy_to_user(timer->usptasklet, timer, sizeof(struct rt_tasklet_struct));
 	}
 // timer insertion in timers_list
-	tmr = &timers_list;
-	flags = rt_spin_lock_irqsave(&timers_lock);
-	while (firing_time >= (tmr = tmr->next)->firing_time);
-	timer->next     = tmr;
-	timer->prev     = tmr->prev;
-	(tmr->prev)->next = timer;
-	tmr->prev         = timer;
-	rt_spin_unlock_irqrestore(flags, &timers_lock);
+	flags = rt_spin_lock_irqsave(lock = &timers_lock[LIST_CPUID]);
+	enq_timer(timer);
+	rt_spin_unlock_irqrestore(flags, lock);
 // timers_manager priority inheritance
-	if (timer->priority < timers_manager.priority) {
-		timers_manager.priority = timer->priority;
+	if (timer->priority < (timer_manager = &timers_manager[LIST_CPUID])->priority) {
+		timer_manager->priority = timer->priority;
 	}
 // timers_task deadline inheritance
 	flags = rt_global_save_flags_and_cli();
-	if (timers_list.next == timer && (timers_manager.state & RT_SCHED_DELAYED) && firing_time < timers_manager.resume_time) {
-		timers_manager.resume_time = firing_time;
-		rem_timed_task(&timers_manager);
-		enq_timed_task(&timers_manager);
+	if (timers_list[LIST_CPUID].next == timer && (timer_manager->state & RT_SCHED_DELAYED) && firing_time < timer_manager->resume_time) {
+		timer_manager->resume_time = firing_time;
+		rem_timed_task(timer_manager);
+		enq_timed_task(timer_manager);
 		rt_schedule();
 	}
 	rt_global_restore_flags(flags);
@@ -467,18 +529,17 @@ int rt_insert_timer(struct rt_tasklet_struct *timer, int priority, RTIME firing_
  * @param timer is the pointer to the timer structure to be used to manage the
  * timer at hand.
  *
- * @note To be used only with RTAI24.x.xx.
  */
-void rt_remove_timer(struct rt_tasklet_struct *timer)
+
+RTAI_SYSCALL_MODE void rt_remove_timer(struct rt_tasklet_struct *timer)
 {
 	if (timer->next != timer && timer->prev != timer) {
+		spinlock_t *lock;
 		unsigned long flags;
-		flags = rt_spin_lock_irqsave(&timers_lock);
-		(timer->next)->prev = timer->prev;
-		(timer->prev)->next = timer->next;
-		timer->next = timer->prev = timer;
-		rt_spin_unlock_irqrestore(flags, &timers_lock);
-		asgn_min_prio();
+		flags = rt_spin_lock_irqsave(lock = &timers_lock[TIMER_CPUID]);
+		rem_timer(timer);
+		rt_spin_unlock_irqrestore(flags, lock);
+		asgn_min_prio(TIMER_CPUID);
 	}
 }
 
@@ -496,15 +557,15 @@ void rt_remove_timer(struct rt_tasklet_struct *timer)
  *
  * This function can be used within the timer handler.
  *
- * @note To be used only with RTAI24.x.xx.
  */
-void rt_set_timer_priority(struct rt_tasklet_struct *timer, int priority)
+
+RTAI_SYSCALL_MODE void rt_set_timer_priority(struct rt_tasklet_struct *timer, int priority)
 {
 	timer->priority = priority;
 	if (timer->task) {
 		(timer->task)->priority = priority;
 	}
-	asgn_min_prio();
+	asgn_min_prio(TIMER_CPUID);
 }
 
 /**
@@ -525,18 +586,19 @@ void rt_set_timer_priority(struct rt_tasklet_struct *timer, int priority)
  *
  * @retval 0 on success.
  *
- * @note To be used only with RTAI24.x.xx.
  */
-void rt_set_timer_firing_time(struct rt_tasklet_struct *timer, RTIME firing_time)
+
+RTAI_SYSCALL_MODE void rt_set_timer_firing_time(struct rt_tasklet_struct *timer, RTIME firing_time)
 {
 	unsigned long flags;
+	RT_TASK *timer_manager;
 
 	set_timer_firing_time(timer, firing_time);
 	flags = rt_global_save_flags_and_cli();
-	if (timers_list.next == timer && (timers_manager.state & RT_SCHED_DELAYED) && firing_time < timers_manager.resume_time) {
-		timers_manager.resume_time = firing_time;
-		rem_timed_task(&timers_manager);
-		enq_timed_task(&timers_manager);
+	if (timers_list[TIMER_CPUID].next == timer && ((timer_manager = &timers_manager[TIMER_CPUID])->state & RT_SCHED_DELAYED) && firing_time < timer_manager->resume_time) {
+		timer_manager->resume_time = firing_time;
+		rem_timed_task(timer_manager);
+		enq_timed_task(timer_manager);
 		rt_schedule();
 	}
 	rt_global_restore_flags(flags);
@@ -564,70 +626,115 @@ void rt_set_timer_firing_time(struct rt_tasklet_struct *timer, RTIME firing_time
  *
  * @retval 0 on success.
  *
- * @note To be used only with RTAI24.x.xx.
  */
-void rt_set_timer_period(struct rt_tasklet_struct *timer, RTIME period)
+
+RTAI_SYSCALL_MODE void rt_set_timer_period(struct rt_tasklet_struct *timer, RTIME period)
 {
+	spinlock_t *lock;
 	unsigned long flags;
-	flags = rt_spin_lock_irqsave(&timers_lock);
+	flags = rt_spin_lock_irqsave(lock = &timers_lock[TIMER_CPUID]);
 	timer->period = period;
-	rt_spin_unlock_irqrestore(flags, &timers_lock);
+	rt_spin_unlock_irqrestore(flags, lock);
 }
 
+RTAI_SYSCALL_MODE void rt_get_timer_times(struct rt_tasklet_struct *timer, RTIME timer_times[])
+{
+	RTIME firing;
+	
+	firing = -rt_get_time();
+	firing += timer->firing_time;
+		
+	timer_times[0] = firing > 0 ? firing : -1;
+	timer_times[1] = timer->period;
+}
+
+RTAI_SYSCALL_MODE RTIME rt_get_timer_overrun(struct rt_tasklet_struct *timer)
+{
+	return timer->overrun;
+}
+
+static int TimersManagerPrio = 0;
+RTAI_MODULE_PARM(TimersManagerPrio, int);
+
 // the timers_manager task function
 
-static void rt_timers_manager(int dummy)
+static void rt_timers_manager(long cpuid)
 {
 	RTIME now;
-	struct rt_tasklet_struct *tmr, *timer;
-	unsigned long flags;
+	RT_TASK *timer_manager;
+	struct rt_tasklet_struct *tmr, *timer, *timerl;
+	spinlock_t *lock;
+	unsigned long flags, timer_tol;
 	int priority, used_fpu;
 
+	timer_manager = &timers_manager[LIST_CPUID];
+	timerl = &timers_list[LIST_CPUID];
+	lock = &timers_lock[LIST_CPUID];
+	timer_tol = tuned.timers_tol[LIST_CPUID];
+
 	while (1) {
-		rt_sleep_until((timers_list.next)->firing_time);
-		now = timers_manager.resume_time + tuned.timers_tol[0];
+		int retval;
+		retval = rt_sleep_until((timerl->next)->firing_time);
+//		now = timer_manager->resume_time + timer_tol;
+		now = rt_get_time() + timer_tol;
 // find all the timers to be fired, in priority order
 		while (1) {
 			used_fpu = 0;
-			tmr = timer = &timers_list;
+			tmr = timer = timerl;
 			priority = RT_SCHED_LOWEST_PRIORITY;
-			flags = rt_spin_lock_irqsave(&timers_lock);
+			flags = rt_spin_lock_irqsave(lock);
 			while ((tmr = tmr->next)->firing_time <= now) {
 				if (tmr->priority < priority) {
 					priority = (timer = tmr)->priority;
 				}
 			}
-			timers_manager.priority = priority;
-			rt_spin_unlock_irqrestore(flags, &timers_lock);
-			if (timer == &timers_list) {
+			rt_spin_unlock_irqrestore(flags, lock);
+			if (timer == timerl) {
+				if (timer_manager->priority > TimersManagerPrio) {
+					timer_manager->priority = TimersManagerPrio;
+				}
 				break;
 			}
+			timer_manager->priority = priority;
+#if 1
+			flags = rt_spin_lock_irqsave(lock);
+			rem_timer(timer);
+			if (timer->period) {
+				timer->firing_time += timer->period;
+				enq_timer(timer);
+			}
+			rt_spin_unlock_irqrestore(flags, lock);
+#else
 			if (!timer->period) {
-				flags = rt_spin_lock_irqsave(&timers_lock);
-				(timer->next)->prev = timer->prev;
-				(timer->prev)->next = timer->next;
-				timer->next = timer->prev = timer;
-				rt_spin_unlock_irqrestore(flags, &timers_lock);
+				flags = rt_spin_lock_irqsave(lock);
+				rem_timer(timer);
+				rt_spin_unlock_irqrestore(flags, lock);
 			} else {
 				set_timer_firing_time(timer, timer->firing_time + timer->period);
 			}
+#endif
+	//	if (retval != RTE_TMROVRN) {
+			tmr->overrun = 0;
 			if (!timer->task) {
 				if (!used_fpu && timer->uses_fpu) {
 					used_fpu = 1;
-					save_cr0_and_clts(linux_cr0);
-					save_fpenv(timers_manager.fpu_reg);
+					save_fpcr_and_enable_fpu(linux_cr0);
+					save_fpenv(timer_manager->fpu_reg);
 				}
 				timer->handler(timer->data);
 			} else {
 				rt_task_resume(timer->task);
 			}
+	//	} else {
+	//		tmr->overrun++;
+	//	}
 		}
 		if (used_fpu) {
-			restore_fpenv(timers_manager.fpu_reg);
-			restore_cr0(linux_cr0);
+			restore_fpenv(timer_manager->fpu_reg);
+			restore_fpcr(linux_cr0);
 		}
 // set next timers_manager priority according to the highest priority timer
-		asgn_min_prio();
+		asgn_min_prio(LIST_CPUID);
 // if no more timers in timers_struct remove timers_manager from tasks list
 	}
 }
@@ -648,19 +755,19 @@ static void rt_timers_manager(int dummy)
 struct rt_tasklet_struct *rt_init_tasklet(void)
 {
 	struct rt_tasklet_struct *tasklet;
-	tasklet = sched_malloc(sizeof(struct rt_tasklet_struct));
+	tasklet = rt_malloc(sizeof(struct rt_tasklet_struct));
 	memset(tasklet, 0, sizeof(struct rt_tasklet_struct));
 	return tasklet;
 }
 
-void rt_register_task(struct rt_tasklet_struct *tasklet, struct rt_tasklet_struct *usptasklet, RT_TASK *task)
+RTAI_SYSCALL_MODE void rt_register_task(struct rt_tasklet_struct *tasklet, struct rt_tasklet_struct *usptasklet, RT_TASK *task)
 {
 	tasklet->task = task;
 	tasklet->usptasklet = usptasklet;
-	copy_to_user(usptasklet, tasklet, sizeof(struct rt_tasklet_struct));
+	rt_copy_to_user(usptasklet, tasklet, sizeof(struct rt_tasklet_struct));
 }
 
-void rt_wait_tasklet_is_hard(struct rt_tasklet_struct *tasklet, int thread)
+RTAI_SYSCALL_MODE void rt_wait_tasklet_is_hard(struct rt_tasklet_struct *tasklet, long thread)
 {
 	tasklet->thread = thread;
 	while (!tasklet->task || !((tasklet->task)->state & RT_SCHED_SUSPENDED)) {
@@ -682,51 +789,198 @@ void rt_wait_tasklet_is_hard(struct rt_tasklet_struct *tasklet, int thread)
  * it is just an empty macro, as the user can, and must  allocate the related
  * structure directly, either statically or dynamically.
  *
- * @note To be used only with RTAI24.x.xx.
  */
-int rt_delete_tasklet(struct rt_tasklet_struct *tasklet)
+
+RTAI_SYSCALL_MODE int rt_delete_tasklet(struct rt_tasklet_struct *tasklet)
 {
 	int thread;
 
 	rt_remove_tasklet(tasklet);
 	tasklet->handler = 0;
-	copy_to_user(tasklet->usptasklet, tasklet, sizeof(struct rt_tasklet_struct));
+	rt_copy_to_user(tasklet->usptasklet, tasklet, sizeof(struct rt_tasklet_struct));
 	rt_task_resume(tasklet->task);
 	thread = tasklet->thread;	
-	sched_free(tasklet);
+	rt_free(tasklet);
 	return thread;	
 }
 
-static int tasklets_stacksize = TASKLET_STACK_SIZE;
-MODULE_PARM(tasklets_stacksize, "i");
+/*
+ * Posix Timers support function
+ */
+ 
+ 
+static int PosixTimers = POSIX_TIMERS;
+RTAI_MODULE_PARM(PosixTimers, int);
 
-static RT_TASK *rt_base_linux_task;
+static spinlock_t ptimer_lock = SPIN_LOCK_UNLOCKED;
+static volatile int ptimer_index;
+struct ptimer_list { int t_indx, p_idx; struct ptimer_list *p_ptr; struct rt_tasklet_struct *timer;} *posix_timer;
+
+static int init_ptimers(void)
+{
+	int i;
+	
+	if (!(posix_timer = (struct ptimer_list *)kmalloc((PosixTimers)*sizeof(struct ptimer_list), GFP_KERNEL))) {
+		printk("Init MODULE no memory for Posix Timer's list.\n");
+		return -ENOMEM;
+	}
+	for (i = 0; i < PosixTimers; i++) {
+		posix_timer[i].t_indx = posix_timer[i].p_idx = i;
+		posix_timer[i].p_ptr = posix_timer + i;
+	}
+	return 0;
+
+}
+
+static void cleanup_ptimers(void)
+{
+	kfree(posix_timer);
+} 
+ 
+static inline int get_ptimer_indx(struct rt_tasklet_struct *timer)
+{
+	unsigned long flags;
+
+	flags = rt_spin_lock_irqsave(&ptimer_lock);
+	if (ptimer_index < PosixTimers) {
+		struct ptimer_list *p;
+		p = posix_timer[ptimer_index++].p_ptr;
+		p->timer = timer;
+		rt_spin_unlock_irqrestore(flags, &ptimer_lock);
+		return p->t_indx;
+	}
+	rt_spin_unlock_irqrestore(flags, &ptimer_lock);
+	return 0;
+}
+
+static inline int gvb_ptimer_indx(int itimer)
+{
+	unsigned long flags;
+
+	flags = rt_spin_lock_irqsave(&ptimer_lock);
+	if (itimer < PosixTimers) {
+		struct ptimer_list *tmp_p;
+		int tmp_place;
+		tmp_p = posix_timer[--ptimer_index].p_ptr;
+		tmp_place = posix_timer[itimer].p_idx;
+		posix_timer[itimer].p_idx = ptimer_index;
+		posix_timer[ptimer_index].p_ptr = &posix_timer[itimer];
+		tmp_p->p_idx = tmp_place;
+		posix_timer[tmp_place].p_ptr = tmp_p;
+		rt_spin_unlock_irqrestore(flags, &ptimer_lock);
+		return 0;
+	}
+	rt_spin_unlock_irqrestore(flags, &ptimer_lock);
+	return -EINVAL;
+}
+
+RTAI_SYSCALL_MODE timer_t rt_ptimer_create(struct rt_tasklet_struct *timer, void (*handler)(unsigned long), unsigned long data, long pid, long thread)
+{
+	if (thread) {
+		rt_wait_tasklet_is_hard(timer, thread);
+	}
+	timer->next = timer;
+	timer->prev = timer;
+	timer->data = data;
+	timer->handler = handler;
+	return get_ptimer_indx(timer);
+}
+EXPORT_SYMBOL(rt_ptimer_create);
+
+RTAI_SYSCALL_MODE void rt_ptimer_settime(timer_t timer, const struct itimerspec *value, unsigned long data, long flags)
+{
+	struct rt_tasklet_struct *tasklet;
+	RTIME now;
+	
+	tasklet = posix_timer[timer].timer;
+	rt_remove_timer(tasklet);
+	now = rt_get_time();
+	if (flags == TIMER_ABSTIME)	{
+		if (timespec2count(&(value->it_value)) < now) {
+			now -= timespec2count (&(value->it_value));
+		}else {
+			now = 0;
+		}
+	}	
+	if (timespec2count ( &(value->it_value)) > 0) {
+		if (data) {
+			rt_insert_timer(tasklet, 0, now + timespec2count ( &(value->it_value) ), timespec2count ( &(value->it_interval) ), NULL, data, -1);
+		} else {
+			rt_insert_timer(tasklet, 0, now + timespec2count ( &(value->it_value) ), timespec2count ( &(value->it_interval) ), tasklet->handler, tasklet->data, 0);
+		}
+	}
+}
+EXPORT_SYMBOL(rt_ptimer_settime);
+
+RTAI_SYSCALL_MODE int rt_ptimer_overrun(timer_t timer)
+{
+	return rt_get_timer_overrun(posix_timer[timer].timer);
+}
+EXPORT_SYMBOL(rt_ptimer_overrun);
+
+RTAI_SYSCALL_MODE void rt_ptimer_gettime(timer_t timer, RTIME timer_times[])
+{
+	rt_get_timer_times(posix_timer[timer].timer, timer_times);
+}
+EXPORT_SYMBOL(rt_ptimer_gettime);
+
+RTAI_SYSCALL_MODE int rt_ptimer_delete(timer_t timer, long space)
+{
+	struct rt_tasklet_struct *tasklet;
+	int rtn = 0;
+	
+	tasklet = posix_timer[timer].timer;
+	gvb_ptimer_indx(timer);
+	rt_remove_tasklet(tasklet);	
+	if (space) {
+		tasklet->handler = 0;
+		rt_copy_to_user(tasklet->usptasklet, tasklet, sizeof(struct rt_tasklet_struct));
+		rt_task_resume(tasklet->task);
+		rtn = tasklet->thread;	
+	} 
+	rt_free(tasklet);
+	return rtn;
+}		
+EXPORT_SYMBOL(rt_ptimer_delete);
+
+ /*
+ * End Posix timers support function
+ */
+
+static int TaskletsStacksize = TASKLET_STACK_SIZE;
+RTAI_MODULE_PARM(TaskletsStacksize, int);
 
 int __rtai_tasklets_init(void)
 {
-	RT_TASK *rt_linux_tasks[NR_RT_CPUS];
-	rt_base_linux_task = rt_get_base_linux_task(rt_linux_tasks);
-	if (rt_sched_type() == RT_SCHED_MUP) {
-		tuned.timers_tol[0] = tuned.timers_tol[timers_manager.runnable_on_cpus];
-	}
-        if(rt_base_linux_task->task_trap_handler[0]) {
-                if(((int (*)(void *, int))rt_base_linux_task->task_trap_handler[0])(rt_tasklet_fun, TSKIDX)) {
-                        printk("Recompile your module with a different index\n");
-                        return -EACCES;
-                }
+	int cpuid;
+
+	if(set_rt_fun_ext_index(rt_tasklet_fun, TASKLETS_IDX)) {
+		printk("Recompile your module with a different index\n");
+		return -EACCES;
         }
-	rt_task_init(&timers_manager, rt_timers_manager, 0, tasklets_stacksize, RT_SCHED_LOWEST_PRIORITY, 0, 0);
-	rt_task_resume(&timers_manager);
+	if (init_ptimers()) {
+		return -ENOMEM;
+	}	
+	for (cpuid = 0; cpuid < NUM_CPUS; cpuid++) {
+		timers_lock[cpuid] = timers_lock[0];
+		timers_list[cpuid] = timers_list[0];
+		timers_list[cpuid].cpuid = cpuid;
+		timers_list[cpuid].next = timers_list[cpuid].prev = &timers_list[cpuid];
+		rt_task_init_cpuid(&timers_manager[cpuid], rt_timers_manager, cpuid, TaskletsStacksize, TimersManagerPrio, 0, 0, cpuid);
+		rt_task_resume(&timers_manager[cpuid]);
+	}
 	printk(KERN_INFO "RTAI[tasklets]: loaded.\n");
 	return 0;
 }
 
 void __rtai_tasklets_exit(void)
 {
-	rt_task_delete(&timers_manager);
-        if(rt_base_linux_task->task_trap_handler[1]) {
-                ((int (*)(void *, int))rt_base_linux_task->task_trap_handler[1])(rt_tasklet_fun, TSKIDX);
-        }
+	int cpuid;
+ 	reset_rt_fun_ext_index(rt_tasklet_fun, TASKLETS_IDX);
+	cleanup_ptimers();    
+	for (cpuid = 0; cpuid < NUM_CPUS; cpuid++) {
+		rt_task_delete(&timers_manager[cpuid]);
+	}
 	printk(KERN_INFO "RTAI[tasklets]: unloaded.\n");
 }
 
@@ -755,4 +1009,6 @@ EXPORT_SYMBOL(rt_init_tasklet);
 EXPORT_SYMBOL(rt_register_task);
 EXPORT_SYMBOL(rt_wait_tasklet_is_hard);
 EXPORT_SYMBOL(rt_delete_tasklet);
+EXPORT_SYMBOL(rt_get_timer_times);
+EXPORT_SYMBOL(rt_get_timer_overrun);
 #endif /* CONFIG_KBUILD */
