@@ -89,7 +89,7 @@
  * 5. Keeps a record of bad tasks (apart from those that have been killed) that 
  *    can be examined via a /proc interface. (/proc/rtai/watchdog)
  * 
- * ID: @(#)$Id: wd.c,v 1.3 2005/03/18 09:29:59 rpm Exp $
+ * ID: @(#)$Id: wd.c,v 1.9 2007/12/18 15:01:51 mante Exp $
  *
  *******************************************************************************/
 
@@ -98,6 +98,8 @@
 #include <linux/version.h>
 #include <asm/io.h>
 
+MODULE_LICENSE("GPL");
+
 #ifdef CONFIG_PROC_FS
 #include <linux/stat.h>
 #include <linux/proc_fs.h>
@@ -137,7 +139,7 @@ static BAD_RT_TASK bad_task_pool[BAD_TASK_MAX];
 #endif
 
 // The current version number
-static char version[] = "$Revision: 1.3 $";
+static char version[] = "$Revision: 1.9 $";
 static char ver[10];
 
 // User friendly policy names
@@ -155,34 +157,34 @@ static int          sched;		// Scheduler type (UP, SMP or MUP)
 // -------------------------- CONFIGURABLE PARAMETERS --------------------------
 // Module parameters
 static int TickPeriod = 10000000;	// Task period in nano seconds
-MODULE_PARM(TickPeriod, "i");		// (should be shorter than all others)
+RTAI_MODULE_PARM(TickPeriod, int);	// (should be shorter than all others)
 
-static int wd_OneShot = 1;			// One shot timer mode or not (periodic)
-MODULE_PARM(wd_OneShot, "i");		// (should be the same as other tasks)
+static int wd_OneShot = 1;		// One shot timer mode or not (periodic)
+RTAI_MODULE_PARM(wd_OneShot, int);	// (should be the same as other tasks)
 
 static int Grace = 3;			// How much a task can be overdue
-MODULE_PARM(Grace, "i");		// (in periods, always 1 in some modes)
+RTAI_MODULE_PARM(Grace, int);		// (in periods, always 1 in some modes)
 
 static int GraceDiv = 1;		// Divisor to allow Gracevalues < 1
-MODULE_PARM(GraceDiv, "i");		// overrun = period * Grace / Gracediv
+RTAI_MODULE_PARM(GraceDiv, int);	// overrun = period * Grace / Gracediv
 
 static int Safety = 100;		// Safety net to suspend infinite loops
-MODULE_PARM(Safety, "i");		// (overrides policy, -ve disables)
+RTAI_MODULE_PARM(Safety, int);		// (overrides policy, -ve disables)
 
-static wd_policy Policy = WD_SUSPEND;	// How to punish misbehavers
-MODULE_PARM(Policy, "i");		// (see above and header for details)
+static int Policy = WD_SUSPEND;		// How to punish misbehavers
+RTAI_MODULE_PARM(Policy, int);		// (see above and header for details)
 
 static int Stretch = 10;		// %ge to increase period by
-MODULE_PARM(Stretch, "i");		// (can be over 100%, 100% is doubling)
+RTAI_MODULE_PARM(Stretch, int);		// (can be over 100%, 100% is doubling)
 
 static int Slip = 10;			// %ge of period to slip a task
-MODULE_PARM(Slip, "i");			// (can be over 100%)
+RTAI_MODULE_PARM(Slip, int);			// (can be over 100%)
 
 static int Limit = 100;			// Maximum number of offences
-MODULE_PARM(Limit, "i");		// (-ve means disabled ie. no limit)
+RTAI_MODULE_PARM(Limit, int);		// (-ve means disabled ie. no limit)
 
 // Parameter configuring API
-int rt_wdset_grace(int new)		// How much a task can be overdue
+RTAI_SYSCALL_MODE int rt_wdset_grace(int new) // How much a task can be overdue
 {
     int old = Grace;
 
@@ -192,7 +194,7 @@ int rt_wdset_grace(int new)		// How much a task can be overdue
     return old;
 }
 
-int rt_wdset_gracediv(int new)		// Divisor for Gracevalues < 1
+RTAI_SYSCALL_MODE int rt_wdset_gracediv(int new) // Divisor for Gracevalues < 1
 {
     int old = GraceDiv;
 
@@ -202,7 +204,7 @@ int rt_wdset_gracediv(int new)		// Divisor for Gracevalues < 1
     return old;
 }
 
-int rt_wdset_safety(int new)		// Safety net to suspend infinite loops
+RTAI_SYSCALL_MODE int rt_wdset_safety(int new) // Safety net to suspend infinite loops
 {
     int old = Safety;
 
@@ -211,7 +213,7 @@ int rt_wdset_safety(int new)		// Safety net to suspend infinite loops
     return old;
 }
 
-wd_policy rt_wdset_policy(wd_policy new) // How to punish misbehavers
+RTAI_SYSCALL_MODE wd_policy rt_wdset_policy(wd_policy new) // How to punish misbehavers
 {
     wd_policy old = Policy;
 
@@ -222,7 +224,7 @@ wd_policy rt_wdset_policy(wd_policy new) // How to punish misbehavers
     return old;
 }
 
-int rt_wdset_slip(int new)		// %ge of period to slip a task
+RTAI_SYSCALL_MODE int rt_wdset_slip(int new)  // %ge of period to slip a task
 {
     int old = Slip;
 
@@ -231,7 +233,7 @@ int rt_wdset_slip(int new)		// %ge of period to slip a task
     return old;
 }
 
-int rt_wdset_stretch(int new)		// %ge to increase period by
+RTAI_SYSCALL_MODE int rt_wdset_stretch(int new)	 // %ge to increase period by
 {
     int old = Stretch;
 
@@ -240,7 +242,7 @@ int rt_wdset_stretch(int new)		// %ge to increase period by
     return old;
 }
 
-int rt_wdset_limit(int new)		// Maximum number of offences
+RTAI_SYSCALL_MODE int rt_wdset_limit(int new)  // Maximum number of offences
 {
     int old = Limit;
 
@@ -471,7 +473,7 @@ static void handle_badtask(int wd, RT_TASK *t, BAD_RT_TASK *bt, RTIME overrun)
 }
 
 // -------------------------- THE MAIN WATCHDOG TASK ---------------------------
-static void watchdog(int wd)
+static void watchdog(long wd)
 {
 #ifdef WDBUG
     int 	 led    = 0;
@@ -557,7 +559,7 @@ static void watchdog(int wd)
 }
 
 // -------------------------- DUMMY WATCHDOG TASK ------------------------------
-static void dummy(int wd)
+static void dummy(long wd)
 {
     // Go straight back to sleep - see SMP proof suspend and delete
     while (1) {
@@ -583,7 +585,7 @@ static int wdog_read_proc(char *page, char **start, off_t off, int count,
     PROC_PRINT(  "--------------------\n");
     PROC_PRINT("%d Watchdog task%s running @ %dHz in %s mode\n", 
 	       num_wdogs, num_wdogs > 1 ? "s" : "",
-	       imuldiv(NSECS_PER_SEC, 1, TickPeriod), 
+	       (int)imuldiv(NSECS_PER_SEC, 1, TickPeriod), 
 	       wd_OneShot ? "oneshot" : "periodic");
 #ifdef MY_ALLOC
     PROC_PRINT("Using static memory management (%d entries)\n", BAD_TASK_MAX);
@@ -640,12 +642,12 @@ static int wdog_read_proc(char *page, char **start, off_t off, int count,
 		asec  = ulldiv( count2nano_cpuid(task->period, cpuid),
 			        NSECS_PER_SEC, 
 			        &ansec);
-		PROC_PRINT( "0x%08x %-2d "
+		PROC_PRINT( "0x%08lx %-2d "
 			    "%s%-2d%s "
 			    "%-8d 0x%-3x %-5d "
 			    "%02ds %09dns %02ds %09dns "
 			    "%s\n",
-			    (int)task, id, 
+			    (long)task, id, 
 			    (sched == RT_SCHED_SMP) ?  "0x" : "",
 			    (sched == RT_SCHED_UP)  ?
 			        0 : (int)task->runnable_on_cpus,
@@ -661,6 +663,22 @@ static int wdog_read_proc(char *page, char **start, off_t off, int count,
 }
 #endif
 
+// ------------------------ WATCHDOG ENTRIES TABLE -----------------------------
+
+#include <rtai_lxrt.h>
+
+static struct rt_fun_entry rt_watchdog_fun[]  __attribute__ ((__unused__));
+
+static struct rt_fun_entry rt_watchdog_fun[] = {
+        [WD_SET_GRACE]	  = { 0, rt_wdset_grace },
+        [WD_SET_GRACEDIV] = { 0, rt_wdset_gracediv },
+        [WD_SET_SAFETY]   = { 0, rt_wdset_safety },
+        [WD_SET_POLICY]   = { 0, rt_wdset_policy },
+        [WD_SET_SLIP]     = { 0, rt_wdset_slip },
+        [WD_SET_STRETCH]  = { 0, rt_wdset_stretch },
+        [WD_SET_LIMIT]    = { 0, rt_wdset_limit }
+};
+
 // ----------------------------- MODULE CONTROL --------------------------------
 int __rtai_wd_init(void)
 {
@@ -670,6 +688,10 @@ int __rtai_wd_init(void)
     struct apic_timer_setup_data apic_data[NR_RT_CPUS];
     char	*c;
 
+    if(set_rt_fun_ext_index(rt_watchdog_fun, WD_INDX)) {
+	printk("Recompile your module with a different index\n");
+	return -EACCES;
+    }
     // Some parameters have to be forced
     if (Policy <= WD_STRETCH) Grace  = GraceDiv = 1;
     if (Policy == WD_DEBUG)   Safety = Limit = -1;
@@ -700,14 +722,14 @@ int __rtai_wd_init(void)
 	    apic_data[dog].mode  = !wd_OneShot;	 // <--- This bit...
 	    apic_data[dog].count = TickPeriod;
 	    if (wd_OneShot) {
-		rt_preempt_always_cpuid(1, dog); // <--- ...and this!
+//		rt_preempt_always_cpuid(1, dog); // <--- ...and this!
 	    }
 	}
 	start_rt_apic_timers(apic_data, 0);
     } else {
 	if (wd_OneShot) {
 	    rt_set_oneshot_mode();
-	    rt_preempt_always(1);
+//	    rt_preempt_always(1);
 	} else {
 	    rt_set_periodic_mode();
 	}
@@ -803,6 +825,8 @@ void __rtai_wd_exit(void)
 	}
     }
 
+    reset_rt_fun_ext_index(rt_watchdog_fun, WD_INDX);
+
     // It's all over :(
     WDLOG("unloaded.\n");
 }
